
#include <io.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <math.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>


#include "..\..\Include\define.h"
#include "..\..\Include\PCI3009.h"
#include "..\..\Utils\Utils.h"

//----------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------

struct fifo_t
{
    struct fifo_t *next;
    WORD *data;
};

//----------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------

struct fifo_t *dataFIFO = NULL;
DWORD valuesPerInterrupt = 0;
BYTE stop = 0;

//----------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------

int fifoPut(struct fifo_t **fifo, PWORD data)
{
    struct fifo_t *cell = (struct fifo_t *) malloc(sizeof(struct fifo_t));
    static struct fifo_t *last = NULL;

    if (cell == NULL)
        return -1;

    cell->next = NULL;
    cell->data = data;
    if (*fifo == NULL)
    {
        *fifo = cell;
        last = cell;
    }
    else
    {
        /* This is the standart way to find the last element
           To be faster the last element is saved as a static variable.
      
        struct fifo_t *tmp = *fifo;
        while (tmp->next != NULL)
            tmp = tmp->next;
        tmp->next = cell;
        */
        last->next = cell;
        last = cell;
    }

    return 0;
}

//----------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------

void gotoxy(HANDLE h_ConsoleHandle, int X, int Y)
{
    COORD s_CoordScreen = { 0, 0 };

    if (h_ConsoleHandle != NULL)
    {
        s_CoordScreen.X = X;
        s_CoordScreen.Y = Y;
        SetConsoleCursorPosition(h_ConsoleHandle, s_CoordScreen);
    }
}

//----------------------------------------------------------------------------------------

void clrscr(HANDLE h_ConsoleHandle)
{
    DWORD                      dw_ConSize;
    DWORD                      dw_CharsWritten;
    COORD                       s_CoordScreen = { 0, 0 };
    CONSOLE_SCREEN_BUFFER_INFO  csbi;

    if (h_ConsoleHandle != NULL)
    {
        GetConsoleScreenBufferInfo(h_ConsoleHandle, &csbi);
        dw_ConSize = csbi.dwSize.X * csbi.dwSize.Y;

        FillConsoleOutputCharacter(h_ConsoleHandle,
            (TCHAR) ' ',
            dw_ConSize,
            s_CoordScreen,
            &dw_CharsWritten);

        SetConsoleCursorPosition(h_ConsoleHandle, s_CoordScreen);

    }
}

//----------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------

VOID WINAPI v_InterruptRoutine(DWORD dw_DriverHandle,
                                DWORD dw_InterruptSrc,
                                DWORD dw_InterruptMask,
                                PDWORD pdw_Value,
                                DWORD dw_UserCallingMode,
                                VOID *pv_UserSharedMemory)
{
    /* FIFO overflow */
    if (dw_InterruptSrc == 0x5)
    {
        /* Driver FIFO overflow */
        if (dw_InterruptMask == 0x1)
        {
            stop = 1;
            goto end;
        }
    }

    /* Test if analog input interrupt */
    if (dw_InterruptSrc != 0)
        goto end;

    /* PLD FIFO overflow */
    if (dw_InterruptMask == 0x8)
    {
        stop = 2;
        goto end;
    }

    /* Test if sequence interrupt with DMA */
    if ((dw_InterruptMask & 0x4) != 0x4)
        goto end;

    {
        DWORD sizeToRead = valuesPerInterrupt * sizeof(WORD);
        LONG size = 0;
        PWORD data = (PWORD) malloc(sizeToRead);

        if (data == NULL)
        {
            stop = 3;
            goto end;
        }

        size = l_PCI3009_ReadSequenceData(dw_DriverHandle, sizeToRead, data);
        if (size != sizeToRead)
        {
            stop = 4;
            goto end;
        }

        if (fifoPut(&dataFIFO, data))
        {
            stop = 5;
            goto end;
        }
    }   

end:
    if (stop)
        i_PCI3009_AnalogInputStopSequence(dw_DriverHandle);

    return;
}

//----------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------

void main (void)
{
    BYTE b_InitIRQOK = 0;
    WORD w_Cpt = 0;
    BYTE b_SequenceSize = 0;
    BYTE b_ChannelSequence[16];
    INT i_ReturnValue = 0;
    DWORD dw_DriverHandle = 0;
    BYTE b_Polarity = 0;
    BYTE b_Gain = 0;
    BYTE b_SingleDiffMode = 0;
    WORD w_ConvertTime = 0;
    BYTE b_ConvertTimeUnit = 0;
    BYTE b_DelayMode = 0;
    BYTE b_DelayTimeUnit = 0;
    WORD w_DelayTime = 0;
    DWORD dw_NumberOfSequence = 0;
    DWORD dw_NumberOfSequenceForEachInterrupt = 0;
    BYTE b_HardwareTriggerAction = 0;
    BYTE b_HardwareTriggerLevel = 0;
    DWORD dw_HardwareTriggerCount = 0;
    DWORD dw_HardwareTriggerCycleCount = 0;
    INT i_Handle = 0;
    CHAR c_Write[300];
    BYTE b_Mode = PCI3009_SINGLE;
    HANDLE h_ConsoleHandle = NULL;

    h_ConsoleHandle = GetStdHandle(STD_OUTPUT_HANDLE);
    memset (b_ChannelSequence, 255, sizeof (b_ChannelSequence));
        
    printf ("\n\n");
    printf ("\n+-----------------------------------------------------------+");
    printf ("\n| ADDIDATA analog input:                                    |");
    printf ("\n|    Analog input sequence acquisition with interrupt.      |");
    printf ("\n|    The user define the channels he wants to use.          |");
    printf ("\n|    Save analog value into samples.txt                     |");
    printf ("\n+-----------------------------------------------------------+");
    printf ("\n| Connection suggested : -                                  |");
    printf ("\n+-----------------------------------------------------------+");
    printf ("\n| Required connection : -                                   |");
    printf ("\n+-----------------------------------------------------------+");
    printf ("\n| <ESC> exit sample                                         |");
    printf ("\n+-----------------------------------------------------------+");
    printf ("\n| Press any key for start the sample                        |");
    printf ("\n+-----------------------------------------------------------+");
    printf ("\n\n");

    getch ();
        
    /* Open the ADDI-DATA driver */
    i_ReturnValue = i_Utils_OpenBoard(&dw_DriverHandle);
    if (i_ReturnValue != 0)
    {
        getch ();
        return;
    }

    if ((i_Handle = open("samples.txt", O_CREAT | O_TRUNC | O_RDWR, S_IREAD | S_IWRITE)) == -1)
    {
        getch();
        return;
    }

    /* Interrupt function installation */
    i_ReturnValue = i_PCI3009_SetBoardIntRoutineNew (dw_DriverHandle, v_InterruptRoutine);

    switch (i_ReturnValue)
    {
        case 0:
            printf ("\ni_PCI3009_SetBoardIntRoutineNew OK");
            b_InitIRQOK = 1;
        break;

        default:
            printf ("\ni_PCI3009_SetBoardIntRoutineNew: error = %i", i_ReturnValue);
        break;
    }

    if (!i_ReturnValue)
    {
        printf("\nAvailable modes:");
        printf("\n0: Single");
        printf("\n1: Differential");
        b_Mode = (BYTE)ll_Utils_ReadInteger(PCI3009_SINGLE, PCI3009_DIFF, "\nSelect the mode: ");

        i_ReturnValue = i_PCI3009_AnalogInputInitMode(dw_DriverHandle, b_Mode);
        switch (i_ReturnValue)
        {
            case 0:
                printf ("\ni_PCI3009_AnalogInputInitMode OK");
            break;

            default:
                printf ("\ni_PCI3009_AnalogInputInitMode: error = %i", i_ReturnValue);
            break;
        }
    }
                   
    if (!i_ReturnValue)
    {
        b_SequenceSize = (BYTE)ll_Utils_ReadInteger(1, (b_Mode == PCI3009_SINGLE ? 16 : 8), "\n\nSelect the sequence size (0 to %i): ", (b_Mode == PCI3009_SINGLE ? 16 : 8));

        for (w_Cpt = 0;(w_Cpt < b_SequenceSize) && (!i_ReturnValue); w_Cpt++)
        {
            b_ChannelSequence[w_Cpt] = ll_Utils_ReadInteger(0, (b_Mode == PCI3009_SINGLE ? 16 : 8), "\n\nSequence index %i - Select a channel: ", w_Cpt);

            printf("\n\nAvailable polarities:");
            printf("\n0: Unipolar");
            printf("\n1: Bipolar");

            b_Polarity = ll_Utils_ReadInteger(0, 1, "\nSelect the polarity: ");

            if (b_Polarity != 0)
                b_Polarity = PCI3009_BIPOLAR;
            else
                b_Polarity = PCI3009_UNIPOLAR;

            printf("\nAvailable gains:");
            printf("\n0: Gain 1");
            printf("\n1: Gain 2");
            printf("\n2: Gain 5");
            printf("\n3: Gain 10");

            b_Gain = ll_Utils_ReadInteger(PCI3009_1_GAIN, PCI3009_10_GAIN, "\nSelect the gain: ");

            switch (b_Gain)
            {
                case 0: b_Gain = PCI3009_1_GAIN;
                    break;
                case 1: b_Gain = PCI3009_2_GAIN;
                    break;
                case 2: b_Gain = PCI3009_5_GAIN;
                    break;
                case 3: b_Gain = PCI3009_10_GAIN;
                    break;
                default: b_Gain = PCI3009_1_GAIN;
            }

            i_ReturnValue = i_PCI3009_AnalogInputInitChannel (dw_DriverHandle, b_ChannelSequence[w_Cpt], b_Gain, b_Polarity);
            switch (i_ReturnValue)
            {
                case 0:
                    printf ("\ni_PCI3009_AnalogInputInitChannel OK");
                break;

                default:
                    printf ("\ni_PCI3009_AnalogInputInitChannel: error = %i", i_ReturnValue);
                break;
            }
        }
    }

    if (i_ReturnValue == 0)
    {      
        b_HardwareTriggerAction = PCI3009_DISABLE;
        b_HardwareTriggerLevel = 0;
        dw_HardwareTriggerCount = 0;
        dw_HardwareTriggerCycleCount = 0;

        b_HardwareTriggerAction = ll_Utils_ReadInteger(0, 1, "\n\nEnable the hardware trigger (0: Disable, 1: Enable): ");

        if (b_HardwareTriggerAction)
        {
            printf("\nAvailable hardware trigger actions:");
            printf("\n0: One shot trigger");
            printf("\n1: Start a series of HardwareTriggerCycleCount acquisitions");

            b_HardwareTriggerAction = ll_Utils_ReadInteger(0, 1, "\nSelect the hardware trigger action: ");

            switch (b_HardwareTriggerAction)
            {
            case 0: b_HardwareTriggerAction = PCI3009_ONE_SHOT_TRIGGER;
                break;
            case 1: b_HardwareTriggerAction = PCI3009_TRIGGER_START_A_SERIES;
                break;
            default: b_HardwareTriggerAction = PCI3009_ONE_SHOT_TRIGGER;
            }

            printf("\nAvailable hardware trigger levels:");
            printf("\n0: Low front");
            printf("\n1: High front");
            printf("\n2: Low and high front");

            b_HardwareTriggerLevel = ll_Utils_ReadInteger(0, 2, "\nSelect the hardware trigger level: ");

            switch (b_HardwareTriggerLevel)
            {
                case 0: b_HardwareTriggerLevel = PCI3009_LOW_FRONT;
                    break;
                case 1: b_HardwareTriggerLevel = PCI3009_HIGH_FRONT;
                    break;
                case 2: b_HardwareTriggerLevel = PCI3009_LOW_HIGH_FRONT;
                    break;
                default: b_HardwareTriggerLevel = PCI3009_LOW_FRONT;
            }

            dw_HardwareTriggerCount = ll_Utils_ReadInteger(1, 65535, "\nNumber of input pulses before generating the trigger (1 to 65535): ");
            dw_HardwareTriggerCycleCount = ll_Utils_ReadInteger(1, 65535, "\nNumber of sequences to acquire for each trigger (1 to 65535): ");
        }

        i_ReturnValue = i_PCI3009_AnalogInputEnableDisableHardwareTrigger (dw_DriverHandle, 
                                                                            b_HardwareTriggerAction, 
                                                                            b_HardwareTriggerLevel, 
                                                                            dw_HardwareTriggerCount, 
                                                                            dw_HardwareTriggerCycleCount);

        switch (i_ReturnValue)
        {
            case 0:
                printf ("\ni_PCI3009_AnalogInputEnableDisableHardwareTrigger OK");
            break;

            default:
                printf ("\ni_PCI3009_AnalogInputEnableDisableHardwareTrigger: error = %i", i_ReturnValue);
            break;
        }	
    }    

    if (!i_ReturnValue)
    {
        b_DelayMode = PCI3009_DISABLE;
        b_DelayTimeUnit = 0;

        dw_NumberOfSequence = ll_Utils_ReadInteger(0, 4294967295, "\n\nNumber of sequences to acquire (0: continuous): ");
        dw_NumberOfSequenceForEachInterrupt = ll_Utils_ReadInteger(1, 4294967295, "\nNumber of sequences to acquire before generating the interrupt (1 to 4294967295): ");

        printf("\nAvailable conversion time units:");
        printf("\n0: Microseconds");
        printf("\n1: Milliseconds");
        
        b_ConvertTimeUnit = ll_Utils_ReadInteger(0, 1, "\nSelect the conversion time unit: ");

        switch (b_ConvertTimeUnit)
        {
            case 0: b_ConvertTimeUnit = PCI3009_MICRO_SECOND;
                break;
            case 1: b_ConvertTimeUnit = PCI3009_MILLI_SECOND;
                break;
            default: b_ConvertTimeUnit = PCI3009_MICRO_SECOND;
        }

        w_ConvertTime = ll_Utils_ReadInteger(1, 65535, "\nSet the conversion time (10 to 65535): ");

        b_DelayMode = ll_Utils_ReadInteger(0, 1, "\n\nEnable the delay (0: Disable, 1: Enable): ");

        if (b_DelayMode)
        {
            printf("\nAvailable delay modes:");
            printf("\n0: Mode 1");
            printf("\n1: Mode 2");

            b_DelayMode = ll_Utils_ReadInteger(0, 1, "\nSelect the delay mode: ");

            switch (b_DelayMode)
            {
                case 0: b_DelayMode = PCI3009_DELAY_MODE_1;
                        break;
                case 1: b_DelayMode = PCI3009_DELAY_MODE_2;
                        break;
                default: b_DelayMode = PCI3009_DELAY_MODE_1;
            }

            printf("\nAvailable delay time units:");
            printf("\n0: Microseconds");
            printf("\n1: Milliseconds");

            b_DelayTimeUnit = ll_Utils_ReadInteger(0, 1, "\nSelect the delay time unit: ");

            switch (b_DelayTimeUnit)
            {
            case 0: b_DelayTimeUnit = PCI3009_MICRO_SECOND;
                break;
            case 1: b_DelayTimeUnit = PCI3009_MILLI_SECOND;
                break;
            default: b_DelayTimeUnit = PCI3009_MICRO_SECOND;
            }

            w_DelayTime = ll_Utils_ReadInteger(1, 65535, "\nDelay time (1 to 65535): ");
        }

        i_ReturnValue = i_PCI3009_AnalogInputInitSequence (dw_DriverHandle,
                                                            b_SequenceSize,
                                                            b_ChannelSequence,
                                                            b_ConvertTimeUnit,
                                                            w_ConvertTime,
                                                            b_DelayMode,
                                                            b_DelayTimeUnit,
                                                            w_DelayTime,
                                                            dw_NumberOfSequence,
                                                            PCI3009_ENABLE,
                                                            dw_NumberOfSequenceForEachInterrupt); 

        switch (i_ReturnValue)
        {
            case 0:
                printf ("\ni_PCI3009_AnalogInputInitSequence OK");
            break;

            default:
                printf ("\ni_PCI3009_AnalogInputInitSequence: error = %i", i_ReturnValue);
            break;
        }	           
    }

    if (i_ReturnValue == 0)
    {
        valuesPerInterrupt = (dw_NumberOfSequenceForEachInterrupt * b_SequenceSize);

        i_ReturnValue = i_PCI3009_AnalogInputStartSequence(dw_DriverHandle);
        switch (i_ReturnValue)
        {
            case 0:
                printf ("\ni_PCI3009_AnalogInputStartSequence OK");
            break;
         
            default:
                printf ("\ni_PCI3009_AnalogInputStartSequence: error = {0}", i_ReturnValue);
            break;
        }
    }

    if (!i_ReturnValue)
    {    
        int i = 0;
        int interruptCounter = 0;
        double *data = (double *) malloc(sizeof(double) * valuesPerInterrupt);

        clrscr(h_ConsoleHandle);

        while (!stop)
        {
            if (kbhit())
                if (getch() == 27)
                    break;

            /* Wait on data */
            if (dataFIFO == NULL)
            {
                Sleep(1);
                continue;
            }

            i_ReturnValue = i_PCI3009_AnalogInputConvertDMADigitalToRealAnalogValuesNew(dw_DriverHandle, valuesPerInterrupt, dataFIFO->data, data);

            /* Remove the cell from the fifo */
            {
                struct fifo_t *tmp = dataFIFO->next;
                free(dataFIFO->data);
                free(dataFIFO);
                dataFIFO = tmp;
            }

            interruptCounter++;

            gotoxy(h_ConsoleHandle, 0, 0);
            printf("Use ESC key to quit the exemple");
            printf("\nInterrupt: %i", interruptCounter);

            sprintf(c_Write, "\n\nInterrupt: %i", interruptCounter);
            write(i_Handle, c_Write, strlen(c_Write));

            gotoxy(h_ConsoleHandle, 0, 2);

            for (i = 0; i < valuesPerInterrupt && (!stop); i++)
            {
                printf("\n%i \t CH%02i \t %.4f", i, (i % b_SequenceSize), data[i]);
                if ((i % b_SequenceSize) == (b_SequenceSize - 1))
                    gotoxy(h_ConsoleHandle, 0, 2);

                sprintf(c_Write, "\n%i \t CH%02i \t %.4f %c", i, (i % b_SequenceSize), data[i], ((i % b_SequenceSize) == (b_SequenceSize - 1)) ? '\n' : ' ');
                write(i_Handle, c_Write, strlen(c_Write));
            }
        }
    }
    
end:
    clrscr(h_ConsoleHandle);

    switch (stop)
    {
        case 1: printf("\nDriver FIFO overflow");
            break;

        case 2: printf("\nPLD FIFO overflow");
            break;

        case 3: printf("\nInterrupt data malloc");
            break;

        case 4: printf("\nReading data error");
            break;

        case 5: printf("\nFIFO put error");
            break;
    }

	/* Close the sequence */
    i_ReturnValue = i_PCI3009_AnalogInputStopSequence(dw_DriverHandle);

    switch (i_ReturnValue)
    {
    case 0:
    printf("\ni_PCI3009_AnalogInputStopSequence OK");
    break;

    default:
    printf("\ni_PCI3009_AnalogInputStopSequence: error = %i", i_ReturnValue);
    break;
    }

    /* Release the sequence handle */
    i_ReturnValue = i_PCI3009_AnalogInputReleaseSequence(dw_DriverHandle);
    switch (i_ReturnValue)
    {
        case 0:
            printf("\ni_PCI3009_AnalogInputReleaseSequence OK");
        break;

        default:
            printf("\ni_PCI3009_AnalogInputReleaseSequence: error = %i", i_ReturnValue);
        break;
    }

    if (!i_ReturnValue)
    {                                              
        i_ReturnValue = i_PCI3009_AnalogInputEnableDisableHardwareTrigger (dw_DriverHandle, PCI3009_DISABLE, b_HardwareTriggerLevel, dw_HardwareTriggerCount, dw_HardwareTriggerCycleCount);

        switch (i_ReturnValue)
        {
            case 0:
                printf ("\ni_PCI3009_AnalogInputEnableDisableHardwareTrigger OK");
            break;

            default:
                printf ("\ni_PCI3009_AnalogInputEnableDisableHardwareTrigger: error = %i", i_ReturnValue);
            break;
        }	
    } 

    /* Clear the interrupt function 
     * Interrupt function deinstallation 
     */
    if (b_InitIRQOK)
    {
        i_ReturnValue = i_PCI3009_ResetBoardIntRoutine (dw_DriverHandle);
        switch (i_ReturnValue)
        {
            case 0:
                printf ("\ni_PCI3009_ResetBoardIntRoutine OK");
            break;

            default:
                printf ("\ni_PCI3009_ResetBoardIntRoutine: error = %i", i_ReturnValue);
            break;
        }
    }

    i_ReturnValue = i_PCI3009_CloseBoard (dw_DriverHandle);

    switch (i_ReturnValue)
    {
        case 0:
            printf ("\ni_PCI3009_CloseBoard OK");
        break;

        default:
            printf ("\ni_PCI3009_CloseBoard: error = %i", i_ReturnValue);
        break;
    }

    printf ("\n\n");
    printf ("\n+---------------------------------------------------+");
    printf ("\n| Program concluded.                                |");
    printf ("\n+---------------------------------------------------+");
    printf ("\n| Press ESC for exit from sample                    |");
    printf ("\n+---------------------------------------------------+");
    printf ("\n\n");
        
    /* Wait for 'ESC' hit */
    for (;;)
        if (kbhit ())
            if (getch () == 27)
                break;

    close(i_Handle);
    printf ("\n\n");
}
