/**
* This sample shows how to calibrate the analog inputs
**/
#include <stdio.h>
#include <tchar.h>

#include <conio.h>
#include <ctype.h>

#include "..\..\Include\define.h"
#include "..\..\Include\PCI3009.h"
#include "..\..\Utils\Utils.h"

#define OFFSET 0
#define GAIN   2

INT getMeanValue(DWORD boardHandle, BYTE adcNum, DOUBLE *meanValue)
{
	INT returnValue;
	WORD i;
	DWORD tmpMean = 0;

	for (i = 256; i > 0; i--)
	{
		DWORD digitalValue;
		returnValue = i_PCI3009_AnalogInputRead1Channel(boardHandle, (BYTE)(adcNum != 0 ? 8 : 0), PCI3009_MILLI_SECOND, 1, PCI3009_DISABLE, &digitalValue);
		if (returnValue != 0)
		{
			printf("Error %d in i_PCI3009_AnalogInputRead1Channel\n", returnValue);
			return returnValue;
		}
		tmpMean += digitalValue;
	}
	tmpMean >>= 8;

	returnValue = i_PCI3009_AnalogInputConvertDigitalToRealAnalogValue(boardHandle, (BYTE)(adcNum != 0 ? 8 : 0), tmpMean, meanValue);
	if (returnValue != 0)
		printf("Error %d in i_PCI3009_AnalogInputConvertDigitalToRealAnalogValue\n", returnValue);

	return returnValue;
}

INT doCalibration(DWORD boardHandle, BYTE adcNum, BYTE type)
{
	INT returnValue = 0;
	BYTE i;
	BYTE potiValue = 0x80;
	DOUBLE bestDelta = 1e99;
	BYTE bestPotiValue = potiValue;

	printf("Please set a voltage of %.4fV on channel %d, then press a key to proceed.\n", type == GAIN ? 9.9951 : 0, adcNum != 0 ? 8 : 0);
	_getch();

	for (i = 0; i < 8; i++)
	{
		DOUBLE meanValue;
		DOUBLE delta, absDelta;

		returnValue = i_PCI3009_WritePotis(boardHandle, (BYTE)(adcNum + type), potiValue);
		if (returnValue != 0)
		{
			printf("Error %d in i_PCI3009_WritePotis\n", returnValue);
			return returnValue;
		}

		returnValue = getMeanValue(boardHandle, adcNum, &meanValue);
		if (returnValue != 0)
			return returnValue;

		if (type == GAIN)
			delta = meanValue - 9.9951;
		else
			delta = meanValue;
		absDelta = (delta > 0 ? delta : -delta);
		if (absDelta <= bestDelta)
		{
			bestDelta = absDelta;
			bestPotiValue = potiValue;
		}

		if ((type == OFFSET && meanValue > 0) || (type == GAIN && adcNum == 0 && meanValue > 9.9951) || (type == GAIN && adcNum != 0 && meanValue < 9.9951))
			potiValue &= ~(0x80 >> i);
		potiValue |= 0x40 >> i;
	}

	returnValue = i_PCI3009_WritePotis(boardHandle, (BYTE)(adcNum + type), bestPotiValue);
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3009_WritePotis\n", returnValue);
		return returnValue;
	}
	printf("Best delta found: %.4fV\n\n", bestDelta);

	return 0;
}

INT doGainCalibration(DWORD boardHandle, BYTE adcNum)
{

	return 0;
}

int main(int argc, char* argv[])
{
	INT returnValue = 0;
	DWORD boardHandle = 0;
	BYTE i;

    /* Open the ADDI-DATA driver */
	returnValue = i_Utils_OpenBoard(&boardHandle);
    if (returnValue != 0)
    {
        getch ();
        return;
    }

	/* reset the potis */
	for (i = 0; i < 3; i++)
	{
		returnValue = i_PCI3009_WritePotis(boardHandle, i, 0x7F);
		if (returnValue != 0)
		{
			i_PCI3009_CloseBoard(boardHandle);
			if (returnValue == -2)
				printf("Calibrating this board is not allowed!\n");
			else
				printf("Error %d in i_PCI3009_WritePotis\n", returnValue);
			_getch();
			return -1;
		}
	}

	/* set single ended input mode */
	returnValue = i_PCI3009_AnalogInputInitMode(boardHandle, PCI3009_SINGLE);
	if (returnValue != 0)
	{
		i_PCI3009_CloseBoard(boardHandle);
		printf("Error %d in i_PCI3009_AnalogInputInitMode\n", returnValue);
		_getch();
		return -1;
	}

	/* calibration of the bipolar analog inputs */
	for (i = 0; i < 2; i++)
	{
		/* Set bipolar input with gain 1 */
		returnValue = i_PCI3009_AnalogInputInitChannel(boardHandle, (BYTE)(i * 8), PCI3009_1_GAIN, PCI3009_BIPOLAR);
		if (returnValue != 0)
		{
			i_PCI3009_CloseBoard(boardHandle);
			printf("Error %d in i_PCI3009_AnalogInputInitChannel\n", returnValue);
			_getch();
			return -1;
		}

		returnValue = doCalibration(boardHandle, i, OFFSET);
		if (returnValue != 0)
		{
			i_PCI3009_CloseBoard(boardHandle);
			_getch();
			return -1;
		}
		returnValue = doCalibration(boardHandle, i, GAIN);
		if (returnValue != 0)
		{
			i_PCI3009_CloseBoard(boardHandle);
			_getch();
			return -1;
		}
		returnValue = doCalibration(boardHandle, i, OFFSET);
		if (returnValue != 0)
		{
			i_PCI3009_CloseBoard(boardHandle);
			_getch();
			return -1;
		}
	}

	/* save the potis value in the eeprom */
	returnValue = i_PCI3009_SavePotis(boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3009_SavePotis\n", returnValue);
		i_PCI3009_CloseBoard(boardHandle);
		_getch();
		return -1;
	}
	
	/* close the board */
	returnValue = i_PCI3009_CloseBoard(boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3009_CloseBoard \n", returnValue);
	}

	printf("Hit a key to exit this program...\n");
	_getch();

	return 0;
}
