/**
* This sample shows how to calibrate the analog outputs
**/
#include <stdio.h>
#include <tchar.h>

#include <conio.h>
#include <ctype.h>

#include "..\..\Utils\Utils.h"


INT doCalibration(DWORD boardHandle, BYTE polarity)
{
	INT returnValue = 0;
	DWORD potiValue = 0x7F;

	returnValue = i_PCI3009_AnalogOutputWrite1Channel(boardHandle, 0, (WORD)(polarity != 0 ? 4095 : 1));
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3009_AnalogOutputWrite1Channel\n", returnValue);
		return -1;
	}

	printf("Please measure the analog output 0 voltage.\n");
	printf("Press the + and - keys to set %+.4fV, then press enter.\n", polarity != 0 ? 9.9991 : -9.9951);

	for (;;)
	{
		printf("\rCurrent setting: %3d", potiValue);
		switch( _getch() )
		{
			case '+':
			{
				if (potiValue < 255)
				{
					returnValue = i_PCI3009_WritePotis(boardHandle, (BYTE)(polarity != 0 ? 6 : 7), ++potiValue);
					if (returnValue != 0)
					{
						printf("Error %d in i_PCI3009_WritePotis\n", returnValue);
						return -1;
					}
				}
				break;
			}
			case '-':
			{
				if (potiValue > 0)
				{
					returnValue = i_PCI3009_WritePotis(boardHandle, (BYTE)(polarity != 0 ? 6 : 7), --potiValue);
					if (returnValue != 0)
					{
						printf("Error %d in i_PCI3009_WritePotis\n", returnValue);
						return -1;
					}
				}
				break;
			}
			case VK_RETURN:
			{
				printf("\n\n");
				return 0;
			}
		}
	}
}

int main(int argc, char* argv[])
{
	INT returnValue = 0;
	DWORD boardHandle = 0;

	/* open a board */
	returnValue = i_Utils_OpenBoard(&boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d opening the board \n", returnValue);
		_getch();
		return -1;
	}

	/* reset the potis */
	returnValue = i_PCI3009_WritePotis(boardHandle, 6, 0x7F);
	if (returnValue != 0)
	{
		if (returnValue == -2)
			printf("Calibrating this board is not allowed!\n");
		else
			printf("Error %d in i_PCI3009_WritePotis\n", returnValue);
		_getch();
		return -1;
	}
	returnValue = i_PCI3009_WritePotis(boardHandle, 7, 0x7F);
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3009_WritePotis\n", returnValue);
		_getch();
		return -1;
	}

	/* calibration of the negative offset */
	returnValue = doCalibration(boardHandle, 0);
	if (returnValue != 0)
	{
		i_PCI3009_CloseBoard(boardHandle);
		_getch();
		return -1;
	}

	/* calibration of the positive offset */
	returnValue = doCalibration(boardHandle, 1);
	if (returnValue != 0)
	{
		i_PCI3009_CloseBoard(boardHandle);
		_getch();
		return -1;
	}

	/* save the potis value in the eeprom */
	returnValue = i_PCI3009_SavePotis(boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3009_SavePotis\n", returnValue);
		i_PCI3009_CloseBoard(boardHandle);
		_getch();
		return -1;
	}
	
	/* close the board */
	returnValue = i_PCI3009_CloseBoard(boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d in i_PCI3009_CloseBoard \n", returnValue);
	}

	printf("Hit a key to exit this program...\n");
	_getch();

	return 0;
}
