
#include <io.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <math.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "../Include/define.h"
#include "../Include/PCI3009.h"
#include "../Utils/Utils.h"

/*+----------------------------------------------------------------------------+*/
/*| Function   Name   : void main (void)                                       |*/
/*+----------------------------------------------------------------------------+*/
/*| Task              : Main                                                   |*/
/*+----------------------------------------------------------------------------+*/
/*| Input Parameters  : -                                                      |*/
/*+----------------------------------------------------------------------------+*/
/*| Output Parameters : -                                                      |*/
/*+----------------------------------------------------------------------------+*/
/*| Return Value      : -                                                      |*/
/*+----------------------------------------------------------------------------+*/
void    main (void)
{
    int              i_ReturnValue          = 0;
    unsigned int    ui_NbrOfDev             = 0;
    DWORD           dw_Functionality        = 0;
    DWORD           dw_FirmwareRevision     = 0;
    DWORD           dw_FirmwareVersion      = 0;
    DWORD *        pdw_Address3Ring0        = NULL;
    DWORD *        pdw_Address3Ring3        = NULL;
    INT              i_RBFHandle            = 0;
    WORD             w_VendorID             = 0x15B8;
    WORD             w_DeviceID             = 0x3025;
    LONG             l_RBFFileLength        = 0;
    char *          pc_EndPtr               = NULL;
    unsigned char    b_Selection            = 0;
    unsigned char * pb_RBFFileData          = NULL;
    char             c_Path[120];
    // LELP-C: Less or Equal Leiter Platine C
    // GELP-E: Greater or Equal Leiter Platine E
    char             c_SelectedRBFDirectory [2] [10] = {{"LELP-C"}, {"GELP-E"}};
    char             c_SelectedRBFFile [3] [100] = {{"CPCI-3009_Inc_Cpt.rbf"}, {"CPCI-3009_Chronos.rbf"}, {"CPCI-3009_Digital_IO.rbf"}};
    DWORD dw_BoardHandle = 0;
    char index = 0;

    /*******************/
    /* Open the driver */
    /*******************/
    i_ReturnValue = i_PCI3009_OpenBoardViaIndex (0, &dw_BoardHandle);

    switch (i_ReturnValue)
    {
    case 0:
        printf ("\ni_PCI3009_OpenBoardViaIndex OK");
        break;

    case -1:
        printf ("\ni_PCI3009_OpenBoardViaIndex: Unable to get the number of available boards");
        break;

    case -2:
        printf ("\ni_PCI3009_OpenBoardViaIndex: Wrong device index parameter, the parameter is too hight");
        break;

    case -3:
        printf ("\ni_PCI3009_OpenBoardViaIndex: Error while getting the device's class");
        break;

    case -4:
        printf ("\ni_PCI3009_OpenBoardViaIndex: Error while getting informations about the device");
        break;

    case -5:
        printf ("\ni_PCI3009_OpenBoardViaIndex: Memory allocation error");
        break;
		
    case -6:
        printf ("\ni_PCI3009_OpenBoardViaIndex: Get device interface detail error");
        break;
		
    case -7:
        printf ("\ni_PCI3009_OpenBoardViaIndex: Error while creating communication handle");
        break;
		
    case -8:
        printf ("\ni_PCI3009_OpenBoardViaIndex: Error in the initialization callback function");
        break;

    default:
        printf ("\ni_PCI3009_OpenBoardViaIndex: error = %i", i_ReturnValue);
        break;
    }

    if (i_ReturnValue != 0)
		goto end;
    
	printf ("\nThe first detected board will be used\n");

	/*****************************/
	/* Get the firmware revision */
	/*****************************/
	i_ReturnValue = i_PCI3009_PLFimrwareRevision (dw_BoardHandle, &dw_FirmwareRevision);

	if (i_ReturnValue != 0)
	{
		printf ("i_PCI3009_PLFimrwareRevision error\n");
		goto end;
	}

	dw_FirmwareVersion = dw_FirmwareRevision;

	/***************************/
	/* Get the revision number */
	/***************************/
	dw_FirmwareRevision = (((dw_FirmwareRevision >> 4) & 0x7F) - '0') |
			((((dw_FirmwareRevision >> 11) & 0x7F) - '0') << 4);

	/**************************/
	/* Get the version number */
	/**************************/
	dw_FirmwareVersion = ((dw_FirmwareVersion >> 18) & 0x7F)  |
			(((dw_FirmwareVersion >> 25) & 0x7F) << 8);

	/*************************/
	/* Get the functionality */
	/*************************/

	i_ReturnValue = i_PCI3009_PLFimrwareFunctionality (dw_BoardHandle, &dw_Functionality);

	if (i_ReturnValue != 0)
	{
		printf ("i_PCI3009_PLFimrwareFunctionality error\n");
		goto end;
	}

	if ((dw_Functionality & 0xFFFF0000UL) == 0x53430000UL)
	{
		printf ("\nFirmware revision : %c%c%X. Actual configuration : Incremental counter %c%c", dw_FirmwareVersion >> 8, dw_FirmwareVersion & 0xFF, dw_FirmwareRevision, (dw_Functionality >> 8) & 0xFF, dw_Functionality & 0xFF);
	}

	if ((dw_Functionality & 0xFFFF0000UL) == 0x43480000UL)
	{
		printf ("\nFirmware revision : %c%c%X. Actual configuration : Chronos %c%c", dw_FirmwareVersion >> 8, dw_FirmwareVersion & 0xFF, dw_FirmwareRevision, (dw_Functionality >> 8) & 0xFF, dw_Functionality & 0xFF);
	}

	if ((dw_Functionality & 0xFFFF0000UL) == 0x44490000UL)
	{
		printf ("\nFirmware revision : %c%c%X. Actual configuration : Digital I/O %c%c", dw_FirmwareVersion >> 8, dw_FirmwareVersion & 0xFF, dw_FirmwareRevision, (dw_Functionality >> 8) & 0xFF, dw_Functionality & 0xFF);
	}

	printf ("\n\nNew configuration selection:");
	printf ("\n   0 : Incremental counter");
	printf ("\n   1 : Chronos");
	printf ("\n   2 : Digital I/O");
	printf ("\n   3 : Exit\n");

	do
	{
		b_Selection = getch ();
	}
	while ((b_Selection < '0') || (b_Selection > '3'));

	b_Selection = b_Selection - '0';

	if (b_Selection == 3)
		goto end;

	/*****************/
	/* Open RBF File */
	/*****************/

	memset (c_Path, '\0', sizeof (c_Path));

	// 0x30 is up the Leiter Platine E
	if (dw_FirmwareRevision < 0x30)
	{
		// If the currently programmed firmware version is < 0x30
		sprintf (c_Path, "%s\\%s", c_SelectedRBFDirectory[0], c_SelectedRBFFile[b_Selection]);
	}
	else
	{
		// If the currently programmed firmware version is >= 0x30
		sprintf (c_Path, "%s\\%s", c_SelectedRBFDirectory[1], c_SelectedRBFFile[b_Selection]);
	}          
	
	printf ("\nProgramming the firmware, please wait...\n");

	/**********************/
	/* Write the firmware */
	/**********************/
	i_ReturnValue = i_PCI3009_PLFimrwareConfigure (dw_BoardHandle, c_Path);

	switch (i_ReturnValue)
	{
	case 0:
		printf ("\ni_PCI3009_PLFimrwareConfigure OK");
		break;

	case -1:
		printf ("\ni_PCI3009_PLFimrwareConfigure: IOCTL error");
		break;

	case -2:
		printf ("\ni_PCI3009_PLFimrwareConfigure: Timeout");
		break;

	case -3:
		printf ("\ni_PCI3009_PLFimrwareConfigure: Readback error");
		break;

	case -4:
		printf ("\ni_PCI3009_PLFimrwareConfigure: The computer must be restarted to complete the programmation");
		break;

	case -5:
		printf ("\ni_PCI3009_PLFimrwareConfigure: Busy");
		break;
		
	case -6:
		printf ("\ni_PCI3009_PLFimrwareConfigure: Open RBF file error");
		break;
		
	case -7:
		printf ("\ni_PCI3009_PLFimrwareConfigure: RBF file size error");
		break;
		
	case -8:
		printf ("\ni_PCI3009_PLFimrwareConfigure: Memory allocation error");
		break;

	default:
		printf ("\ni_PCI3009_PLFimrwareConfigure: error = %i", i_ReturnValue);
		break;
	}
	
	/*****************************/
	/* Get the firmware revision */
	/*****************************/
	i_ReturnValue = i_PCI3009_PLFimrwareRevision (dw_BoardHandle, &dw_FirmwareRevision);

	if (i_ReturnValue != 0)
	{
		printf ("i_PCI3009_PLFimrwareRevision error\n");
		goto end;
	}

	dw_FirmwareVersion = dw_FirmwareRevision;

	/***************************/
	/* Get the revision number */
	/***************************/
	dw_FirmwareRevision = (((dw_FirmwareRevision >> 4) & 0x7F) - '0') |
			((((dw_FirmwareRevision >> 11) & 0x7F) - '0') << 4);

	/**************************/
	/* Get the version number */
	/**************************/
	dw_FirmwareVersion = ((dw_FirmwareVersion >> 18) & 0x7F)  |
			(((dw_FirmwareVersion >> 25) & 0x7F) << 8);

	/*************************/
	/* Get the functionality */
	/*************************/

	i_ReturnValue = i_PCI3009_PLFimrwareFunctionality (dw_BoardHandle, &dw_Functionality);

	if (i_ReturnValue != 0)
	{
		printf ("i_PCI3009_PLFimrwareFunctionality error\n");
		goto end;
	}

	if ((dw_Functionality & 0xFFFF0000UL) == 0x53430000UL)
	{
		printf ("\n\nNew firmware revision : %c%c%X. New configuration : Incremental counter %c%c", dw_FirmwareVersion >> 8, dw_FirmwareVersion & 0xFF, dw_FirmwareRevision, (dw_Functionality >> 8) & 0xFF, dw_Functionality & 0xFF);
	}

	if ((dw_Functionality & 0xFFFF0000UL) == 0x43480000UL)
	{
		printf ("\n\nNew firmware revision : %c%c%X. New configuration : Chronos %c%c", dw_FirmwareVersion >> 8, dw_FirmwareVersion & 0xFF, dw_FirmwareRevision, (dw_Functionality >> 8) & 0xFF, dw_Functionality & 0xFF);
	}

	if ((dw_Functionality & 0xFFFF0000UL) == 0x44490000UL)
	{
		printf ("\n\nNew firmware revision : %c%c%X. New configuration : Digital I/O %c%c", dw_FirmwareVersion >> 8, dw_FirmwareVersion & 0xFF, dw_FirmwareRevision, (dw_Functionality >> 8) & 0xFF, dw_Functionality & 0xFF);
	}

end:

    i_ReturnValue = i_PCI3009_CloseBoard (dw_BoardHandle);

    switch (i_ReturnValue)
    {
    case 0:
        printf ("\ni_PCI3009_CloseBoard OK");
        break;

    case -1:
        printf ("\ni_PCI3009_CloseBoard: The handle parameter of the board is wrong");
        break;

    default:
        printf ("\ni_PCI3009_CloseBoard: error = %i", i_ReturnValue);
        break;
    }

    printf ("\n\nPress any key");
    getch ();
}
