using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace PCI3009_NET
{
    public unsafe class cl_PCI3009
    {
        #region "Define"

        public const ulong PCI3009_INCREMENTAL_COUNTER = 0x53430000UL;
        public const ulong PCI3009_SSI_COUNTER = 0x53490000UL;
        public const ulong PCI3009_DIGITAL_IO = 0x44490000UL;
        public const byte PCI3009_DISABLE = 0;
        public const byte PCI3009_ENABLE = 1;
        public const byte PCI3009_SYNCHRONOUS_MODE = 1;
        public const byte PCI3009_ASYNCHRONOUS_MODE = 0;

        public const byte PCI3009_16BIT_COUNTER = 0x10;
        public const byte PCI3009_32BIT_COUNTER = 0x0;
        public const byte PCI3009_QUADRUPLE_MODE = 0x0;
        public const byte PCI3009_DOUBLE_MODE = 0x3;
        public const byte PCI3009_SIMPLE_MODE = 0xF;
        public const byte PCI3009_DIRECT_MODE = 0x80;
        public const byte PCI3009_HYSTERESIS_ON = 0x60;
        public const byte PCI3009_HYSTERESIS_OFF = 0x0;
        public const byte PCI3009_INCREMENT = 0x60;
        public const byte PCI3009_DECREMENT = 0x0;
        public const byte PCI3009_LATCH_COUNTER = 0x1;
        public const byte PCI3009_CLEAR_COUNTER = 0x0;
        public const byte PCI3009_LOW = 0x0;
        public const byte PCI3009_HIGH = 0x1;
        public const byte PCI3009_HIGH_EDGE_CLEAR_COUNTER = 0x0;
        public const byte PCI3009_HIGH_EDGE_LATCH_COUNTER = 0x1;
        public const byte PCI3009_LOW_EDGE_CLEAR_COUNTER = 0x2;
        public const byte PCI3009_LOW_EDGE_LATCH_COUNTER = 0x3;
        public const byte PCI3009_HIGH_EDGE_LATCH_AND_CLEAR_COUNTER = 0x4;
        public const byte PCI3009_LOW_EDGE_LATCH_AND_CLEAR_COUNTER = 0x5;
        public const byte PCI3009_SOURCE_0 = 0x0;
        public const byte PCI3009_SOURCE_1 = 0x1;

        public const byte PCI3009_BINARY_MODE = 0x1;
        public const byte PCI3009_GRAY_MODE = 0x0;

        public const byte PCI3009_SINGLE = 0;
        public const byte PCI3009_DIFF = 1;
        public const byte PCI3009_CONTINUOUS = 1;

        public const byte PCI3009_1_GAIN = 0;
        public const byte PCI3009_2_GAIN = 1;
        public const byte PCI3009_5_GAIN = 2;
        public const byte PCI3009_10_GAIN = 3;
        public const byte PCI3009_BIPOLAR = 0;
        public const byte PCI3009_UNIPOLAR = 1;

        public const byte PCI3009_MICRO_SECOND = 1;
        public const byte PCI3009_MILLI_SECOND = 2;
        public const byte PCI3009_SECOND = 3;

        public const byte PCI3009_DELAY_MODE_1 = 1;
        public const byte PCI3009_DELAY_MODE_2 = 2;

        public const byte PCI3009_ONE_SHOT_TRIGGER = 0x1;
        public const byte PCI3009_TRIGGER_START_A_SERIES = 0x11;

        public const byte PCI3009_LOW_FRONT = 0x4;
        public const byte PCI3009_HIGH_FRONT = 0x2;
        public const byte PCI3009_LOW_HIGH_FRONT = 0x6;

        public const long PCI3009_TIMER = 0x4010;
        public const long PCI3009_UP_COUNTER = 0xE0000;
        public const long PCI3009_DOWN_COUNTER = 0xA0000;
        public const long PCI3009_WATCHDOG = 0x0;

        public const long PCI3009_SET_LOW = 0x1000;
        public const long PCI3009_SET_HIGH = 0x800;
        public const long PCI3009_FLIP_FLOP = 0x2800;

        #endregion

        /* Interrupt function prototype */
        delegate void v_FunctionName(uint dw_BoardHandle,
                                     uint dw_InterruptSrc,
                                     uint dw_InterruptMask,
                                     uint* pdw_Value,
                                     uint dw_UserCallingMode,
                                     uint* pv_UserSharedMemory);


        /* Interrupt event */

        #region "Interrupt event handler"

        public delegate void v_EventHandlerDelegate(object source, cl_EventHandlerArgs Args);
        public event v_EventHandlerDelegate e_EventHandlerDelegate;
        public byte b_EventHandlerInitialised;

        public class cl_EventHandlerArgs : EventArgs
        {
            public long dw_InterruptSrc;
            public long dw_InterruptMask;
            public long dw_NumberOfValue;
            public long[] dw_Value = new long[128];
            public uint[] dw_LegacyArgs = new uint[4];	// The legacy parameter is to enable the use of i_AnalogInputConvertDMADigitalToRealAnalogValues
        }

        #endregion

        void v_InterruptFunction(uint dw_BoardHandle,
                                uint dw_InterruptSrc,
                                uint dw_InterruptMask,
                                uint* pdw_Value,
                                uint dw_UserCallingMode,
                                uint* pv_UserSharedMemory)
        {
            cl_EventHandlerArgs Args = new cl_EventHandlerArgs();
            int i_Cpt = 0;

            /* Test if the interrupt handle is installed */
            if (b_EventHandlerInitialised == 1)
            {
                Args.dw_InterruptMask = (long)dw_InterruptMask;
                Args.dw_InterruptSrc = (long)dw_InterruptSrc;

                /* Single acquisition interrupt */
                if (dw_InterruptSrc == 0)
                {
                    if (dw_InterruptMask == 0x1)
                    {
                        Args.dw_NumberOfValue = (long)pdw_Value[0];
                        for (i_Cpt = 1; i_Cpt < (pdw_Value[0] + 1); i_Cpt++)
                            Args.dw_Value[i_Cpt - 1] = (long)pdw_Value[i_Cpt];
                    }

                    /* Sequence acquisition interrupt without DMA */
                    if (dw_InterruptMask == 0x2)
                    {
                        Args.dw_NumberOfValue = (long)pdw_Value[0];
                        for (i_Cpt = 1; i_Cpt < (pdw_Value[0] + 1); i_Cpt++)
                            Args.dw_Value[i_Cpt - 1] = (long)pdw_Value[i_Cpt];
                    }

                    /* Sequence acquisition interrupt with DMA */
                    if ((b_NewInterrupt == 0) && (dw_InterruptMask == 0x4))
                    {
                        Args.dw_NumberOfValue = (long)pdw_Value[1];
                        uint n = (uint)(Args.dw_NumberOfValue >> 1);

                        Array.Resize(ref Args.dw_Value, (int)n);
                        Array.Resize(ref Args.dw_LegacyArgs, (int)(n + 4));

                        Args.dw_LegacyArgs[0] = (uint)pdw_Value[0];     // Number of buffers
                        Args.dw_LegacyArgs[1] = (uint)pdw_Value[1];     // Number of values

                        unsafe
                        {
                            fixed (uint* adr = &Args.dw_LegacyArgs[4])
                            {
                                Args.dw_LegacyArgs[2] = (uint)adr;
                                Args.dw_LegacyArgs[3] = (uint)adr;
                            }
                        }

                        UInt32* values = (UInt32*)pdw_Value[3];
                        for (i_Cpt = 0; i_Cpt < n; i_Cpt++)
                        {
                            Args.dw_Value[i_Cpt] = values[i_Cpt];
                            Args.dw_LegacyArgs[4 + i_Cpt] = values[i_Cpt];
                        }
                    }
                }

                if (dw_InterruptSrc == 1)
                {
                    /* Hardware latch of the 1. register (High level) */
                    if (dw_InterruptMask == 0x1)
                    {
                        Args.dw_Value[0] = (long)pdw_Value[0];
                    }

                    /* Hardware latch of the 1. register (Low level) */
                    if (dw_InterruptMask == 0x10001)
                    {
                        Args.dw_Value[0] = (long)pdw_Value[0];
                    }

                    /* Hardware latch of the 2. register (High level) */
                    if (dw_InterruptMask == 0x2)
                    {
                        Args.dw_Value[0] = (long)pdw_Value[0];
                    }

                    /* Hardware latch of the 2. register (Low level) */
                    if (dw_InterruptMask == 0x10002)
                    {
                        Args.dw_Value[0] = (long)pdw_Value[0];
                    }

                    /* Index interrupt (High level) */
                    if (dw_InterruptMask == 0x4)
                    {
                        Args.dw_Value[0] = (long)pdw_Value[0];
                    }

                    /* Index interrupt (Low level) */
                    if (dw_InterruptMask == 0x10004)
                    {
                        Args.dw_Value[0] = (long)pdw_Value[0];
                    }

                    /* Compare interrupt */
                    if (dw_InterruptMask == 0x8)
                    {
                    }

                    /* Interrupt at the end of frequency measurement */
                    if (dw_InterruptMask == 0x10)
                    {
                        Args.dw_Value[0] = (long)pdw_Value[0];
                    }
                }

                if (dw_InterruptSrc == 2)
                {
                    /* Interrupt at the end of chronos measurement */
                    if (dw_InterruptMask == 0x1)
                    {
                        Args.dw_Value[0] = (long)pdw_Value[0];
                    }
                }

                e_EventHandlerDelegate(this, Args);
            }
        }

        #region "Dll imports"

        #region "str_AutoRefreshDataPackage declaration "

        [StructLayout(LayoutKind.Explicit)]
        struct str_DigitalizedValue
        {
            [FieldOffset(0)]
            public byte b_Open;
            [FieldOffset(1)]
            public byte b_IsInFirstRange;
            [FieldOffset(2)]
            public byte b_IsInSecondRange;
            [FieldOffset(3)]
            public byte b_IsInThirdRange;
            [FieldOffset(4)]
            public ushort w_Reserved0;
            [FieldOffset(6)]
            public ushort w_Reserved1;
        }

        [StructLayout(LayoutKind.Explicit)]
        struct str_AutoRefreshDataPackage
        {
            [FieldOffset(0)]
            public uint dw_ChannelIncrementValue;
            [FieldOffset(4)]
            public ushort w_Reserverd0;
            [FieldOffset(6)]
            public ushort w_Reserverd1;
            [FieldOffset(8)]
            public double d_ChannelValue;
            [FieldOffset(16)]
            public str_DigitalizedValue s_DigitalizedValue;
        }

        #endregion

        #region "System functions"

        [DllImport("msvcrt.dll")]
        static extern int _controlfp(int IN_New, int IN_Mask);

        #endregion

        #region "Initialisation functions"

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_OpenBoardViaIdentifer(string pc_Identification, ref uint pdw_BoardHandle);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_OpenBoardViaIndex(byte b_Index, ref uint pdw_DeviceHandle);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_GetNumberOfBoards(out byte b_NumberOfBoards);

        [DllImport("pci3009.dll")]
        public static extern int i_PCI3009_GetBoardInformation(Byte b_BoardIndex,
            UInt32 dw_IdentifierStringSize,
            Byte[] pc_Identifier,
            out UInt32 pdw_UINumber,
            out UInt32 pdw_DeviceNumber,
            out UInt32 pdw_BusNumber,
            out UInt32 pdw_BaseAddress0,
            out UInt32 pdw_BaseAddress1,
            out UInt32 pdw_BaseAddress2,
            out UInt32 pdw_BaseAddress3,
            out Byte pb_Interrupt,
            out UInt32 pdw_ModuleConfiguration,
            out UInt32 pdw_Revision);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_GetDLLVersion(out uint dw_Version);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_GetDeviceDriverVersion(uint dw_BoardHandle, out uint dw_Version);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_CloseBoard(uint dw_BoardHandle);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_ResetBoardIntRoutine(uint dw_BoardHandle);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_SetBoardIntRoutine(uint dw_BoardHandle,
                                                       uint dw_UserCallingMode,
                                                       uint dw_GlobalBufferSize,
                                                       ref uint ppv_UserBufferRing0,
                                                       ref uint ppv_UserBufferRing3,
                                                       v_FunctionName v_Function);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_SetBoardIntRoutineNew(uint dw_BoardHandle,
                                                       v_FunctionName v_Function);

        [DllImport("pci3009.dll")]
        static extern int l_PCI3009_ReadSequenceData(uint dw_BoardHandle,
                                                       uint dw_Size,
                                                       ushort[] data);
        #endregion

        #region "Timer, counter, watchdog"

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_TmrCptWatInit(uint dw_BoardHandle,
                                                   byte b_TmrCptWat,
                                                   uint dw_OperatingMode,
                                                   uint dw_ReloadValue,
                                                   byte b_TimeBase,
                                                   uint dw_OutputAction,
                                                   byte b_InterruptEnable);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_TmrCptWatEnable(uint dw_BoardHandle,
                                                    byte b_TmrCptWat);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_TmrCptWatDisable(uint dw_BoardHandle,
                                                     byte b_TmrCptWat);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_TmrCptWatGetStatus(uint dw_BoardHandle,
                                                       byte b_TmrCptWat,
                                                       ref uint pdw_Value,
                                                       ref byte pb_DownStatus,
                                                       ref byte pb_TriggerStatus,
                                                       ref byte pb_OutputState);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_TmrCptWatTrigger(uint dw_BoardHandle,
                                                     byte b_TmrCptWat);

        #endregion

        #region "Analog output functions"

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_AnalogInputEnableDisableHardwareTrigger(uint dw_BoardHandle,
                                                                            byte b_HardwareTriggerAction,
                                                                            byte b_HardwareTriggerLevel,
                                                                            uint dw_HardwareTriggerCount,
                                                                            uint dw_HardwareTriggerCycleCount);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_AnalogInputGetHardwareTriggerStatus(uint dw_BoardHandle,
                                                                        ref byte pb_HardwareTriggerAction,
                                                                        ref byte pb_HardwareTriggerStatus,
                                                                        ref uint pdw_HardwareTriggerCount,
                                                                        ref byte pb_HardwareTriggerState);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_AnalogOutputWrite1Channel(uint dw_BoardHandle,
                                                              byte b_Channel,
                                                              ushort w_AnalogOutputValue);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_AnalogOutputWriteMoreChannels(uint dw_BoardHandle,
                                                                  byte b_NbrOfChannels,
                                                                  byte[] pb_ChannelList,
                                                                  ushort[] pw_AnalogOutputValueList);

        #endregion

        #region "Analog input initialisation functions"

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_AnalogInputInitChannel(uint dw_BoardHandle,
                                                           byte b_Channel,
                                                           byte b_Gain,
                                                           byte b_Polarity);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_AnalogInputInitMode(uint dw_BoardHandle,
                                                        byte b_SingelDiffMode);

        #endregion

        #region "Calibration"
        
        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_ReadPotis(uint dw_BoardHandle, byte b_PotisCs, ref uint pdw_Value);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_WritePotis(uint dw_BoardHandle, byte b_PotisCs, uint dw_Value);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_SavePotis(uint dw_BoardHandle);

        #endregion

        #region "Analog input convert functions"

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_AnalogInputConvertDigitalToRealAnalogValue(uint dw_BoardHandle,
                                                                               byte b_Channel,
                                                                               uint dw_DigitalValue,
                                                                               ref double pd_AnalogValue);
        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_AnalogInputConvertMoreDigitalToRealAnalogValues(uint dw_BoardHandle,
                                                                                    byte b_NbrOfChannels,
                                                                                    byte[] pb_ChannelList,
                                                                                    uint[] pdw_DigitalValue,
                                                                                    double[] pd_AnalogValue);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_AnalogInputConvertDMADigitalToRealAnalogValues(uint dw_BoardHandle,
                                                                                   uint[] pdw_DigitalValue,
                                                                                   double[] pd_AnalogValue);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_AnalogInputConvertDMADigitalToRealAnalogValuesNew(uint dw_BoardHandle,
                                                                                    uint dw_Size,
                                                                                   ushort[] pw_DigitalValue,
                                                                                   double[] pd_AnalogValue);
        #endregion

        #region "Analog input single acquisition functions"

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_AnalogInputRead1Channel(uint dw_BoardHandle,
                                                            byte b_Channel,
                                                            byte b_ConvertTimeUnit,
                                                            ushort w_ConvertTime,
                                                            byte b_Interrupt,
                                                            ref uint pdw_DigitalValue);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_AnalogInputReadMoreChannels(uint dw_BoardHandle,
                                                                byte b_NbrOfChannels,
                                                                byte[] pb_ChannelList,
                                                                byte b_ConvertTimeUnit,
                                                                ushort w_ConvertTime,
                                                                byte b_Interrupt,
                                                                uint[] pdw_DigitalValue);

        #endregion

        #region "Analog input autorefresh acquisition functions"

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_AnalogInputReadAutoRefreshValueAndCounter(uint dw_BoardHandle,
                                                                              uint[] pdw_ReadValue,
                                                                              ref uint pdw_CycleIndex);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_AnalogInputStartAutoRefresh(uint dw_BoardHandle,
                                                                ushort w_ChannelMask,
                                                                byte b_ConvertTimeUnit,
                                                                uint w_ConvertTime);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_AnalogInputStopAutoRefresh(uint dw_BoardHandle);

        #endregion

        #region "Analog input sequence acquisition functions"

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_AnalogInputInitSequence(uint dw_BoardHandle,
                                                            byte b_NbrOfChannels,
                                                            byte[] pb_ChannelList,
                                                            byte b_ConvertTimeUnit,
                                                            ushort w_ConvertTime,
                                                            byte b_DelayMode,
                                                            byte b_DelayTimeUnit,
                                                            ushort w_DelayTime,
                                                            uint dw_NumberOfSequence,
                                                            byte b_UseDMA,
                                                            uint dw_NumberOfSequenceForEachInterrupt);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_AnalogInputReleaseSequence(uint dw_BoardHandle);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_AnalogInputStartSequence(uint dw_BoardHandle);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_AnalogInputStopSequence(uint dw_BoardHandle);

        #endregion

        #region "Analog input compare logic functions"

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_AnalogInputEnableDisableCompareLogic(uint dw_BoardHandle,
                                                                         byte b_CompareLogicFlag);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_AnalogInputGetAllCompareLogicStates(uint dw_BoardHandle,
                                                                        ref uint pdw_CompareLogicStates);
        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_AnalogInputGetAutoRefreshCompareLogicStates(uint dw_BoardHandle,
                                                                                byte b_Channel,
                                                                                ref str_AutoRefreshDataPackage ps_AutoRefreshDataPackage,
                                                                                uint dw_StructSize);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_AnalogInputGetSingleCompareLogicStates(uint dw_BoardHandle,
                                                                           byte b_Channel,
                                                                           ref byte pb_CompareLogicStates);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_AnalogInputInitCompareLogic(uint dw_BoardHandle,
                                                                byte b_Channel,
                                                                byte b_CompareLevelAMode,
                                                                double d_CompareLevelAValue,
                                                                byte b_CompareLevelBMode,
                                                                double d_CompareLevelBValue);

        #endregion

        #region Load logic

        [DllImport("pci3009.dll", CharSet = CharSet.Ansi)]
        static extern int i_PCI3009_PLFimrwareConfigure(uint dw_BoardHandle, char[] pc_rbfPath);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_PLFimrwareRevision(uint dw_BoardHandle, ref uint pdw_Revision);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_PLFimrwareFunctionality(uint dw_BoardHandle, ref uint pdw_Functionality);

        #endregion


        #region "Digital I/O functions"

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_InitDigitalIO(uint dw_BoardHandle,
                                                  byte b_ChannelAMode,
                                                  byte b_ChannelBMode);

        /* Digital input */

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_ReadDigitalIOChlValue(uint dw_BoardHandle,
                                                          byte b_InputChannel,
                                                          ref byte pb_ChannelStatus);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_ReadDigitalIOPortValue(uint dw_BoardHandle,
                                                           ref byte pb_PortValue);

        /* Digital output */

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_SetDigitalIOChlOff(uint dw_BoardHandle,
                                                       byte b_OutputChannel);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_SetDigitalIOChlOn(uint dw_BoardHandle,
                                                      byte b_OutputChannel);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_SetDigitalIOMemoryOff(uint dw_BoardHandle);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_SetDigitalIOMemoryOn(uint dw_BoardHandle);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_SetDigitalIOPortOff(uint dw_BoardHandle,
                                                        byte b_PortValue);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_SetDigitalIOPortOn(uint dw_BoardHandle,
                                                       byte b_PortValue);

        #endregion

        #region "SSI functions"

        /* SSI formated tram formated functions */

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_InitSSI(uint dw_BoardHandle,
                                            byte b_SSIProfile,
                                            byte b_PositionTurnLength,
                                            byte b_TurnCptLength,
                                            uint dw_SSIOutputClock,
                                            byte b_SSICountingMode);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_Read1SSIValue(uint dw_BoardHandle,
                                                  byte b_SelectedSSI,
                                                  ref uint pdw_Position,
                                                  ref uint pdw_TurnCpt);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_ReadAllSSIValue(uint dw_BoardHandle,
                                                    uint[] pdw_Position,
                                                    uint[] pdw_TurnCpt);

        /* SSI raw data functions */

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_GetSSIAcquisitionStatus(uint dw_BoardHandle,
                                                            ref byte pb_Status);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_GetSSIRawDataValue(uint dw_BoardHandle,
                                                       byte b_SelectedSSI,
                                                       uint[] pdw_ValueArray,
                                                       byte b_ValueArraySize);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_GetSSIValue(uint dw_BoardHandle,
                                                byte b_SelectedSSI,
                                                ref uint pdw_Position,
                                                ref uint pdw_TurnCpt);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_InitSSIRawData(uint dw_BoardHandle,
                                                   byte b_SSIProfile,
                                                   uint dw_SSIOutputClock);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_Read1SSIRawDataValue(uint dw_BoardHandle,
                                                         byte b_SelectedSSI,
                                                         uint[] pdw_ValueArray,
                                                         byte b_ValueArraySize);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_ReadAllSSIRawDataValue(uint dw_BoardHandle,
                                                           uint[] pdw_ValueArray,
                                                           byte b_ValueArraySize);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_StartSSIAcquisition(uint dw_BoardHandle);

        /* SSI digital input functions */

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_ReadSSI1DigitalInput(uint dw_BoardHandle,
                                                         byte b_InputChannel,
                                                         ref byte pb_ChannelStatus);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_ReadSSIAllDigitalInput(uint dw_BoardHandle,
                                                           ref byte pb_ChannelStatus);

        /* SSI digital output functions */

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_SetSSIDigitalOutputOff(uint dw_BoardHandle);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_SetSSIDigitalOutputOn(uint dw_BoardHandle);

        #endregion

        #region "incremental counter functions"

        /* Incremental counter initialisation functions */

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_ClearCounterValue(uint dw_BoardHandle);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_CounterAutoTest(uint dw_BoardHandle,
                                                    ref byte pb_TestStatus);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_InitCounter(uint dw_BoardHandle,
                                                byte b_CounterRange,
                                                byte b_FirstCounterModus,
                                                byte b_FirstCounterOption,
                                                byte b_SecondCounterModus,
                                                byte b_SecondCounterOption);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_SetInputFilter(uint dw_BoardHandle,
                                                   byte b_Filter);

        /* Incremental counter read functions */

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_DisableLatchInterrupt(uint dw_BoardHandle);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_EnableLatchInterrupt(uint dw_BoardHandle);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_LatchCounter(uint dw_BoardHandle,
                                                 byte b_LatchReg);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_Read16BitCounterValue(uint dw_BoardHandle,
                                                          byte b_SelectedCounter,
                                                          ref uint pw_CounterValue);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_Read32BitCounterValue(uint dw_BoardHandle,
                                                          ref uint pw_CounterValue);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_ReadLatchRegisterStatus(uint dw_BoardHandle,
                                                            byte b_LatchReg,
                                                            ref byte pb_LatchStatus);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_ReadLatchRegisterValue(uint dw_BoardHandle,
                                                           byte b_LatchReg,
                                                           ref uint pdw_LatchValue);

        /* incremental counter write functions */

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_Write16BitCounterValue(uint dw_BoardHandle,
                                                           byte b_SelectedCounter,
                                                           ushort w_WriteValue);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_Write32BitCounterValue(uint dw_BoardHandle,
                                                           uint w_WriteValue);

        /* Incremental counter index functions */

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_DisableIndex(uint dw_BoardHandle);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_EnableIndex(uint dw_BoardHandle);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_GetIndexStatus(uint dw_BoardHandle,
                                                   ref byte pb_IndexStatus);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_GetReferenceStatus(uint dw_BoardHandle,
                                                    ref byte pb_ReferenceStatus);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_InitIndex(uint dw_BoardHandle,
                                              byte b_ReferenceAction,
                                              byte b_IndexOperation,
                                              byte b_AutoMode,
                                              byte b_InterruptEnable);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_InitReference(uint dw_BoardHandle,
                                                  byte b_ReferenceLevel);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_SetIndexAndReferenceSource(uint dw_BoardHandle,
                                                               byte b_SourceSelection);

        /* Incremental counter UAS,CBx,U/D#, external strobe functions */

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_Get16BitCBStatus(uint dw_BoardHandle,
                                                     ref byte pb_CBStatusCounter0,
                                                     ref byte pb_CBStatusCounter1);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_GetCBStatus(uint dw_BoardHandle,
                                                ref byte pb_CBStatus);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_GetInterruptUDLatchedStatus(uint dw_BoardHandle,
                                                                ref byte pb_UDStatus);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_GetUASStatus(uint dw_BoardHandle,
                                                 ref byte pb_UASStatus);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_GetUDStatus(uint dw_BoardHandle,
                                                ref byte pb_UDStatus);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_InitExternalStrobe(uint dw_BoardHandle,
                                                       byte b_ExternalStrobe,
                                                       byte b_ExternalStrobeLevel);

        /* Incremental counter compare functions */

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_DisableCompareLogic(uint dw_BoardHandle);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_EnableCompareLogic(uint dw_BoardHandle);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_InitCompareLogic(uint dw_BoardHandle,
                                                     uint dw_CompareValue);

        /* Incremental counter frequency measurement functions */

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_DisableFrequencyMeasurement(uint dw_BoardHandle);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_EnableFrequencyMeasurement(uint dw_BoardHandle,
                                                               byte b_InterruptEnable);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_InitFrequencyMeasurement(uint dw_BoardHandle,
                                                             byte b_TimingUnity,
                                                             uint dw_TimingInterval,
                                                             ref uint pdw_RealTimingInterval);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_ReadFrequencyMeasurement(uint dw_BoardHandle,
                                                             ref byte pb_Status,
                                                             ref byte pb_UDStatus,
                                                             ref uint pdw_ReadValue);

        /* Incremental counter digital output functions */

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_SetDigitalChlOff(uint dw_BoardHandle);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_SetDigitalChlOn(uint dw_BoardHandle);

        #endregion

        #region "Chronos functions"

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_InitChrono(uint dw_BoardHandle,
                                                  byte b_ChronoMode,
                                                  byte b_TimingUnit,
                                                  uint dw_TimingInterval,
                                                  ref uint pdw_RealTimingInterval);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_EnableChrono(uint dw_BoardHandle,
                                                  byte b_CycleMode,
                                                  byte b_InterruptEnable);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_DisableChrono(uint dw_BoardHandle);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_GetChronoProgressStatus(uint dw_BoardHandle,
                                                                  ref byte pb_ChronoStatus);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_ReadChronoValue(uint dw_BoardHandle,
                                                          uint dw_TimeOut,
                                                          ref byte pb_ChronoStatus,
                                                          ref uint pdw_ChronoValue);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_ConvertChronoValue(uint dw_BoardHandle,
                                                            uint dw_ChronoValue,
                                                            ref uint pdw_Hour,
                                                            ref byte pb_Minute,
                                                            ref byte pb_Second,
                                                            ref uint pdw_MilliSecond,
                                                            ref uint pdw_MicroSecond,
                                                            ref uint pdw_NanoSecond);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_SetChronoChlOn(uint dw_BoardHandle,
                                                          byte b_OutputChannel);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_SetChronoChlOff(uint dw_BoardHandle,
                                                          byte b_OutputChannel);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_ReadChronoChlValue(uint dw_BoardHandle,
                                                          byte b_InputChannel,
                                                          ref byte pb_ChannelStatus);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_ReadChronoPortValue(uint dw_BoardHandle,
                                                          ref byte pb_PortValue);

        #endregion

        #region "Digital input functions"

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_DigitalInputRead1(uint dw_BoardHandle,
                                                      byte b_Channel,
                                                      ref byte pb_Status);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_DigitalInputReadAll(uint dw_BoardHandle,
                                                        ref byte pb_Status);

        #endregion

        #region "Digital output functions"

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_DigitalOutputReadAll(uint dw_BoardHandle,
                                                         ref byte pb_Status);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_DigitalOutputWriteAll(uint dw_BoardHandle,
                                                          byte b_WriteValue);

        [DllImport("pci3009.dll")]
        static extern int i_PCI3009_DigitalInputEnableDisableFilter(uint dw_BoardHandle,
                                                                    byte b_Filter,
                                                                    uint dw_FilterTime);

        #endregion

        #endregion

        #region "Class variable and structure"

        uint dw_BoardHandle;
        static v_FunctionName v_IntFunc;
        byte b_NewInterrupt;

        public class cl_AutoRefreshDataPackage
        {
            public long dw_ChannelIncrementValue;
            public double d_ChannelValue;
            public byte b_Open;
            public byte b_IsInFirstRange;
            public byte b_IsInSecondRange;
            public byte b_IsInThirdRange;
        }

        #endregion

        #region "Class functions"

        #region "Initialisation functions"

        public int i_GetDLLVersion(out uint version)
        {
            return i_PCI3009_GetDLLVersion(out version);
        }

        public int i_GetDeviceDriverVersion(out uint version)
        {
            return i_PCI3009_GetDeviceDriverVersion(dw_BoardHandle, out version);
        }

        public int i_GetNumberOfBoards(out byte numberOfBoards)
        {
            return i_PCI3009_GetNumberOfBoards(out numberOfBoards);
        }

        public int i_GetBoardInformation(Byte b_BoardIndex,
            UInt32 dw_IdentifierStringSize,
            Byte[] pc_Identifier,
            out UInt32 pdw_UINumber,
            out UInt32 pdw_DeviceNumber,
            out UInt32 pdw_BusNumber,
            out UInt32 pdw_BaseAddress0,
            out UInt32 pdw_BaseAddress1,
            out UInt32 pdw_BaseAddress2,
            out UInt32 pdw_BaseAddress3,
            out Byte pb_Interrupt,
            out UInt32 pdw_ModuleConfiguration,
            out UInt32 pdw_Revision)
        {
            return i_PCI3009_GetBoardInformation(b_BoardIndex,
                dw_IdentifierStringSize,
                pc_Identifier,
                out pdw_UINumber,
                out pdw_DeviceNumber,
                out pdw_BusNumber,
                out pdw_BaseAddress0,
                out pdw_BaseAddress1,
                out pdw_BaseAddress2,
                out pdw_BaseAddress3,
                out pb_Interrupt,
                out pdw_ModuleConfiguration,
                out pdw_Revision);
        }


        public int i_OpenBoardViaIdentifer(string pc_Identification)
        {
            int i_ReturnValue = 0;

            if ((i_ReturnValue = i_PCI3009_OpenBoardViaIdentifer(pc_Identification, ref dw_BoardHandle)) != 0)
            {
                dw_BoardHandle = 0;
            }

            return (i_ReturnValue);
        }

        public int i_OpenBoardViaIndex(byte index)
        {
            int i_ReturnValue = 0;
            if ((i_ReturnValue = i_PCI3009_OpenBoardViaIndex(index, ref dw_BoardHandle)) != 0)
            {
                dw_BoardHandle = 0;
            }

            return (i_ReturnValue);
        }

        public int i_CloseBoard()
        {
            return (i_PCI3009_CloseBoard(dw_BoardHandle));
        }

        public int i_SetBoardIntRoutine()
        {
            int i_ReturnValue = 0;
            uint dw_Dummy0 = 0;
            uint dw_Dummy3 = 0;

            b_NewInterrupt = 0;

            if (v_IntFunc == null)
                v_IntFunc = new v_FunctionName(v_InterruptFunction);

            i_ReturnValue = i_PCI3009_SetBoardIntRoutine(dw_BoardHandle,
                                                         PCI3009_ASYNCHRONOUS_MODE,
                                                         0,
                                                         ref dw_Dummy0,
                                                         ref dw_Dummy3,
                                                         v_IntFunc);

            return (i_ReturnValue);
        }

        public int i_SetBoardIntRoutineNew()
        {
            int i_ReturnValue = 0;

            b_NewInterrupt = 1;

            if (v_IntFunc == null)
                v_IntFunc = new v_FunctionName(v_InterruptFunction);

            i_ReturnValue = i_PCI3009_SetBoardIntRoutineNew(dw_BoardHandle, v_IntFunc);

            return (i_ReturnValue);
        }

        public int ReadSequenceData(uint dw_Size, ushort[] data)
        {
            return (l_PCI3009_ReadSequenceData(dw_BoardHandle, dw_Size, data));
        }

        public int i_ResetBoardIntRoutine()
        {
            return (i_PCI3009_ResetBoardIntRoutine(dw_BoardHandle));
        }

        #endregion

        #region "Calibration"

        public int i_ReadPotis(byte b_PotisCs, ref long pdw_Value)
        {
            int i_ReturnValue = 0;
            uint ui_Value = 0;

            i_ReturnValue = i_PCI3009_ReadPotis(dw_BoardHandle, b_PotisCs, ref ui_Value);
            pdw_Value = (long)ui_Value;

            return (i_ReturnValue);
        }

        public int i_WritePotis(byte b_PotisCs, long dw_Value)
        {
            uint ui_Value = (uint)dw_Value;
            return (i_PCI3009_WritePotis(dw_BoardHandle, b_PotisCs, ui_Value));
        }

        public int i_SavePotis()
        {
            return (i_PCI3009_SavePotis(dw_BoardHandle));
        }

        #endregion

        #region "SSI"

        public int i_InitSSI(byte b_SSIProfile,
                             byte b_PositionTurnLength,
                             byte b_TurnCptLength,
                             long dw_SSIOutputClock,
                             byte b_SSICountingMode)
        {
            uint ui_SSIOutputClock = (uint)dw_SSIOutputClock;

            return (i_PCI3009_InitSSI(dw_BoardHandle,
                                      b_SSIProfile,
                                      b_PositionTurnLength,
                                      b_TurnCptLength,
                                      ui_SSIOutputClock,
                                      b_SSICountingMode));
        }

        public int i_Read1SSIValue(byte b_SelectedSSI,
                                   ref long pdw_Position,
                                   ref long pdw_TurnCpt)
        {
            uint ui_Position = 0;
            uint ui_TurnCpt = 0;
            int i_ReturnValue = 0;

            i_ReturnValue = i_PCI3009_Read1SSIValue(dw_BoardHandle,
                                                    b_SelectedSSI,
                                                    ref ui_Position,
                                                    ref ui_TurnCpt);
            pdw_Position = (long)ui_Position;
            pdw_TurnCpt = (long)ui_TurnCpt;

            return (i_ReturnValue);
        }

        public int i_ReadAllSSIValue(byte b_SelectedSSI,
                                     long[] pdw_Position,
                                     long[] pdw_TurnCpt)
        {
            uint[] pui_Position = new uint[3];
            uint[] pui_TurnCpt = new uint[3];
            int i_ReturnValue = 0;
            int i = 0;

            i_ReturnValue = i_PCI3009_ReadAllSSIValue(dw_BoardHandle,
                                                      pui_Position,
                                                      pui_TurnCpt);
            for (i = 0; i < 3; i++)
            {
                pdw_Position[i] = pui_Position[i];
                pdw_TurnCpt[i] = pui_TurnCpt[i];
            }

            return (i_ReturnValue);
        }

        /* SSI raw data functions */

        public int i_GetSSIAcquisitionStatus(ref byte pb_Status)
        {
            return (i_PCI3009_GetSSIAcquisitionStatus(dw_BoardHandle,
                                                      ref pb_Status));
        }

        public int i_GetSSIRawDataValue(byte b_SelectedSSI,
                                        long[] pdw_ValueArray,
                                        byte b_ValueArraySize)
        {
            uint[] pui_ValueArray = new uint[b_ValueArraySize];
            int i_ReturnValue = 0;
            int i = 0;

            i_ReturnValue = i_PCI3009_GetSSIRawDataValue(dw_BoardHandle,
                                                         b_SelectedSSI,
                                                         pui_ValueArray,
                                                         b_ValueArraySize);

            for (i = 0; i < b_ValueArraySize; i++)
            {
                pdw_ValueArray[i] = pui_ValueArray[i];
            }

            return (i_ReturnValue);
        }

        public int i_GetSSIValue(byte b_SelectedSSI,
                                 ref long pdw_Position,
                                 ref long pdw_TurnCpt)
        {
            uint ui_Position = 0;
            uint ui_TurnCpt = 0;
            int i_ReturnValue = 0;

            i_ReturnValue = i_PCI3009_GetSSIValue(dw_BoardHandle,
                                                  b_SelectedSSI,
                                                  ref ui_Position,
                                                  ref ui_TurnCpt);
            pdw_Position = (long)ui_Position;
            pdw_TurnCpt = (long)ui_TurnCpt;

            return (i_ReturnValue);
        }

        public int i_InitSSIRawData(byte b_SSIProfile,
                                    long dw_SSIOutputClock)
        {
            uint ui_SSIOutputClock = (uint)dw_SSIOutputClock;

            return (i_PCI3009_InitSSIRawData(dw_BoardHandle,
                                             b_SSIProfile,
                                             ui_SSIOutputClock));
        }

        public int i_Read1SSIRawDataValue(byte b_SelectedSSI,
                                          long[] pdw_ValueArray,
                                          byte b_ValueArraySize)
        {
            uint[] pui_ValueArray = new uint[b_ValueArraySize];
            int i_ReturnValue = 0;
            int i = 0;

            i_ReturnValue = i_PCI3009_Read1SSIRawDataValue(dw_BoardHandle,
                                                           b_SelectedSSI,
                                                           pui_ValueArray,
                                                           b_ValueArraySize);

            for (i = 0; i < b_ValueArraySize; i++)
            {
                pdw_ValueArray[i] = (long)pui_ValueArray[i];
            }

            return (i_ReturnValue);
        }

        public int i_ReadAllSSIRawDataValue(long[] pdw_ValueArray,
                                            byte b_ValueArraySize)
        {
            uint[] pui_ValueArray = new uint[b_ValueArraySize];
            int i_ReturnValue = 0;
            int i = 0;

            i_ReturnValue = i_PCI3009_ReadAllSSIRawDataValue(dw_BoardHandle,
                                                             pui_ValueArray,
                                                             b_ValueArraySize);

            for (i = 0; i < b_ValueArraySize; i++)
            {
                pdw_ValueArray[i] = pui_ValueArray[i];
            }

            return (i_ReturnValue);
        }

        public int i_StartSSIAcquisition()
        {
            return (i_PCI3009_StartSSIAcquisition(dw_BoardHandle));
        }

        /* SSI digital input functions */

        public int i_ReadSSI1DigitalInput(byte b_InputChannel,
                                          ref byte pb_ChannelStatus)
        {
            return (i_PCI3009_ReadSSI1DigitalInput(dw_BoardHandle,
                                                   b_InputChannel,
                                                   ref pb_ChannelStatus));
        }

        public int i_ReadSSIAllDigitalInput(ref byte pb_ChannelStatus)
        {
            return (i_PCI3009_ReadSSIAllDigitalInput(dw_BoardHandle,
                                                     ref pb_ChannelStatus));
        }

        /* SSI digital output functions */

        public int i_SetSSIDigitalOutputOff()
        {
            return (i_PCI3009_SetSSIDigitalOutputOff(dw_BoardHandle));
        }

        public int i_SetSSIDigitalOutputOn()
        {
            return (i_PCI3009_SetSSIDigitalOutputOn(dw_BoardHandle));
        }

        #endregion

        #region "Chonos functions"

        public int i_InitChrono(byte b_ChronoMode,
                                byte b_TimingUnit,
                                long dw_TimingInterval,
                                ref long pdw_RealTimingInterval)
        {
            uint ui_TimingInterval = (uint)dw_TimingInterval;
            uint pui_RealTimingInterval = (uint)pdw_RealTimingInterval;
            int i_ReturnValue = 0;

            if ((i_ReturnValue = i_PCI3009_InitChrono(dw_BoardHandle,
                                                     b_ChronoMode,
                                                     b_TimingUnit,
                                                     ui_TimingInterval,
                                                     ref pui_RealTimingInterval)) == 0)
            {
                pdw_RealTimingInterval = (uint)pui_RealTimingInterval;
            }

            return (i_ReturnValue);
        }

        public int i_EnableChrono(byte b_CycleMode,
                                  byte b_InterruptEnable)
        {

            return (i_PCI3009_EnableChrono(dw_BoardHandle,
                                           b_CycleMode,
                                           b_InterruptEnable));
        }

        public int i_DisableChrono()
        {

            return (i_PCI3009_DisableChrono(dw_BoardHandle));
        }

        public int i_GetChronoProgressStatus(ref byte pb_ChronoStatus)
        {
            return (i_PCI3009_GetChronoProgressStatus(dw_BoardHandle,
                                                      ref pb_ChronoStatus));
        }

        public int i_ReadChronoValue(long dw_TimeOut,
                                     ref byte pb_ChronoStatus,
                                     ref long pdw_ChronoValue)
        {
            uint pui_ChronoValue = (uint)pdw_ChronoValue;
            uint ui_TimeOut = (uint)dw_TimeOut;
            int i_ReturnValue = 0;

            if ((i_ReturnValue = i_PCI3009_ReadChronoValue(dw_BoardHandle,
                                                          ui_TimeOut,
                                                          ref pb_ChronoStatus,
                                                          ref pui_ChronoValue)) == 0)
            {
                pdw_ChronoValue = (uint)pui_ChronoValue;
            }

            return (i_ReturnValue);
        }

        public int i_ConvertChronoValue(long dw_ChronoValue,
                                        ref long pdw_Hour,
                                        ref byte pb_Minute,
                                        ref byte pb_Second,
                                        ref long pdw_MilliSecond,
                                        ref long pdw_MicroSecond,
                                        ref long pdw_NanoSecond)
        {
            uint ui_ChronoValue = (uint)dw_ChronoValue;
            uint pui_Hour = (uint)pdw_Hour;
            uint pui_MilliSecond = (uint)pdw_MilliSecond;
            uint pui_MicroSecond = (uint)pdw_MicroSecond;
            uint pui_NanoSecond = (uint)pdw_NanoSecond;
            int i_ReturnValue = 0;

            if ((i_ReturnValue = i_PCI3009_ConvertChronoValue(dw_BoardHandle,
                                                             ui_ChronoValue,
                                                            ref pui_Hour,
                                                            ref pb_Minute,
                                                            ref pb_Second,
                                                            ref pui_MilliSecond,
                                                            ref pui_MicroSecond,
                                                            ref pui_NanoSecond)) == 0)
            {
                pdw_Hour = (uint)pui_Hour;
                pdw_MilliSecond = (uint)pui_MilliSecond;
                pdw_MicroSecond = (uint)pui_MicroSecond;
                pdw_NanoSecond = (uint)pui_NanoSecond;
            }

            return (i_ReturnValue);
        }

        public int i_SetChronoChlOn(byte b_OutputChannel)
        {
            return (i_PCI3009_SetChronoChlOn(dw_BoardHandle, b_OutputChannel));
        }

        public int i_SetChronoChlOff(byte b_OutputChannel)
        {
            return (i_PCI3009_SetChronoChlOff(dw_BoardHandle, b_OutputChannel));
        }

        public int i_ReadChronoChlValue(byte b_InputChannel,
                                        ref byte pb_ChannelStatus)
        {
            return (i_PCI3009_ReadChronoChlValue(dw_BoardHandle,
                                                 b_InputChannel,
                                                 ref pb_ChannelStatus));
        }

        public int i_ReadChronoPortValue(ref byte pb_PortValue)
        {
            return (i_PCI3009_ReadChronoPortValue(dw_BoardHandle, ref pb_PortValue));
        }

        #endregion

        #region "Timer, counter, watchdog"

        public int i_TmrCptWatInit(byte b_TmrCptWat,
                                   long dw_OperatingMode,
                                   long dw_ReloadValue,
                                   byte b_TimeBase,
                                   long dw_OutputAction,
                                   byte b_InterruptEnable)
        {
            uint ui_OperatingMode = (uint)dw_OperatingMode;
            uint ui_ReloadValue = (uint)dw_ReloadValue;
            uint ui_OutputAction = (uint)dw_OutputAction;

            return (i_PCI3009_TmrCptWatInit(dw_BoardHandle,
                                            b_TmrCptWat,
                                            ui_OperatingMode,
                                            ui_ReloadValue,
                                            b_TimeBase,
                                            ui_OutputAction,
                                            b_InterruptEnable));
        }

        public int i_TmrCptWatEnable(byte b_TmrCptWat)
        {
            return (i_PCI3009_TmrCptWatEnable(dw_BoardHandle, b_TmrCptWat));
        }

        public int i_TmrCptWatDisable(byte b_TmrCptWat)
        {
            return (i_PCI3009_TmrCptWatDisable(dw_BoardHandle, b_TmrCptWat));
        }

        public int i_TmrCptWatGetStatus(byte b_TmrCptWat,
                                        ref long pdw_Value,
                                        ref byte pb_DownStatus,
                                        ref byte pb_TriggerStatus,
                                        ref byte pb_OutputState)
        {
            int i_ReturnValue = 0;
            uint ui_Value = 0;

            i_ReturnValue = i_PCI3009_TmrCptWatGetStatus(dw_BoardHandle,
                                                         b_TmrCptWat,
                                                         ref ui_Value,
                                                         ref pb_DownStatus,
                                                         ref pb_TriggerStatus,
                                                         ref pb_OutputState);

            pdw_Value = ui_Value;

            return (i_ReturnValue);
        }

        public int i_TmrCptWatTrigger(byte b_TmrCptWat)
        {
            return (i_PCI3009_TmrCptWatTrigger(dw_BoardHandle,
                                               b_TmrCptWat));
        }

        #endregion

        #region "Analog output"

        public int i_AnalogOutputWrite1Channel(byte b_Channel,
                                                int w_AnalogOutputValue)
        {
            ushort us_AnalogOutputValue = (ushort)w_AnalogOutputValue;

            return (i_PCI3009_AnalogOutputWrite1Channel(dw_BoardHandle,
                                                        b_Channel,
                                                        us_AnalogOutputValue));
        }

        public int i_AnalogOutputWriteMoreChannels(byte b_NbrOfChannels,
                                                   byte[] pb_ChannelList,
                                                   int[] pw_AnalogOutputValueList)
        {
            ushort[] us_AnalogOutputValueList = new ushort[b_NbrOfChannels];
            int i = 0;

            for (i = 0; i < b_NbrOfChannels; i++)
                us_AnalogOutputValueList[i] = (ushort)pw_AnalogOutputValueList[i];

            return (i_PCI3009_AnalogOutputWriteMoreChannels(dw_BoardHandle,
                                                            b_NbrOfChannels,
                                                            pb_ChannelList,
                                                            us_AnalogOutputValueList));
        }

        #endregion

        #region "Analog input initialisation functions"

        public int i_AnalogInputInitChannel(byte b_Channel,
                                            byte b_Gain,
                                            byte b_Polarity)
        {
            return (i_PCI3009_AnalogInputInitChannel(dw_BoardHandle, b_Channel, b_Gain, b_Polarity));
        }

        public int i_AnalogInputInitMode(byte b_SingelDiffMode)
        {
            return (i_PCI3009_AnalogInputInitMode(dw_BoardHandle, b_SingelDiffMode));
        }

        public int i_AnalogInputEnableDisableHardwareTrigger(byte b_HardwareTriggerAction,
                                                      byte b_HardwareTriggerLevel,
                                                      long dw_HardwareTriggerCount,
                                                      long dw_HardwareTriggerCycleCount)
        {
            uint ui_HardwareTriggerCount = (uint)dw_HardwareTriggerCount;
            uint ui_HardwareTriggerCycleCount = (uint)dw_HardwareTriggerCycleCount;

            return (i_PCI3009_AnalogInputEnableDisableHardwareTrigger(dw_BoardHandle,
                                                                      b_HardwareTriggerAction,
                                                                      b_HardwareTriggerLevel,
                                                                      ui_HardwareTriggerCount,
                                                                      ui_HardwareTriggerCycleCount));
        }

        public int i_AnalogInputGetHardwareTriggerStatus(ref byte pb_HardwareTriggerAction,
                                                  ref byte pb_HardwareTriggerStatus,
                                                  ref long pdw_HardwareTriggerCount,
                                                  ref byte pb_HardwareTriggerState)
        {
            uint ui_HardwareTriggerCount = 0;
            int i_ReturnValue = 0;

            i_ReturnValue = i_PCI3009_AnalogInputGetHardwareTriggerStatus(dw_BoardHandle,
                                                                          ref pb_HardwareTriggerAction,
                                                                          ref pb_HardwareTriggerStatus,
                                                                          ref ui_HardwareTriggerCount,
                                                                          ref pb_HardwareTriggerState);
            pdw_HardwareTriggerCount = (long)ui_HardwareTriggerCount;

            return (i_ReturnValue);
        }

        #endregion

        #region "Analog input convert functions"

        public int i_AnalogInputConvertDigitalToRealAnalogValue(byte b_Channel,
                                                                long dw_DigitalValue,
                                                                ref double pd_AnalogValue)
        {
            uint ui_DigitalValue = (uint)dw_DigitalValue;

            return (i_PCI3009_AnalogInputConvertDigitalToRealAnalogValue(dw_BoardHandle,
                                                                         b_Channel,
                                                                         ui_DigitalValue,
                                                                         ref pd_AnalogValue));
        }

        public int i_AnalogInputConvertMoreDigitalToRealAnalogValues(byte b_NbrOfChannels,
                                                                     byte[] pb_ChannelList,
                                                                     long[] pdw_DigitalValue,
                                                                     double[] pd_AnalogValue)
        {
            uint[] pui_DigitalValue = new uint[b_NbrOfChannels];

            for (int i = 0; i < b_NbrOfChannels; i++)
            {
                pui_DigitalValue[i] = (uint)pdw_DigitalValue[i];
            }

            return (i_PCI3009_AnalogInputConvertMoreDigitalToRealAnalogValues(dw_BoardHandle,
                                                                              b_NbrOfChannels,
                                                                              pb_ChannelList,
                                                                              pui_DigitalValue,
                                                                              pd_AnalogValue));
        }

        public int i_AnalogInputConvertDMADigitalToRealAnalogValues(uint[] pdw_DigitalValue,
                                                                    double[] pd_AnalogValue)
        {
            return (i_PCI3009_AnalogInputConvertDMADigitalToRealAnalogValues(dw_BoardHandle,
                                                                             pdw_DigitalValue,
                                                                             pd_AnalogValue));
        }

        public int i_AnalogInputConvertDMADigitalToRealAnalogValuesNew(uint dw_Size,
                                                                    ushort[] pw_DigitalValue,
                                                                    double[] pd_AnalogValue)
        {
            return (i_PCI3009_AnalogInputConvertDMADigitalToRealAnalogValuesNew(dw_BoardHandle,
                                                                            dw_Size,
                                                                             pw_DigitalValue,
                                                                             pd_AnalogValue));
        }

        #endregion

        #region "Analog input single acquisition functions"

        public int i_AnalogInputRead1Channel(byte b_Channel,
                                             byte b_ConvertTimeUnit,
                                             int w_ConvertTime,
                                             byte b_Interrupt,
                                             ref long pdw_DigitalValue)
        {
            ushort i_ConvertTime = (ushort)w_ConvertTime;
            uint ui_DigitalValue = 0;
            int i_ReturnValue = 0;

            if ((i_ReturnValue = i_PCI3009_AnalogInputRead1Channel(dw_BoardHandle,
                                                                   b_Channel,
                                                                   b_ConvertTimeUnit,
                                                                   i_ConvertTime,
                                                                   b_Interrupt,
                                                                   ref ui_DigitalValue)) == 0)
            {
                pdw_DigitalValue = (long)ui_DigitalValue;
            }

            return (i_ReturnValue);
        }

        public int i_AnalogInputReadMoreChannels(byte b_NbrOfChannels,
                                                 byte[] pb_ChannelList,
                                                 byte b_ConvertTimeUnit,
                                                 int w_ConvertTime,
                                                 byte b_Interrupt,
                                                 long[] pdw_DigitalValue)
        {
            ushort i_ConvertTime = (ushort)w_ConvertTime;
            uint[] pui_DigitalValue = new uint[b_NbrOfChannels];
            int i_ReturnValue = 0;

            if ((i_ReturnValue = i_PCI3009_AnalogInputReadMoreChannels(dw_BoardHandle,
                                                          b_NbrOfChannels,
                                                          pb_ChannelList,
                                                          b_ConvertTimeUnit,
                                                          i_ConvertTime,
                                                          b_Interrupt,
                                                          pui_DigitalValue)) == 0)
            {
                for (int i = 0; i < b_NbrOfChannels; i++)
                {
                    pdw_DigitalValue[i] = (long)pui_DigitalValue[i];
                }
            }
            return (i_ReturnValue);
        }

        #endregion

        #region "Analog input sequence acquisition funtctions"

        public int i_AnalogInputInitSequence(byte b_NbrOfChannels,
                                             byte[] pb_ChannelList,
                                             byte b_ConvertTimeUnit,
                                             int w_ConvertTime,
                                             byte b_DelayMode,
                                             byte b_DelayTimeUnit,
                                             int w_DelayTime,
                                             long dw_NumberOfSequence,
                                             byte b_UseDMA,
                                             long dw_NumberOfSequenceForEachInterrupt)
        {
            ushort us_ConvertTime = (ushort)w_ConvertTime;
            ushort us_DelayTime = (ushort)w_DelayTime;
            uint ui_NumberOfSequence = (uint)dw_NumberOfSequence;
            uint ui_NumberOfSequenceForEachInterrupt = (uint)dw_NumberOfSequenceForEachInterrupt;

            return (i_PCI3009_AnalogInputInitSequence(dw_BoardHandle,
                                                      b_NbrOfChannels,
                                                      pb_ChannelList,
                                                      b_ConvertTimeUnit,
                                                      us_ConvertTime,
                                                      b_DelayMode,
                                                      b_DelayTimeUnit,
                                                      us_DelayTime,
                                                      ui_NumberOfSequence,
                                                      b_UseDMA,
                                                      ui_NumberOfSequenceForEachInterrupt));
        }

        public int i_AnalogInputReleaseSequence()
        {
            return (i_PCI3009_AnalogInputReleaseSequence(dw_BoardHandle));
        }

        public int i_AnalogInputStartSequence()
        {
            return (i_PCI3009_AnalogInputStartSequence(dw_BoardHandle));
        }

        public int i_AnalogInputStopSequence()
        {
            return (i_PCI3009_AnalogInputStopSequence(dw_BoardHandle));
        }

        #endregion

        #region "Analog input autorefresh acquisition functions"

        public int i_AnalogInputReadAutoRefreshValueAndCounter(long[] pdw_ReadValue,
                                                               ref long pdw_CycleIndex)
        {
            uint[] pui_ReadValue = new uint[16];
            int i_ReturnValue = 0;
            uint ui_CycleIndex = 0;

            if ((i_ReturnValue = i_PCI3009_AnalogInputReadAutoRefreshValueAndCounter(dw_BoardHandle,
                                                                                     pui_ReadValue,
                                                                                     ref ui_CycleIndex)) == 0)
            {
                pdw_CycleIndex = (long)ui_CycleIndex;

                for (int i = 0; i < 16; i++)
                {
                    pdw_ReadValue[i] = (long)pui_ReadValue[i];
                }
            }
            return (i_ReturnValue);
        }

        public int i_AnalogInputStartAutoRefresh(int w_ChannelMask,
                                                 byte b_ConvertTimeUnit,
                                                 int w_ConvertTime)
        {
            ushort us_ChannelMask = (ushort)w_ChannelMask;
            ushort us_ConvertTime = (ushort)w_ConvertTime;

            return (i_PCI3009_AnalogInputStartAutoRefresh(dw_BoardHandle,
                                                          us_ChannelMask,
                                                          b_ConvertTimeUnit,
                                                          us_ConvertTime));
        }

        public int i_AnalogInputStopAutoRefresh()
        {
            return (i_PCI3009_AnalogInputStopAutoRefresh(dw_BoardHandle));
        }

        #endregion

        #region "Analog input compare logic functions"

        public int i_AnalogInputEnableDisableCompareLogic(byte b_CompareLogicFlag)
        {
            return (i_PCI3009_AnalogInputEnableDisableCompareLogic(dw_BoardHandle, b_CompareLogicFlag));
        }

        public int i_AnalogInputGetAllCompareLogicStates(ref long pdw_CompareLogicStates)
        {
            int i_ReturnValue = 0;
            uint ui_CompareLogicStates = 0;

            if ((i_ReturnValue = i_PCI3009_AnalogInputGetAllCompareLogicStates(dw_BoardHandle, ref ui_CompareLogicStates)) == 0)
            {
                pdw_CompareLogicStates = (long)ui_CompareLogicStates;
            }

            return (i_ReturnValue);
        }

        public int i_AnalogInputGetAutoRefreshCompareLogicStates(byte b_Channel,
                                                                 ref cl_AutoRefreshDataPackage pcl_AutoRefreshDataPackage)
        {
            int i_ReturnValue = 0;
            str_AutoRefreshDataPackage s_AutoRefreshDataPackage = new str_AutoRefreshDataPackage();
            uint dw_StructSize = (uint)Marshal.SizeOf(s_AutoRefreshDataPackage);

            if ((i_ReturnValue = i_PCI3009_AnalogInputGetAutoRefreshCompareLogicStates(dw_BoardHandle,
                                                                          b_Channel,
                                                                          ref s_AutoRefreshDataPackage,
                                                                          dw_StructSize)) != 0)
            {
                pcl_AutoRefreshDataPackage.b_IsInFirstRange = 0;
                pcl_AutoRefreshDataPackage.b_IsInSecondRange = 0;
                pcl_AutoRefreshDataPackage.b_IsInThirdRange = 0;
                pcl_AutoRefreshDataPackage.b_Open = 0;
                pcl_AutoRefreshDataPackage.d_ChannelValue = 0;
                pcl_AutoRefreshDataPackage.dw_ChannelIncrementValue = 0;

                return (i_ReturnValue);
            }

            pcl_AutoRefreshDataPackage.b_IsInFirstRange = s_AutoRefreshDataPackage.s_DigitalizedValue.b_IsInFirstRange;
            pcl_AutoRefreshDataPackage.b_IsInSecondRange = s_AutoRefreshDataPackage.s_DigitalizedValue.b_IsInSecondRange;
            pcl_AutoRefreshDataPackage.b_IsInThirdRange = s_AutoRefreshDataPackage.s_DigitalizedValue.b_IsInThirdRange;
            pcl_AutoRefreshDataPackage.b_Open = s_AutoRefreshDataPackage.s_DigitalizedValue.b_Open;
            pcl_AutoRefreshDataPackage.d_ChannelValue = s_AutoRefreshDataPackage.d_ChannelValue;
            pcl_AutoRefreshDataPackage.dw_ChannelIncrementValue = s_AutoRefreshDataPackage.dw_ChannelIncrementValue;

            return (i_ReturnValue);
        }

        public int i_AnalogInputGetSingleCompareLogicStates(byte b_Channel,
                                                            ref byte pb_CompareLogicStates)
        {
            return (i_PCI3009_AnalogInputGetSingleCompareLogicStates(dw_BoardHandle,
                                                                     b_Channel,
                                                                     ref pb_CompareLogicStates));
        }

        public int i_AnalogInputInitCompareLogic(byte b_Channel,
                                                 byte b_CompareLevelAMode,
                                                 double d_CompareLevelAValue,
                                                 byte b_CompareLevelBMode,
                                                 double d_CompareLevelBValue)
        {
            return (i_PCI3009_AnalogInputInitCompareLogic(dw_BoardHandle,
                                                          b_Channel,
                                                          b_CompareLevelAMode,
                                                          d_CompareLevelAValue,
                                                          b_CompareLevelBMode,
                                                          d_CompareLevelBValue));
        }

        #region Programming logic

        public int i_PLFimrwareConfigure(string pc_rbfPath)
        {
            return (i_PCI3009_PLFimrwareConfigure(dw_BoardHandle, pc_rbfPath.ToCharArray()));
        }

        public int i_PLFimrwareRevision(ref long pdw_Revision)
        {
            uint ui_Revision = 0;
            int ret = i_PCI3009_PLFimrwareRevision(dw_BoardHandle, ref ui_Revision);

            pdw_Revision = ui_Revision;

            return ret;
        }

        public int i_PLFimrwareFunctionality(ref long pdw_Functionality)
        {
            uint ui_Functionality = 0;
            int ret = i_PCI3009_PLFimrwareFunctionality(dw_BoardHandle, ref ui_Functionality);

            pdw_Functionality = ui_Functionality;

            return ret;
        }

        #endregion

        #region "Digital input functions"

        public int i_DigitalInputRead1(byte b_Channel,
                                       ref byte pb_Status)
        {
            return (i_PCI3009_DigitalInputRead1(dw_BoardHandle, b_Channel, ref pb_Status));
        }

        public int i_DigitalInputReadAll(ref byte pb_Status)
        {
            return (i_PCI3009_DigitalInputReadAll(dw_BoardHandle, ref pb_Status));
        }

        #endregion

        #region "Digital output functions"

        public int i_DigitalOutputReadAll(ref byte pb_Status)
        {
            return (i_PCI3009_DigitalOutputReadAll(dw_BoardHandle, ref pb_Status));
        }

        public int i_DigitalOutputWriteAll(byte b_WriteValue)
        {
            return (i_PCI3009_DigitalOutputWriteAll(dw_BoardHandle, b_WriteValue));
        }

        public int i_DigitalInputEnableDisableFilter(byte b_Filter, long dw_FilterTime)
        {
            return (i_PCI3009_DigitalInputEnableDisableFilter(dw_BoardHandle,
                                                              b_Filter,
                                                              (uint)dw_FilterTime));
        }

        #endregion

        #region DIGIO

        public int i_InitDigitalIO(byte b_ChannelAMode, byte b_ChannelBMode)
        {
            return (i_PCI3009_InitDigitalIO(dw_BoardHandle, b_ChannelAMode, b_ChannelBMode));
        }

        public int i_ReadDigitalIOChlValue(byte b_InputChannel, ref byte pb_ChannelStatus)
        {
            return (i_PCI3009_ReadDigitalIOChlValue(dw_BoardHandle, b_InputChannel, ref pb_ChannelStatus));
        }

        public int i_ReadDigitalIOPortValue(ref byte pb_PortValue)
        {
            return (i_PCI3009_ReadDigitalIOPortValue(dw_BoardHandle, ref pb_PortValue));
        }

        public int i_SetDigitalIOMemoryOn()
        {
            return (i_PCI3009_SetDigitalIOMemoryOn(dw_BoardHandle));
        }

        public int i_SetDigitalIOMemoryOff()
        {
            return (i_PCI3009_SetDigitalIOMemoryOff(dw_BoardHandle));
        }

        public int i_SetDigitalIOChlOn(byte b_OutputChannel)
        {
            return (i_PCI3009_SetDigitalIOChlOn(dw_BoardHandle, b_OutputChannel));
        }

        public int i_SetDigitalIOChlOff(byte b_OutputChannel)
        {
            return (i_PCI3009_SetDigitalIOChlOff(dw_BoardHandle, b_OutputChannel));
        }

        public int i_SetDigitalIOPortOn(byte b_PortValue)
        {
            return (i_PCI3009_SetDigitalIOPortOn(dw_BoardHandle, b_PortValue));
        }

        public int i_SetDigitalIOPortOff(byte b_PortValue)
        {
            return (i_PCI3009_SetDigitalIOPortOff(dw_BoardHandle, b_PortValue));
        }

        #endregion

        #region INC_CPT

        public int i_InitCounter(byte b_CounterRange,
                                 byte b_FirstCounterModus,
                                 byte b_FirstCounterOption,
                                 byte b_SecondCounterModus,
                                 byte b_SecondCounterOption)
        {
            return (i_PCI3009_InitCounter(dw_BoardHandle,
                                          b_CounterRange,
                                          b_FirstCounterModus,
                                          b_FirstCounterOption,
                                          b_SecondCounterModus,
                                          b_SecondCounterOption));
        }

        public int i_CounterAutoTest(ref byte pb_TestStatus)
        {
            return (i_PCI3009_CounterAutoTest(dw_BoardHandle, ref pb_TestStatus));
        }

        public int i_SetInputFilter(byte b_Filter)
        {
            return (i_PCI3009_SetInputFilter(dw_BoardHandle, b_Filter));
        }

        public int i_ClearCounterValue()
        {
            return (i_PCI3009_ClearCounterValue(dw_BoardHandle));
        }

        /* Counter index functions */

        public int i_InitIndex(byte b_ReferenceAction,
                               byte b_IndexOperation,
                               byte b_AutoMode,
                               byte b_InterruptEnable)
        {
            return (i_PCI3009_InitIndex(dw_BoardHandle,
                                        b_ReferenceAction,
                                        b_IndexOperation,
                                        b_AutoMode,
                                        b_InterruptEnable));
        }

        public int i_EnableIndex()
        {
            return (i_PCI3009_EnableIndex(dw_BoardHandle));
        }

        public int i_DisableIndex()
        {
            return (i_PCI3009_DisableIndex(dw_BoardHandle));
        }

        public int i_GetIndexStatus(ref byte pb_IndexStatus)
        {
            return (i_PCI3009_GetIndexStatus(dw_BoardHandle, ref pb_IndexStatus));
        }

        public int i_SetIndexAndReferenceSource(byte b_SourceSelection)
        {
            return (i_PCI3009_SetIndexAndReferenceSource(dw_BoardHandle, b_SourceSelection));
        }

        /* Reference */

        public int i_InitReference(byte b_ReferenceLevel)
        {
            return (i_PCI3009_InitReference(dw_BoardHandle, b_ReferenceLevel));
        }

        public int i_GetReferenceStatus(ref byte pb_ReferenceStatus)
        {
            return (i_PCI3009_GetReferenceStatus(dw_BoardHandle, ref pb_ReferenceStatus));
        }

        /* Incremental counter UAS,CBx,U/D#, external strobe functions */

        public int i_GetUASStatus(ref byte pb_UASStatus)
        {
            return (i_PCI3009_GetUASStatus(dw_BoardHandle, ref pb_UASStatus));
        }

        public int i_GetCBStatus(ref byte pb_CBStatus)
        {
            return (i_PCI3009_GetCBStatus(dw_BoardHandle, ref pb_CBStatus));
        }

        public int i_Get16BitCBStatus(ref byte pb_CBStatusCounter0, ref byte pb_CBStatusCounter1)
        {
            return (i_PCI3009_Get16BitCBStatus(dw_BoardHandle, ref pb_CBStatusCounter0, ref pb_CBStatusCounter1));
        }

        public int i_GetUDStatus(ref byte pb_UDStatus)
        {
            return (i_PCI3009_GetUDStatus(dw_BoardHandle, ref pb_UDStatus));
        }

        public int i_GetInterruptUDLatchedStatus(ref byte pb_UDStatus)
        {
            return (i_PCI3009_GetInterruptUDLatchedStatus(dw_BoardHandle, ref pb_UDStatus));
        }

        /* Incremental counter frequency measurement functions */

        public int i_InitFrequencyMeasurement(byte b_TimingUnity,
                                              long dw_TimingInterval,
                                              ref long pdw_RealTimingInterval)
        {
            uint ui_TimingInterval = (uint)dw_TimingInterval;
            uint ui_RealTimingInterval = 0;
            int i_ReturnValue = 0;

            i_ReturnValue = i_PCI3009_InitFrequencyMeasurement(dw_BoardHandle, b_TimingUnity, ui_TimingInterval, ref ui_RealTimingInterval);

            pdw_RealTimingInterval = ui_RealTimingInterval;

            return (i_ReturnValue);
        }

        public int i_EnableFrequencyMeasurement(byte b_InterruptEnable)
        {
            return (i_PCI3009_EnableFrequencyMeasurement(dw_BoardHandle, b_InterruptEnable));
        }

        public int i_DisableFrequencyMeasurement()
        {
            return (i_PCI3009_DisableFrequencyMeasurement(dw_BoardHandle));
        }

        public int i_ReadFrequencyMeasurement(ref byte pb_Status,
                                              ref byte pb_UDStatus,
                                              ref long pdw_ReadValue)
        {
            uint ui_ReadValue = (uint)pdw_ReadValue;
            int i_ReturnValue = 0;

            i_ReturnValue = i_PCI3009_ReadFrequencyMeasurement(dw_BoardHandle, ref pb_Status, ref pb_UDStatus, ref ui_ReadValue);

            pdw_ReadValue = ui_ReadValue;

            return (i_ReturnValue);
        }

        /* Incremental counter digital output functions */

        public int i_SetDigitalChlOn(byte b_InterruptEnable)
        {
            return (i_PCI3009_SetDigitalChlOn(dw_BoardHandle));
        }

        public int i_SetDigitalChlOff(byte b_InterruptEnable)
        {
            return (i_PCI3009_SetDigitalChlOff(dw_BoardHandle));
        }

        /* Read counter functions */

        public int i_LatchCounter(byte b_LatchReg)
        {
            return (i_PCI3009_LatchCounter(dw_BoardHandle, b_LatchReg));
        }

        public int i_ReadLatchRegisterStatus(byte b_LatchReg, ref byte pb_LatchStatus)
        {
            int i_ReturnValue = i_PCI3009_ReadLatchRegisterStatus(dw_BoardHandle, b_LatchReg, ref pb_LatchStatus);

            return (i_ReturnValue);
        }

        public int i_ReadLatchRegisterValue(byte b_LatchReg, ref long pdw_LatchValue)
        {
            int i_ReturnValue = 0;
            uint ui_LatchValue = 0;

            i_ReturnValue = i_PCI3009_ReadLatchRegisterValue(dw_BoardHandle, b_LatchReg, ref ui_LatchValue);

            pdw_LatchValue = (long)ui_LatchValue;

            return (i_ReturnValue);
        }

        public int i_EnableLatchInterrupt()
        {
            return (i_PCI3009_EnableLatchInterrupt(dw_BoardHandle));
        }

        public int i_DisableLatchInterrupt()
        {
            return (i_PCI3009_DisableLatchInterrupt(dw_BoardHandle));
        }

        public int i_Read16BitCounterValue(byte b_SelectedCounter, ref long pw_CounterValue)
        {
            int i_ReturnValue = 0;
            uint ui_CounterValue = 0;

            i_ReturnValue = i_PCI3009_Read16BitCounterValue(dw_BoardHandle, b_SelectedCounter, ref ui_CounterValue);

            pw_CounterValue = (long)ui_CounterValue;

            return (i_ReturnValue);
        }

        public int i_Read32BitCounterValue(ref long pdw_CounterValue)
        {
            int i_ReturnValue = 0;
            uint ui_CounterValue = 0;

            i_ReturnValue = i_PCI3009_Read32BitCounterValue(dw_BoardHandle, ref ui_CounterValue);

            pdw_CounterValue = (long)ui_CounterValue;

            return (i_ReturnValue);
        }

        /* Write counter functions */

        public int i_Write16BitCounterValue(byte b_SelectedCounter, int w_WriteValue)
        {
            ushort ui_WriteValue = (ushort)w_WriteValue;

            return (i_PCI3009_Write16BitCounterValue(dw_BoardHandle, b_SelectedCounter, ui_WriteValue));
        }

        public int i_Write32BitCounterValue(long pdw_WriteValue)
        {
            uint ui_WriteValue = (uint)pdw_WriteValue;

            return (i_PCI3009_Write32BitCounterValue(dw_BoardHandle, ui_WriteValue));
        }

        /* UAS,CBx,U/D# function */

        public int i_InitExternalStrobe(byte b_ExternalStrobe, byte b_ExternalStrobeLevel)
        {
            return (i_PCI3009_InitExternalStrobe(dw_BoardHandle, b_ExternalStrobe, b_ExternalStrobeLevel));
        }

        /* Compare functions */

        public int i_InitCompareLogic(long dw_CompareValue)
        {
            uint ui_CompareValue = (uint)dw_CompareValue;

            return (i_PCI3009_InitCompareLogic(dw_BoardHandle, ui_CompareValue));
        }

        public int i_EnableCompareLogic()
        {
            return (i_PCI3009_EnableCompareLogic(dw_BoardHandle));
        }

        public int i_DisableCompareLogic()
        {
            return (i_PCI3009_DisableCompareLogic(dw_BoardHandle));
        }

        #endregion

        #endregion
        #endregion

        #region "Constructor and destructor"

        public cl_PCI3009()
        {
            dw_BoardHandle = new uint();
            const int _MCW_EW = 0x8001F;
            const int _EM_INVALID = 0x10;

            b_EventHandlerInitialised = 0;

            _controlfp(_MCW_EW, _EM_INVALID);
        }

        ~cl_PCI3009()
        {
        }

        #endregion

    }
}
