using System;
using System.Collections.Generic;
using System.Text;
using PCI3009_NET;
using Utils;

namespace AutoRefresh
{
    class Program
    {
        static void Main(string[] args)
        {
            int  returnValue = 0;
            long dw_CycleIndex = 0;
            PCI3009_NET.cl_PCI3009.cl_AutoRefreshDataPackage s_AutoRefreshDataPackage = new PCI3009_NET.cl_PCI3009.cl_AutoRefreshDataPackage();
            cl_PCI3009 PCI3009 = new cl_PCI3009();

            Console.Write("\nAutorefresh sample.");

            if (!Utils.Utils.openBoard(PCI3009))
            {
                Console.ReadKey();
                return;
            }

            try
            {
                int channelMask = 0;
                List<byte> channelList = new List<byte>();
                long[] pdw_ReadValue = new long[16]; /* The size of this variable must always be 16 with autorefresh! */
                byte channelCount = Utils.Utils.getChannelCount();
                double[] analogValue = new double[channelCount];

                returnValue = PCI3009.i_AnalogInputInitMode(Utils.Utils.getAnalogInputMode());
                if (returnValue == -1)
                    Console.Write("\ni_AnalogInputInitMode: The handle parameter of the board is wrong");
                else if (returnValue == -2)
                    Console.Write("\ni_AnalogInputInitMode: Wrong single/diff mode");
                else if (returnValue == -3)
                    Console.Write("\ni_AnalogInputInitMode: A conversion is already started");
                else if (returnValue != 0)
                    Console.Write("\ni_AnalogInputInitChannel: Error = {0}", returnValue);

                Console.Write("\ni_AnalogInputInitMode OK");

                for (byte i = 0; i < channelCount; i++)
                {
                    byte channel = 0;
                    Utils.Utils.getParameter("\nChoose channel to use: \n", ref channel, (byte)0, (byte)15);
                    if (channelList.Contains(channel))
                    {
                        Console.WriteLine("\nChannel already selected");
                        continue;
                    }

                    returnValue = PCI3009.i_AnalogInputInitChannel(channel,
                        Utils.Utils.getAnalogInputGain(channel),
                        Utils.Utils.getAnalogInputPolarity(channel));
                    if (returnValue == -1)
                        throw new Exception("\ni_AnalogInputInitChannel: The handle parameter of the board is wrong");
                    else if (returnValue == -2)
                        throw new Exception("\ni_AnalogInputInitChannel: Wrong channel number");
                    else if (returnValue == -3)
                        throw new Exception("\ni_AnalogInputInitChannel: Wrong gain");
                    else if (returnValue == -4)
                        throw new Exception("\ni_AnalogInputInitChannel: Wrong polarity");
                    else if (returnValue == -5)
                        throw new Exception("\ni_AnalogInputInitChannel: A conversion is already started");
                    else if (returnValue != 0)
                        throw new Exception("\ni_AnalogInputInitChannel: Error = " + returnValue.ToString());

                    Console.WriteLine("\ni_AnalogInputInitChannel channel " + channel.ToString() + " OK");

                    channelMask |= (1 << channel);
                    channelList.Add(channel);
                }
                
                returnValue = PCI3009.i_AnalogInputStartAutoRefresh(channelMask, Utils.Utils.getConvertTimeUnit(), Utils.Utils.getConvertTimeValue());
                if (returnValue == -1)
                    throw new Exception("\ni_AnalogInputStartAutoRefresh: The handle parameter of the board is wrong");
                else if (returnValue == -2)
                    throw new Exception("\ni_AnalogInputStartAutoRefresh: Wrong channels mask selection");
                else if (returnValue == -3)
                    throw new Exception("\ni_AnalogInputStartAutoRefresh: Wrong channel 0 mask");
                else if (returnValue == -4)
                    throw new Exception("\ni_AnalogInputStartAutoRefresh: Channel 0 not initialised");
                else if (returnValue != 0)
                    throw new Exception("\ni_AnalogInputStartAutoRefresh: Error = " + returnValue.ToString());

                Console.WriteLine("\ni_AnalogInputStartAutoRefresh OK");

                int cursorLeft = Console.CursorLeft;
                int cursorTop = Console.CursorTop;
                do
                {
                    returnValue = PCI3009.i_AnalogInputReadAutoRefreshValueAndCounter(pdw_ReadValue, ref dw_CycleIndex);
                    if (returnValue == -1)
                        Console.Write("\ni_AnalogInputReadAutoRefreshValueAndCounter: The handle parameter of the board is wrong");
                    else if (returnValue == -2)
                        Console.Write("\ni_AnalogInputReadAutoRefreshValueAndCounter: No conversion started");
                    else if (returnValue != 0)
                        Console.Write("\ni_AnalogInputReadAutoRefreshValueAndCounter: Error = " + returnValue.ToString());

                    returnValue = PCI3009.i_AnalogInputConvertMoreDigitalToRealAnalogValues(channelCount, channelList.ToArray(), pdw_ReadValue, analogValue);
                    if (returnValue == -1)
                        throw new Exception("\ni_AnalogInputConvertMoreDigitalToRealAnalogValues: The handle parameter of the board is wrong");
                    else if (returnValue == -2)
                        throw new Exception("\ni_AnalogInputConvertMoreDigitalToRealAnalogValues: Wrong channels number selection ");
                    else if (returnValue == -3)
                        throw new Exception("\ni_AnalogInputConvertMoreDigitalToRealAnalogValues: Wrong channel index 0 number");
                    else if (returnValue != 0)
                        throw new Exception("\ni_AnalogInputConvertMoreDigitalToRealAnalogValues: Error = " + returnValue);

                    Console.SetCursorPosition(cursorLeft, cursorTop);
                    Console.Write("\n\nCounter = {0}", dw_CycleIndex);
                    for (byte channel = 0; channel < channelCount; channel++)
                        Console.Write("\nChannel {0} = 0x{1} = {2}V", channelList[channel], pdw_ReadValue[channel], analogValue[channel].ToString("0.0000"));
                } while (!Console.KeyAvailable);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }

            returnValue = PCI3009.i_AnalogInputStopAutoRefresh();
            if (returnValue == 0)
                Console.Write("\ni_AnalogInputStopAutoRefresh OK");
            else if (returnValue == 1)
                Console.Write("\ni_AnalogInputStopAutoRefresh: The handle parameter of the board is wrong");
            else if (returnValue == -2)
                Console.Write("\ni_AnalogInputStopAutoRefresh: No conversion started");
            else if (returnValue != 0)
                Console.Write("\ni_AnalogInputStopAutoRefresh: Error = {0}", returnValue);

            returnValue = PCI3009.i_CloseBoard();
            if (returnValue == 0)
                Console.Write("\ni_CloseBoard OK");
            else if (returnValue == -1)
                Console.Write("\ni_CloseBoard: The handle parameter of the board is wrong");
            else
                Console.Write("\ni_CloseBoard error = " + returnValue.ToString());

            Console.WriteLine("\nPress any key");
            Console.ReadKey();
        }
    }
}
