using System;
using System.Collections.Generic;
using System.Text;
using PCI3009_NET;

namespace Read1AnalogInput
{
    class Program
    {
        public static int    _interruptCount;
        public static byte   _channel;
        public static double _analogValue;
        public static cl_PCI3009 _PCI3009;
        public static long _digitalValue = 0;

        public static void v_Interrupt(object sender, PCI3009_NET.cl_PCI3009.cl_EventHandlerArgs e)
        {
            int returnValue = _PCI3009.i_AnalogInputConvertDigitalToRealAnalogValue(_channel, e.dw_Value[0], ref _analogValue);
            if (returnValue != 0)
                Console.WriteLine("\ni_AnalogInputConvertMoreDigitalToRealAnalogValues: Error = " + returnValue.ToString());

            _interruptCount = _interruptCount + 1;
        }

        static void Main(string[] args)
        {
            int    returnValue = 0;
            byte   interrupt = 0;
            int    interruptCountMem = 0;
            _PCI3009 = new cl_PCI3009();

            Console.Write("\nRead a single channel.");
            if (!Utils.Utils.openBoard(_PCI3009))
            {
                Console.ReadKey();
                return;
            }

            try
            {
                returnValue = _PCI3009.i_AnalogInputInitMode(Utils.Utils.getAnalogInputMode());
                if (returnValue == -1)
                    throw new Exception("\ni_AnalogInputInitMode: The handle parameter of the board is wrong");
                else if (returnValue == -2)
                    throw new Exception("\ni_AnalogInputInitMode: Wrong single/diff mode");
                else if (returnValue == -3)
                    throw new Exception("\ni_AnalogInputInitMode: A conversion is already started");
                else if (returnValue != 0)
                    throw new Exception("\ni_AnalogInputInitChannel: Error = " + returnValue.ToString());
                Console.WriteLine("\ni_AnalogInputInitMode OK");

                _channel = 0;
                Utils.Utils.getParameter("\nEnter channel to read:\n", ref _channel, (byte)0, (byte)15);
                returnValue = _PCI3009.i_AnalogInputInitChannel(_channel,
                    Utils.Utils.getAnalogInputGain(_channel),
                    Utils.Utils.getAnalogInputPolarity(_channel));
                if (returnValue != 0)
                    throw new Exception("\ni_AnalogInputInitChannel: Error = " + returnValue.ToString());

                Console.WriteLine("\ni_AnalogInputInitChannel channel " + _channel.ToString() + " OK");
                byte convertTimeUnit = Utils.Utils.getConvertTimeUnit();
                int convertTimeValue = Utils.Utils.getConvertTimeValue();

                interrupt = Utils.Utils.getInterruptMode();
                if (interrupt == 1)
                {
                    returnValue = _PCI3009.i_SetBoardIntRoutine();
                    if (returnValue != 0)
                        throw new Exception("\ni_SetBoardIntRoutine: Error = " + returnValue.ToString());
                    _PCI3009.e_EventHandlerDelegate += new PCI3009_NET.cl_PCI3009.v_EventHandlerDelegate(v_Interrupt);
                    _PCI3009.b_EventHandlerInitialised = 1;
                }

                int cursorLeft = Console.CursorLeft;
                int cursorTop = Console.CursorTop;
                do
                {
                    returnValue = _PCI3009.i_AnalogInputRead1Channel(_channel, convertTimeUnit, convertTimeValue, interrupt, ref _digitalValue);
                    if (returnValue == -1)
                        throw new Exception("\ni_AnalogInputRead1Channel: The handle parameter of the board is wrong");
                    else if (returnValue == -2)
                        throw new Exception("\ni_AnalogInputRead1Channel: Wrong channel number");
                    else if (returnValue == -3)
                        throw new Exception("\ni_AnalogInputRead1Channel: Channel not initialised");
                    else if (returnValue == -4)
                        throw new Exception("\ni_AnalogInputRead1Channel: Wrong convert time unit");
                    else if (returnValue == -5)
                        throw new Exception("\ni_AnalogInputRead1Channel: Wrong convert time");
                    else if (returnValue == -6)
                        throw new Exception("\ni_AnalogInputRead1Channel: Wrong interrupt flag");
                    else if (returnValue == -7)
                        throw new Exception("\ni_AnalogInputRead1Channel: Interrupt routine not installed see function \"i_PCI3009_SetBoardIntRoutine\"");
                    else if (returnValue == -8)
                        throw new Exception("\ni_AnalogInputRead1Channel: A conversion is already started");
                    else if (returnValue != 0)
                        throw new Exception("\ni_AnalogInputRead1Channel: Error = " + returnValue.ToString());

                    if (interrupt != 0)
                    {
                        while (_interruptCount == interruptCountMem)
                            System.Threading.Thread.Sleep(100);

                        _interruptCount = interruptCountMem;
                    }
                    else
                    {
                        returnValue = _PCI3009.i_AnalogInputConvertDigitalToRealAnalogValue(_channel, _digitalValue, ref _analogValue);
                        if (returnValue == -1)
                            throw new Exception("\ni_AnalogInputConvertMoreDigitalToRealAnalogValues: The handle parameter of the board is wrong");
                        else if (returnValue == -2)
                            throw new Exception("\ni_AnalogInputConvertMoreDigitalToRealAnalogValues: Wrong channels number selection ");
                        else if (returnValue == -3)
                            throw new Exception("\ni_AnalogInputConvertMoreDigitalToRealAnalogValues: Wrong channel index 0 number");
                        else if (returnValue != 0)
                            throw new Exception("\ni_AnalogInputConvertMoreDigitalToRealAnalogValues: Error = " + returnValue.ToString());
                    }

                    Console.SetCursorPosition(cursorLeft, cursorTop);
                    Console.Write("\nChannel {0} = 0x{1} = {2}              ", _channel, _digitalValue, _analogValue.ToString("0.0000"));
                } while (!Console.KeyAvailable);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }

            if (interrupt != 0)
            {
                returnValue = _PCI3009.i_ResetBoardIntRoutine();
                if (returnValue != 0)
                    Console.Write("\ni_ResetBoardIntRoutine error = " + returnValue.ToString());
                else
                    Console.Write("\ni_ResetBoardIntRoutine OK");
            }

            returnValue = _PCI3009.i_CloseBoard();
            if (returnValue == 0)
                Console.Write("\ni_CloseBoard OK");
            else if (returnValue == -1)
                Console.Write("\ni_CloseBoard: The handle parameter of the board is wrong");
            else
                Console.Write("\ni_CloseBoard error = " + returnValue.ToString());

            Console.WriteLine("\nPress any key");
            Console.ReadKey();
        }
    }
}
