using System;
using System.Collections.Generic;
using System.Text;
using PCI3009_NET;

namespace ReadMoreAnalogInputs
{
    class Program
    {
        public static int _interruptCount = 0;
        public static byte[] _channelList;
        public static double[] _analogValues;
        public static cl_PCI3009 _PCI3009;

        public static void v_Interrupt(object sender, PCI3009_NET.cl_PCI3009.cl_EventHandlerArgs e)
        {
            if (_channelList == null)
                Console.WriteLine("\nError channel list is NULL");

            int returnValue = _PCI3009.i_AnalogInputConvertMoreDigitalToRealAnalogValues((byte)e.dw_NumberOfValue, _channelList, e.dw_Value, _analogValues);
            if (returnValue != 0)
                throw new Exception("\ni_AnalogInputConvertMoreDigitalToRealAnalogValues: Error = " + returnValue.ToString());

            _interruptCount = _interruptCount + 1;
        }

        static void Main(string[] args)
        {
            int  returnValue = 0;
            byte channelCount = 0;
            int  interruptCountMem = 0;
            byte interrupt = 0;

            _PCI3009 = new cl_PCI3009();
            
            Console.Write("\nRead more analog inputs.");

            if (!Utils.Utils.openBoard(_PCI3009))
            {
                Console.ReadKey();
                return;
            }

            try
            {
                channelCount = Utils.Utils.getChannelCount();
                _channelList = new byte[channelCount];
                long[] digitalValues = new long[channelCount];
                _analogValues = new double[channelCount];

                returnValue = _PCI3009.i_AnalogInputInitMode(Utils.Utils.getAnalogInputMode());
                if (returnValue == -1)
                    throw new Exception("\ni_AnalogInputInitMode: The handle parameter of the board is wrong");
                else if (returnValue == -2)
                    throw new Exception("\ni_AnalogInputInitMode: Wrong single/diff mode");
                else if (returnValue == -3)
                    throw new Exception("\ni_AnalogInputInitMode: A conversion is already started");
                else if (returnValue != 0)
                    throw new Exception("\ni_AnalogInputInitChannel: Error = " + returnValue.ToString());

                Console.Write("\ni_AnalogInputInitMode OK");

                for (byte channel = 0; channel < channelCount; channel++)
                {
                    returnValue = _PCI3009.i_AnalogInputInitChannel(channel,
                        PCI3009_NET.cl_PCI3009.PCI3009_1_GAIN,
                        Utils.Utils.getAnalogInputPolarity(channel));
                    if (returnValue == -1)
                        throw new Exception("\ni_AnalogInputInitChannel: The handle parameter of the board is wrong");
                    else if (returnValue == -2)
                        throw new Exception("\ni_AnalogInputInitChannel: Wrong channel number");
                    else if (returnValue == -3)
                        throw new Exception("\ni_AnalogInputInitChannel: Wrong gain");
                    else if (returnValue == -4)
                        throw new Exception("\ni_AnalogInputInitChannel: Wrong polarity");
                    else if (returnValue == -5)
                        throw new Exception("\ni_AnalogInputInitChannel: A conversion is already started");
                    else if (returnValue != 0)
                        throw new Exception("\ni_AnalogInputInitChannel: Error = " + returnValue.ToString());

                    Console.WriteLine("\ni_AnalogInputInitChannel channel " + channel.ToString() + " OK");

                    _channelList[channel] = channel;
                }

                
                byte convertTimeUnit = Utils.Utils.getConvertTimeUnit();
                int convertTimeValue = Utils.Utils.getConvertTimeValue();
                interrupt = Utils.Utils.getInterruptMode();
                if (interrupt == 1)
                {
                    returnValue = _PCI3009.i_SetBoardIntRoutine();
                    if (returnValue != 0)
                        throw new Exception("\ni_SetBoardIntRoutine: Error = " + returnValue.ToString());
                    _PCI3009.e_EventHandlerDelegate += new PCI3009_NET.cl_PCI3009.v_EventHandlerDelegate(v_Interrupt);
                    _PCI3009.b_EventHandlerInitialised = 1;
                }
                int cursorLeft = Console.CursorLeft;
                int cursorTop = Console.CursorTop;
                do
                {
                    returnValue = _PCI3009.i_AnalogInputReadMoreChannels(channelCount,
                                                                        _channelList,
                                                                        convertTimeUnit,
                                                                        convertTimeValue,
                                                                        interrupt,
                                                                        digitalValues);

                    if (returnValue == -1)
                        throw new Exception("\ni_AnalogInputReadMoreChannels: The handle parameter of the board is wrong");
                    else if (returnValue == -2)
                        throw new Exception("\ni_AnalogInputReadMoreChannels: Wrong channels number selection ");
                    else if (returnValue == -3)
                        throw new Exception("\ni_AnalogInputReadMoreChannels: Wrong channel index 0 number");
                    else if (returnValue != 0)
                        throw new Exception("\ni_AnalogInputReadMoreChannels: Error = " + returnValue.ToString());

                    if (interrupt != 0)
                    {
                        while (_interruptCount == interruptCountMem)
                            System.Threading.Thread.Sleep(100);

                        _interruptCount = interruptCountMem;
                    }
                    else
                    {
                        returnValue = _PCI3009.i_AnalogInputConvertMoreDigitalToRealAnalogValues(channelCount, _channelList, digitalValues, _analogValues);
                        if (returnValue == -1)
                            throw new Exception("\ni_AnalogInputConvertMoreDigitalToRealAnalogValues: The handle parameter of the board is wrong");
                        else if (returnValue == -2)
                            throw new Exception("\ni_AnalogInputConvertMoreDigitalToRealAnalogValues: Wrong channels number selection ");
                        else if (returnValue == -3)
                            throw new Exception("\ni_AnalogInputConvertMoreDigitalToRealAnalogValues: Wrong channel index 0 number");
                        else if (returnValue != 0)
                            throw new Exception("\ni_AnalogInputConvertMoreDigitalToRealAnalogValues: Error = " + returnValue.ToString());
                    }
                    Console.SetCursorPosition(cursorLeft, cursorTop);
                    for (byte channel = 0; channel < channelCount; channel++)
                        Console.Write("\nChannel {0} = {1}", _channelList[channel], _analogValues[channel].ToString("0.0000"));
                    
                    System.Threading.Thread.Sleep(100);
                }
                while (!Console.KeyAvailable);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }

            if (interrupt != 0)
            {
                returnValue = _PCI3009.i_ResetBoardIntRoutine();
                if (returnValue != 0)
                    Console.Write("\ni_ResetBoardIntRoutine error = " + returnValue.ToString());
                else
                    Console.Write("\ni_ResetBoardIntRoutine OK");
            }

            returnValue = _PCI3009.i_CloseBoard();
            if (returnValue == 0)
                Console.Write("\ni_CloseBoard OK");
            else if (returnValue == -1)
                Console.Write("\ni_CloseBoard: The handle parameter of the board is wrong");
            else
                Console.Write("\ni_CloseBoard error = " + returnValue.ToString());
            
            Console.WriteLine("\nPress any key to leave the sample");
            Console.ReadKey();
        }
    }
}
