using System;
using System.IO;
using System.Collections.Generic;
using System.Text;
using PCI3009_NET;

/* Read analog input with sequence. */

namespace Sequence
{
    class Program
    {
        static cl_PCI3009 PCI3009 = new cl_PCI3009();
        static Queue<ushort[]> interruptValues = new Queue<ushort[]>();
        static byte b_Stop = 0;
        static uint numberOfValuesPerInterrupt = 0;

        public Program(PCI3009_NET.cl_PCI3009 PCI3009)
        {
            PCI3009.e_EventHandlerDelegate += new PCI3009_NET.cl_PCI3009.v_EventHandlerDelegate(v_Interrupt);
            PCI3009.b_EventHandlerInitialised = 1;
        }

        private static void v_Interrupt(object sender, PCI3009_NET.cl_PCI3009.cl_EventHandlerArgs e)
        {
            /* FIFO overflow */
            if (e.dw_InterruptSrc == 0x5)
            {
                /* Driver FIFO overflow */
                if (e.dw_InterruptMask == 0x1)
                {
                    b_Stop = 1;
                    goto end;
                }        
            }

            /* DMA interrupt */
            if (e.dw_InterruptSrc == 0x0)
            {
                /* PLD FIFO overflow */
                if (e.dw_InterruptMask == 0x8)
                {
                    b_Stop = 2;
                    goto end;
                }
                else if (e.dw_InterruptMask == 0x4)
                {
                    uint numberOfBytePerInterrupt = (numberOfValuesPerInterrupt * sizeof(ushort));
                    ushort[] data = new ushort[numberOfValuesPerInterrupt];

                    int size = PCI3009.ReadSequenceData(numberOfBytePerInterrupt, data);
                    if (size < numberOfBytePerInterrupt)
                    {
                        b_Stop = 3;
                        goto end;
                    }
                    else
                        interruptValues.Enqueue(data);
                }
            }
            
            end:
                if (b_Stop != 0)
                    PCI3009.i_AnalogInputStopSequence();    
        }

        private static void v_GetAndSetUserSequenceConfiguration(ref byte pb_ConvertTimeUnit,
                                           ref UInt16 pw_ConvertTime,
                                           ref byte pb_DelayMode,
                                           ref byte pb_DelayTimeUnit,
                                           ref UInt16 pw_DelayTime,
                                           ref UInt32 pdw_NumberOfSequence,
                                           ref byte pb_UseDMA,
                                           ref UInt32 pdw_NumberOfSequenceForEachInterrupt,
                                           ref byte pb_HardwareTriggerAction,
                                           ref byte pb_HardwareTriggerLevel,
                                           ref UInt32 pdw_HardwareTriggerCount,
                                           ref UInt32 pdw_HardwareTriggerCycleCount)
        {
            /***********************
            * Variable declaration 
            ************************/
            string[] pc_DelayMode = { "Not used", "Delay mode 1", "Delay mode 2" };
            string[] pc_TimeUnit = { "us", "ms" };
            string[] pc_HardwareTriggerAction = { "No trigger", "One shot acquisition", "Series of acquisitions" };
            string[] pc_HardwareTriggerLevel = { "Low", "High", "Low and high" };
            int i_Tmp = 0;
            UInt32 l_Tmp = 0;
            UInt32 dw_MinConvertTime = 0;
            string line;

            Console.Write("\nSelect the convert time unit (0: us / 1: ms): ");

            do
            {
                line = Console.ReadLine();
                if (!int.TryParse(line, out i_Tmp))
                    i_Tmp = 2;
            } while ((i_Tmp != 0) && (i_Tmp != 1));

            Console.Write(pc_TimeUnit[i_Tmp]);

            switch (i_Tmp)
            {
                case 0:
                    pb_ConvertTimeUnit = PCI3009_NET.cl_PCI3009.PCI3009_MICRO_SECOND;
                    dw_MinConvertTime = 10;
                    break;

                case 1:
                    pb_ConvertTimeUnit = PCI3009_NET.cl_PCI3009.PCI3009_MILLI_SECOND;
                    dw_MinConvertTime = 1;
                    break;
            }

            Console.Write("\nSelect the convert time ({0} - 65535): ", dw_MinConvertTime);

            do
            {
                line = Console.ReadLine();
                if (!int.TryParse(line, out i_Tmp))
                    i_Tmp = 65536;
            } while ((i_Tmp <= 0) && (i_Tmp > 65535));

            pw_ConvertTime = (UInt16)i_Tmp;

            Console.Write("\nSelect delay mode: ");
            Console.Write("\n0) Not used");
            Console.Write("\n1) Mode 1");
            Console.Write("\n2) Mode 2");

            do
            {
                line = Console.ReadLine();
                if (!int.TryParse(line, out i_Tmp))
                    i_Tmp = 3;
            } while ((i_Tmp != 0) && (i_Tmp != 1) && (i_Tmp != 2));

            Console.Write(pc_DelayMode[i_Tmp]);

            switch (i_Tmp)
            {
                case 0:
                    pb_DelayMode = PCI3009_NET.cl_PCI3009.PCI3009_DISABLE;
                    break;

                case 1:
                    pb_DelayMode = PCI3009_NET.cl_PCI3009.PCI3009_DELAY_MODE_1;
                    break;

                case 2:
                    pb_DelayMode = PCI3009_NET.cl_PCI3009.PCI3009_DELAY_MODE_2;
                    break;
            }

            if (i_Tmp != 0)
            {
                Console.Write("\nSelect the delay time unit (0: us / 1: ms): ");

                do
                {
                    line = Console.ReadLine();
                    if (!int.TryParse(line, out i_Tmp))
                        i_Tmp = 2;
                } while ((i_Tmp != 0) && (i_Tmp != 1));

                Console.Write(pc_TimeUnit[i_Tmp]);

                switch (i_Tmp)
                {
                    case 0:
                        pb_DelayTimeUnit = PCI3009_NET.cl_PCI3009.PCI3009_MICRO_SECOND;
                        dw_MinConvertTime = 10;
                        break;

                    case 1:
                        pb_DelayTimeUnit = PCI3009_NET.cl_PCI3009.PCI3009_MILLI_SECOND;
                        dw_MinConvertTime = 1;
                        break;
                }

                Console.Write("\nSelect the delay time ({0} - 65535): ", dw_MinConvertTime);

                do
                {
                    line = Console.ReadLine();
                    if (!int.TryParse(line, out i_Tmp))
                        i_Tmp = 65536;
                } while ((i_Tmp <= 0) && (i_Tmp > 65535));

                pw_DelayTime = (UInt16)i_Tmp;
            }

            Console.Write("\nNumber of sequence: ");
            Console.Write("\n0: Continuous mode");
            Console.Write("\n>0: Number of sequnces");

            do
            {
                line = Console.ReadLine();
                if (!UInt32.TryParse(line, out l_Tmp))
                    l_Tmp = 0;
            } while ((l_Tmp <= 0) && (l_Tmp > 4294967295));

            pdw_NumberOfSequence = (UInt32)l_Tmp;

            /* Use always DMA */
            pb_UseDMA = PCI3009_NET.cl_PCI3009.PCI3009_ENABLE;

            Console.Write("\nNumber of sequence to do before generating an interrupt: ");

            do
            {
                line = Console.ReadLine();
                if (!UInt32.TryParse(line, out l_Tmp))
                    l_Tmp = 0;
            } while ((l_Tmp <= 0) && (l_Tmp > 4294967295));

            pdw_NumberOfSequenceForEachInterrupt = (UInt32)l_Tmp;


            Console.Write("\nTrigger action: ");
            Console.Write("\n0) Disable trigger");
            Console.Write("\n1) One shot trigger");
            Console.Write("\n2) One shot trigger");
            do
            {
                line = Console.ReadLine();
                if (!int.TryParse(line, out i_Tmp))
                    i_Tmp = 3;
            } while ((i_Tmp != 0) && (i_Tmp != 1) && (i_Tmp != 2));

            Console.Write(pc_HardwareTriggerAction[i_Tmp]);

            switch (i_Tmp)
            {
                case 0:
                    pb_HardwareTriggerAction = PCI3009_NET.cl_PCI3009.PCI3009_DISABLE;
                    break;

                case 1:
                    pb_HardwareTriggerAction = PCI3009_NET.cl_PCI3009.PCI3009_ONE_SHOT_TRIGGER;
                    break;

                case 2:
                    pb_HardwareTriggerAction = PCI3009_NET.cl_PCI3009.PCI3009_TRIGGER_START_A_SERIES;
                    break;
            }

            if (pb_HardwareTriggerAction != PCI3009_NET.cl_PCI3009.PCI3009_DISABLE)
            {
                Console.Write("\nTrigger level: ");
                Console.Write("\n0) Low");
                Console.Write("\n1) High");
                Console.Write("\n2) Low and high");
                do
                {
                    line = Console.ReadLine();
                    if (!int.TryParse(line, out i_Tmp))
                        i_Tmp = 3;
                } while ((i_Tmp != 0) && (i_Tmp != 1) && (i_Tmp != 2));

                Console.Write(pc_HardwareTriggerLevel[i_Tmp]);

                switch (i_Tmp)
                {
                    case 0:
                        pb_HardwareTriggerLevel = PCI3009_NET.cl_PCI3009.PCI3009_LOW_FRONT;
                        break;

                    case 1:
                        pb_HardwareTriggerLevel = PCI3009_NET.cl_PCI3009.PCI3009_HIGH_FRONT;
                        break;

                    case 2:
                        pb_HardwareTriggerLevel = PCI3009_NET.cl_PCI3009.PCI3009_LOW_HIGH_FRONT;
                        break;
                }

                Console.Write("Set the number of trigger signals to count before the action occur (1 - 65535): ");

                do
                {
                    line = Console.ReadLine();
                    if (!UInt32.TryParse(line, out l_Tmp))
                        l_Tmp = 0;
                } while ((l_Tmp <= 0) && (l_Tmp > 65535));

                pdw_HardwareTriggerCount = (UInt32)l_Tmp;

                Console.Write("Set the number of sequences cycles to trigger (1 - 65535): ");

                do
                {
                    line = Console.ReadLine();
                    if (!UInt32.TryParse(line, out l_Tmp))
                        l_Tmp = 0;
                } while ((l_Tmp <= 0) && (l_Tmp > 65535));

                pdw_HardwareTriggerCycleCount = (UInt32)l_Tmp;
            }
        }

        private static void v_ChannelListSelection(byte b_SingleDiffMode,
                                     ref byte pb_SequenceSize,
                                     byte[] pb_ChannelSequence)
        {
            byte b_Cpt = 0;
            UInt16 w_NumberOfChannels = (b_SingleDiffMode == PCI3009_NET.cl_PCI3009.PCI3009_DIFF) ? (UInt16)8 : (UInt16)16;
            UInt16 w_FirstChannel = 0;
            UInt16 w_LastChannel = (UInt16)(w_NumberOfChannels - 1);
            int i_Tmp = 0;
            string line;

            Console.Write("\nNumber of analog input channel(s) = {0}", w_NumberOfChannels);
            Console.Write("\nMax sequence size = {0}\n", w_NumberOfChannels);
            Console.Write("\nSequence size selection: ");

            do
            {
                line = Console.ReadLine();
                if (!int.TryParse(line, out i_Tmp))
                    i_Tmp = (w_NumberOfChannels + 1);

            } while ((i_Tmp == 0) || (i_Tmp > w_NumberOfChannels));


            pb_SequenceSize = (byte)i_Tmp;

            /**************************
            * Sequence initialisation 
            ***************************/
            Console.Write("\n");

            for (b_Cpt = 0; b_Cpt < pb_SequenceSize; b_Cpt++)
            {
                for (; ; )
                {
                    Console.Write("Sequence index {0} channel selection : ", b_Cpt);

                    line = Console.ReadLine();
                    if (!int.TryParse(line, out i_Tmp))
                        i_Tmp = (w_NumberOfChannels + 1);

                    if ((i_Tmp >= w_FirstChannel) && (i_Tmp <= w_LastChannel))
                    {
                        pb_ChannelSequence[b_Cpt] = (byte)i_Tmp;
                        break;
                    }
                }
            }
        }

        public static void v_GetChannelConfiguration(byte b_Channel,
                                                      ref byte pb_Polarity,
                                                      ref byte pb_Gain)
        {
            string[] pc_Gain = { "Gain 1", "Gain 2", "Gain 5", "Gain 10" };
            string[] pc_Polarity = { "Unipolar", "Bipolar" };
            int i_Tmp = 0;
            string line;

            Console.Write("\n\nChannel {0} configuration", b_Channel);

            Console.Write("\nPolarity selection (0: unipolar / 1: bipolar): ");

            do
            {
                line = Console.ReadLine();
                if (!int.TryParse(line, out i_Tmp))
                    i_Tmp = 2;

            } while ((i_Tmp != 0) && (i_Tmp != 1));

            Console.Write(pc_Polarity[i_Tmp]);

            switch (i_Tmp)
            {
                case 0:
                    pb_Polarity = PCI3009_NET.cl_PCI3009.PCI3009_UNIPOLAR;
                    break;

                case 1:
                    pb_Polarity = PCI3009_NET.cl_PCI3009.PCI3009_BIPOLAR;
                    break;
            }

            Console.Write("\nGain selection: ");
            Console.Write("\n0) Gain 1");
            Console.Write("\n1) Gain 2");
            Console.Write("\n2) Gain 5");
            Console.Write("\n3) Gain 10");
            do
            {
                line = Console.ReadLine();
                if (!int.TryParse(line, out i_Tmp))
                    i_Tmp = 4;
            } while ((i_Tmp != 0) && (i_Tmp != 1) && (i_Tmp != 2) && (i_Tmp != 3));

            Console.Write("\n{0}", pc_Gain[i_Tmp]);

            switch (i_Tmp)
            {
                case 0:
                    pb_Gain = PCI3009_NET.cl_PCI3009.PCI3009_1_GAIN;
                    break;

                case 1:
                    pb_Gain = PCI3009_NET.cl_PCI3009.PCI3009_2_GAIN;
                    break;

                case 2:
                    pb_Gain = PCI3009_NET.cl_PCI3009.PCI3009_5_GAIN;
                    break;

                case 3:
                    pb_Gain = PCI3009_NET.cl_PCI3009.PCI3009_10_GAIN;
                    break;
            }
        }

        static void Main(string[] args)
        {
            int i_ReturnValue = 0;
            byte b_OpenOk = 0;
            byte b_SetIRQOk = 0;
            byte b_NbrOfChannels = 16;
            byte[] pb_ChannelList = new byte[b_NbrOfChannels];
            long[] pdw_DigitalValue = new long[b_NbrOfChannels];
            string line;
            byte b_SingleDiffMode = 0;
            byte b_SequenceSize = 0;
            byte b_Polarity = 0;
            byte b_Gain = 0;
            byte b_ConvertTimeUnit = 0;
            UInt16 w_ConvertTime = 0;
            byte b_DelayMode = 0;
            byte b_DelayTimeUnit = 0;
            UInt16 w_DelayTime = 0;
            UInt32 dw_NumberOfSequence = 0;
            byte b_UseDMA = 0;
            UInt32 dw_NumberOfSequenceForEachInterrupt = 0;
            byte b_HardwareTriggerAction = 0;
            byte b_HardwareTriggerLevel = 0;
            UInt32 dw_HardwareTriggerCount = 0;
            UInt32 dw_HardwareTriggerCycleCount = 0;
            byte[] b_ChannelSequence = new byte[16];
            Program InterruptHandler = new Program(PCI3009);             /* Set the interrupt handler */

            Console.Write("\nRead analog inputs with sequence.");

            if (!Utils.Utils.openBoard(PCI3009))
            {
                Console.ReadKey();
                return;
            }

            if (i_ReturnValue == 0)
            {
                i_ReturnValue = PCI3009.i_SetBoardIntRoutineNew();

                switch (i_ReturnValue)
                {
                    case 0: Console.Write("\ni_SetBoardIntRoutineNew OK");
                        b_SetIRQOk = 1;
                        break;

                    case -1: Console.Write("\ni_SetBoardIntRoutineNew: The handle parameter of the board is wrong");
                        break;

                    case -2: Console.Write("\ni_SetBoardIntRoutineNew: Function call level error");
                        break;

                    case -3: Console.Write("\ni_SetBoardIntRoutineNew: Wrong calling mode");
                        break;

                    case -4: Console.Write("\ni_SetBoardIntRoutineNew: Interrupt function already installed for this board");
                        break;

                    case -5: Console.Write("\ni_SetBoardIntRoutineNew: Interrupt installation error");
                        break;

                    default: Console.Write("\ni_SetBoardIntRoutineNew: Error = {0}", i_ReturnValue);
                        break;
                }
            }

            if (i_ReturnValue == 0)
            {
                b_SingleDiffMode = 3;

                Console.Write("\nSelect the analog input mode (0: Single / 1: Differential): ");

                do
                {
                    line = Console.ReadLine();
                    if (!byte.TryParse(line, out b_SingleDiffMode))
                        b_SingleDiffMode = 3;

                } while ((b_SingleDiffMode != 0) && (b_SingleDiffMode != 1));

                i_ReturnValue = PCI3009.i_AnalogInputInitMode(b_SingleDiffMode);

                switch (i_ReturnValue)
                {
                    case 0: Console.Write("\ni_AnalogInputInitMode OK");
                        break;

                    case -1: Console.Write("\ni_AnalogInputInitMode: The handle parameter of the board is wrong");
                        break;

                    case -2: Console.Write("\ni_AnalogInputInitMode: Wrong single/diff mode");
                        break;

                    case -3: Console.Write("\ni_AnalogInputInitMode: A conversion is already started");
                        break;

                    default: Console.Write("\ni_AnalogInputInitChannel: Error = {0}", i_ReturnValue);
                        break;
                }
            }

            if (i_ReturnValue == 0)
            {
                UInt16 w_Cpt = 0;

                v_ChannelListSelection(b_SingleDiffMode, ref b_SequenceSize, b_ChannelSequence);

                for (w_Cpt = 0; (w_Cpt < b_SequenceSize) && (i_ReturnValue == 0); w_Cpt++)
                {
                    if (b_ChannelSequence[w_Cpt] != 255)
                    {
                        v_GetChannelConfiguration(b_ChannelSequence[w_Cpt], ref b_Polarity, ref b_Gain);

                        if (i_ReturnValue == 0)
                        {
                            i_ReturnValue = PCI3009.i_AnalogInputInitChannel(b_ChannelSequence[w_Cpt], b_Gain, b_Polarity);

                            switch (i_ReturnValue)
                            {
                                case 0:
                                    Console.Write("\ni_PCI3009_AnalogInputInitChannel OK");
                                    break;

                                case -1:
                                    Console.Write("\ni_PCI3009_AnalogInputInitChannel: The handle parameter of the board is wrong");
                                    break;

                                case -2:
                                    Console.Write("\ni_PCI3009_AnalogInputInitChannel: Wrong channel number");
                                    break;

                                case -3:
                                    Console.Write("\ni_PCI3009_AnalogInputInitChannel: Wrong gain");
                                    break;

                                case -4:
                                    Console.Write("\ni_PCI3009_AnalogInputInitChannel: Wrong polarity");
                                    break;

                                case -5:
                                    Console.Write("\ni_PCI3009_AnalogInputInitChannel: A conversion is already started");
                                    break;

                                default:
                                    Console.Write("\ni_PCI3009_AnalogInputInitChannel: error = {0}", i_ReturnValue);
                                    break;
                            }
                        }
                    }
                }
            }

            if (i_ReturnValue == 0)
            {
                v_GetAndSetUserSequenceConfiguration(ref b_ConvertTimeUnit,
                                                     ref w_ConvertTime,
                                                     ref b_DelayMode,
                                                     ref b_DelayTimeUnit,
                                                     ref w_DelayTime,
                                                     ref dw_NumberOfSequence,
                                                     ref b_UseDMA,
                                                     ref dw_NumberOfSequenceForEachInterrupt,
                                                     ref b_HardwareTriggerAction,
                                                     ref b_HardwareTriggerLevel,
                                                     ref dw_HardwareTriggerCount,
                                                     ref dw_HardwareTriggerCycleCount);

                i_ReturnValue = PCI3009.i_AnalogInputEnableDisableHardwareTrigger(b_HardwareTriggerAction,
                                                                                  b_HardwareTriggerLevel,
                                                                                  dw_HardwareTriggerCount,
                                                                                  dw_HardwareTriggerCycleCount);

                switch (i_ReturnValue)
                {
                    case 0:
                        Console.Write("\ni_PCI3009_AnalogInputEnableDisableHardwareTrigger OK");
                        break;

                    case -1:
                        Console.Write("\ni_PCI3009_AnalogInputEnableDisableHardwareTrigger: The handle parameter of the board is wrong");
                        break;

                    case -2:
                        Console.Write("\ni_PCI3009_AnalogInputEnableDisableHardwareTrigger: Wrong hardware trigger action selection");
                        break;

                    case -3:
                        Console.Write("\ni_PCI3009_AnalogInputEnableDisableHardwareTrigger: Wrong hardware trigger level selection");
                        break;

                    case -4:
                        Console.Write("\ni_PCI3009_AnalogInputEnableDisableHardwareTrigger: Wrong hardware trigger count value");
                        break;

                    case -5:
                        Console.Write("\ni_PCI3009_AnalogInputEnableDisableHardwareTrigger: Wrong trigger cycle count value");
                        break;

                    case -6:
                        Console.Write("\ni_PCI3009_AnalogInputEnableDisableHardwareTrigger: A conversion is already started");
                        break;

                    default:
                        Console.Write("\ni_PCI3009_AnalogInputEnableDisableHardwareTrigger: error = {0}", i_ReturnValue);
                        break;
                }
            }

            if (i_ReturnValue == 0)
            {
                /* For the interrupt routine */
                numberOfValuesPerInterrupt = (dw_NumberOfSequenceForEachInterrupt * b_SequenceSize);

                i_ReturnValue = PCI3009.i_AnalogInputInitSequence(b_SequenceSize,
                                                                  b_ChannelSequence,
                                                                  b_ConvertTimeUnit,
                                                                  w_ConvertTime,
                                                                  b_DelayMode,
                                                                  b_DelayTimeUnit,
                                                                  w_DelayTime,
                                                                  dw_NumberOfSequence,
                                                                  b_UseDMA,
                                                                  dw_NumberOfSequenceForEachInterrupt);

                switch (i_ReturnValue)
                {
                    case 0:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence OK");
                        break;

                    case -1:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: The handle parameter of the board is wrong");
                        break;

                    case -2:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Wrong channels number selection");
                        break;

                    case -3:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Wrong channel index 0 number");
                        break;

                    case -4:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Channel index 0 not initialised");
                        break;

                    case -5:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Wrong channel index 1 number");
                        break;

                    case -6:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Channel index 1 not initialised");
                        break;

                    case -7:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Wrong channel index 2 number");
                        break;

                    case -8:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Channel index 2 not initialised");
                        break;

                    case -9:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Wrong channel index 3 number");
                        break;

                    case -10:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Channel index 3 not initialised");
                        break;

                    case -11:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Wrong channel index 4 number");
                        break;

                    case -12:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Channel index 4 not initialised");
                        break;

                    case -13:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Wrong channel index 5 number");
                        break;

                    case -14:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Channel index 5 not initialised");
                        break;

                    case -15:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Wrong channel index 6 number");
                        break;

                    case -16:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Channel index 6 not initialised");
                        break;

                    case -17:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Wrong channel index 7 number");
                        break;

                    case -18:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Channel index 7 not initialised");
                        break;

                    case -19:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Wrong channel index 8 number");
                        break;

                    case -20:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Channel index 8 not initialised");
                        break;

                    case -21:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Wrong channel index 9 number");
                        break;

                    case -22:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Channel index 9 not initialised");
                        break;

                    case -23:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Wrong channel index 10 number");
                        break;

                    case -24:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Channel index 10 not initialised");
                        break;

                    case -25:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Wrong channel index 11 number");
                        break;

                    case -26:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Channel index 11 not initialised");
                        break;

                    case -27:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Wrong channel index 12 number");
                        break;

                    case -28:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Channel index 12 not initialised");
                        break;

                    case -29:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Wrong channel index 13 number");
                        break;

                    case -30:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Channel index 13 not initialised");
                        break;

                    case -31:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Wrong channel index 14 number");
                        break;

                    case -32:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Channel index 14 not initialised");
                        break;

                    case -33:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Wrong channel index 15 number");
                        break;

                    case -34:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Channel index 15 not initialised");
                        break;

                    case -35:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Wrong convert time unit");
                        break;

                    case -36:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Wrong convert time");
                        break;

                    case -37:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Wrong delay mode");
                        break;

                    case -38:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Wrong delay time unit");
                        break;

                    case -39:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Wrong delay time");
                        break;

                    case -40:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Wrong DMA mode");
                        break;

                    case -41:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Wrong number of sequence value for each interrupt");
                        break;

                    case -42:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Conversion is already started");
                        break;

                    case -43:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Scatter gather memory allocation error");
                        break;

                    case -44:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: Scatter gather list descriptor creation error");
                        break;

                    default:
                        Console.Write("\ni_PCI3009_AnalogInputInitSequence: error = {0}", i_ReturnValue);
                        break;
                }
            }

            if (i_ReturnValue == 0)
            {
                i_ReturnValue = PCI3009.i_AnalogInputStartSequence();

                switch (i_ReturnValue)
                {
                    case 0:
                        Console.Write("\ni_PCI3009_AnalogInputStartSequence OK");
                        break;

                    case -1:
                        Console.Write("\ni_PCI3009_AnalogInputStartSequence: The handle parameter of the board is wrong");
                        break;

                    case -2:
                        Console.Write("\ni_PCI3009_AnalogInputStartSequence: Sequence not initialised");
                        break;

                    case -3:
                        Console.Write("\ni_PCI3009_AnalogInputStartSequence: Interrupt routine not installed see function i_PCI3009_SetBoardIntRoutine");
                        break;

                    case -4:
                        Console.Write("\ni_PCI3009_AnalogInputStartSequence: A conversion is already started");
                        break;

                    case -5:
                        Console.Write("\ni_PCI3009_AnalogInputStartSequence: Description list setting error");
                        break;

                    case -6:
                        Console.Write("\ni_PCI3009_AnalogInputStartSequence: Enable DMA transfer error");
                        break;

                    case -7:
                        Console.Write("\ni_PCI3009_AnalogInputStartSequence: Start DMA transfer error");
                        break;

                    case -8:
                        Console.Write("\ni_PCI3009_AnalogInputStartSequence: Enable DMA transfer interrupt error");
                        break;

                    default:
                        Console.Write("\ni_PCI3009_AnalogInputStartSequence: error = {0}", i_ReturnValue);
                        break;
                }
            }

            if ((i_ReturnValue == 0) && (b_SetIRQOk == 1))
            {
                int countInterrupt = 0;
                TextWriter file = new StreamWriter(@"samples.txt", false);

                Console.Clear();
 
                /* Wait the interrupt */
                while (b_Stop == 0)
                {
                    if (interruptValues.Count == 0)
                    {
                        Console.SetCursorPosition(0, 0);
                        Console.Write("Waiting values");
                    }
                    else
                    {
                        file.WriteLine("\nInterrupt {0}", countInterrupt);

                        ushort[] value = (ushort[])interruptValues.Dequeue();
                        double[] volt = new double[value.Length];
                        ushort[] raw = new ushort[value.Length];
                        
                        value.CopyTo(raw, 0);

                        i_ReturnValue = PCI3009.i_AnalogInputConvertDMADigitalToRealAnalogValuesNew((uint)value.Length, raw, volt);
                        if (i_ReturnValue != 0)
                        {
                            Console.Write("\ni_AnalogInputConvertDMADigitalToRealAnalogValuesNew: error = {0}", i_ReturnValue);
                            break;
                        }

                        for (int i = 0; i < value.Length; i++)
                        {
                            if ((i % b_SequenceSize) == 0)
                            { 
                                Console.SetCursorPosition(0, 0);
                                Console.Write("\nInterrupt {0}", countInterrupt);
                            }

                            Console.Write("\nChannel[{0}] = {1:0.####}", b_ChannelSequence[i % b_SequenceSize], volt[i]);

                            file.WriteLine("Channel[{0}] = {1:0.####}", b_ChannelSequence[i % b_SequenceSize], volt[i]);
                        }

                        countInterrupt++;
                    }

                    if (Console.KeyAvailable)
                        break;
                }

                file.Close();
            }

            /* Check if stop was forced */
            switch (b_Stop)
            {
                case 1: Console.Write("\nDriver FIFO overflow");
                    break;

                case 2: Console.Write("\nPLD FIFO overflow");
                    break;

                case 3: Console.Write("\nReading data error");
                    break;
            }

            i_ReturnValue = PCI3009.i_AnalogInputStopSequence();

            switch (i_ReturnValue)
            {
                case 0:
                    Console.Write("\ni_PCI3009_AnalogInputStopSequence OK");
                    break;

                case -1:
                    Console.Write("\ni_PCI3009_AnalogInputStopSequence: The handle parameter of the board is wrong");
                    break;

                case -2:
                    Console.Write("\ni_PCI3009_AnalogInputStopSequence: Sequence not started");
                    break;

                case -3:
                    Console.Write("\ni_PCI3009_AnalogInputStopSequence: Disable the DMA transfer error");
                    break;

                case -4:
                    Console.Write("\ni_PCI3009_AnalogInputStopSequence: Abort the DMA transfer error");
                    break;

                case -5:
                    Console.Write("\ni_PCI3009_AnalogInputStopSequence: Stop the DMA transfer error");
                    break;

                case -6:
                    Console.Write("\ni_PCI3009_AnalogInputStopSequence: Reset descriptor list error");
                    break;

                default:
                    Console.Write("\ni_PCI3009_AnalogInputStopSequence: error = {0}", i_ReturnValue);
                    break;
            }

            i_ReturnValue = PCI3009.i_AnalogInputReleaseSequence();

            switch (i_ReturnValue)
            {
                case 0:
                    Console.Write("\ni_PCI3009_AnalogInputReleaseSequence OK");
                    break;

                case -1:
                    Console.Write("\ni_PCI3009_AnalogInputReleaseSequence: The handle parameter of the board is wrong");
                    break;

                case -2:
                    Console.Write("\ni_PCI3009_AnalogInputReleaseSequence: Sequence not initialised");
                    break;

                case -3:
                    Console.Write("\ni_PCI3009_AnalogInputReleaseSequence: Sequence acquisition started");
                    break;

                case -4:
                    Console.Write("\ni_PCI3009_AnalogInputReleaseSequence: Function call level error");
                    break;

                default:
                    Console.Write("\ni_PCI3009_AnalogInputReleaseSequence: error = {0}", i_ReturnValue);
                    break;
            }

            if (i_ReturnValue == 0)
            {

                i_ReturnValue = PCI3009.i_AnalogInputEnableDisableHardwareTrigger(PCI3009_NET.cl_PCI3009.PCI3009_DISABLE, b_HardwareTriggerLevel, dw_HardwareTriggerCount, dw_HardwareTriggerCycleCount);

                switch (i_ReturnValue)
                {
                    case 0:
                        Console.Write("\ni_PCI3009_AnalogInputEnableDisableHardwareTrigger OK");
                        break;

                    case -1:
                        Console.Write("\ni_PCI3009_AnalogInputEnableDisableHardwareTrigger: The handle parameter of the board is wrong");
                        break;

                    case -2:
                        Console.Write("\ni_PCI3009_AnalogInputEnableDisableHardwareTrigger: Wrong hardware trigger action selection");
                        break;

                    case -3:
                        Console.Write("\ni_PCI3009_AnalogInputEnableDisableHardwareTrigger: Wrong hardware trigger level selection");
                        break;

                    case -4:
                        Console.Write("\ni_PCI3009_AnalogInputEnableDisableHardwareTrigger: Wrong hardware trigger count value");
                        break;

                    case -5:
                        Console.Write("\ni_PCI3009_AnalogInputEnableDisableHardwareTrigger: Wrong trigger cycle count value");
                        break;

                    case -6:
                        Console.Write("\ni_PCI3009_AnalogInputEnableDisableHardwareTrigger: A conversion is already started");
                        break;

                    default:
                        Console.Write("\ni_PCI3009_AnalogInputEnableDisableHardwareTrigger: error = {0}", i_ReturnValue);
                        break;
                }
            }

            if (b_SetIRQOk == 1)
            {
                i_ReturnValue = PCI3009.i_ResetBoardIntRoutine();

                switch (i_ReturnValue)
                {
                    case 0: Console.Write("\ni_ResetBoardIntRoutine OK");
                        break;

                    case -1: Console.Write("\ni_ResetBoardIntRoutine: The handle parameter of the board is wrong");
                        break;

                    case -2: Console.Write("\ni_ResetBoardIntRoutine: Function call level error");
                        break;

                    case -3: Console.Write("\ni_ResetBoardIntRoutine: Interrupt function nor installed for this board");
                        break;

                    default: Console.Write("\ni_ResetBoardIntRoutine: Error = {0}", i_ReturnValue);
                        break;
                }
            }

            if (b_OpenOk == 1)
            {
                i_ReturnValue = PCI3009.i_CloseBoard();

                switch (i_ReturnValue)
                {
                    case 0: Console.Write("\ni_CloseBoard OK");
                        break;

                    case -1: Console.Write("\ni_CloseBoard: The handle parameter of the board is wrong");
                        break;

                    default: Console.Write("\ni_CloseBoard: Error = {0}", i_ReturnValue);
                        break;
                }
            }

            Console.Write("\nPress any key");
            Console.ReadKey();
        }
    }
}
