
using System;
using System.Collections.Generic;
using System.Text;
using PCI3009_NET;

/* 
 *  Initialise chronometer withe the selected mode
 *  - High level of signal C start the chronometer
 *  - Low level of signal C stop the chronometer 
 *  Read chronometer with interrupt
 */

namespace Chrono
{
    class Program
    {
        int i_InterruptCounter = 0;
        long dw_ChronoValue = 0;

        public Program(PCI3009_NET.cl_PCI3009 PCI3009)
        {
            PCI3009.e_EventHandlerDelegate += new PCI3009_NET.cl_PCI3009.v_EventHandlerDelegate(v_Interrupt);
            PCI3009.b_EventHandlerInitialised = 1;
            i_InterruptCounter = 0;
        }

        public void v_Interrupt(object sender, PCI3009_NET.cl_PCI3009.cl_EventHandlerArgs e)
        {
            dw_ChronoValue = e.dw_Value[0];
            i_InterruptCounter = i_InterruptCounter + 1;
        }

        static void initChrono(cl_PCI3009 PCI3009, ref byte chronoMode, ref byte timingUnit, ref long timingInterval, ref long realTimingInterval)
        {
            int returnValue = 0;
            Utils.Utils.getParameter("\nGet the chronometer mode(0 to 7)", ref chronoMode, (byte)0, (byte)7);
            Utils.Utils.getParameter("\nGet the timing unit:\n0: ns\n1: us\n2: ms\n3: s\n4: mn", ref timingUnit, (byte)0, (byte)4);

            String display = "\nGet the timing interval:" +
                             "\n+--------------+--------------------------+--------------------------+" +
                             "\n| b_TimingUnit | dw_TimingInterval (min.) | dw_TimingInterval (max.) |" +
                             "\n+--------------+--------------------------+--------------------------+" +
                             "\n| ns (0)       | 50                       | 4294967295               |" +
                             "\n+--------------+--------------------------+--------------------------+" +
                             "\n| us (1)       |  1                       |  107374182               |" +
                             "\n+--------------+--------------------------+--------------------------+" +
                             "\n| ms (2)       |  1                       |     107374               |" +
                             "\n+--------------+--------------------------+--------------------------+" +
                             "\n|  s (3)       |  1                       |        107               |" +
                             "\n+--------------+--------------------------+--------------------------+" +
                             "\n| mn (4)       |  1                       |          2               |" +
                             "\n+--------------+--------------------------+--------------------------+\n";
            Utils.Utils.getParameter(display, ref timingInterval, (long)0, (long)4294967295);

            returnValue = PCI3009.i_InitChrono(chronoMode, timingUnit, timingInterval, ref realTimingInterval);
            if (returnValue != 0)
                throw new Exception("\ni_InitChrono: Error = " + returnValue.ToString());
        }

        static void Main(string[] args)
        {
            int returnValue = 0;
            byte chronoMode = 0;
            byte timingUnit = 0;
            long timingInterval = 0;
            long realTimingInterval = 0;
            long hour = 0;
            byte minute = 0;
            byte second = 0;
            long milliSecond = 0;
            long microSecond = 0;
            long nanoSecond = 0;
            byte interruptEnable = 0;
            cl_PCI3009 PCI3009 = new cl_PCI3009();
            int currentInterrupt = 0;
            Program interruptHandler = null;
            long timeout = 0;
            byte chronoStatusTemp = 0;

            Console.Write("\n+---------------------------------------------------+");
            Console.Write("\n| APCI-3009 ChronoRead example:                     |");
            Console.Write("\n| Initialise chronometer                            |");
            Console.Write("\n| Select chronometer mode                           |");
            Console.Write("\n| Read chronometer value with/without interrupt     |");
            Console.Write("\n+---------------------------------------------------+");
            Console.Write("\n| Press any key for start the sample                |");
            Console.Write("\n+---------------------------------------------------+\n");
            Console.ReadKey();

            try
            {
				if (!Utils.Utils.openBoard(PCI3009))
				{
					Console.ReadKey();
					return;
				}

                Utils.Utils.getParameter("\nEnable interrupt?\n\t0 -> no\n\t1 -> yes\n", ref interruptEnable, (byte)0, (byte)1);
                if (interruptEnable != 0)
                {
                    interruptHandler = new Program(PCI3009);             /* Set the interrupt handler */
                    returnValue = PCI3009.i_SetBoardIntRoutine();
                    if (returnValue != 0)
                        throw new Exception("i_SetBoardIntRoutine error " + returnValue.ToString());

                    Console.Write("\ni_SetBoardIntRoutine OK");
                }
                else
                {
                    Utils.Utils.getParameter("Get timeout (0 to 65535)", ref timeout, (long)0, (long)65535);
                }

                initChrono(PCI3009, ref chronoMode, ref timingUnit, ref timingInterval, ref realTimingInterval);

                returnValue = PCI3009.i_EnableChrono(PCI3009_NET.cl_PCI3009.PCI3009_CONTINUOUS, interruptEnable);
                if (returnValue != 0)
                    throw new Exception("\ni_EnableChrono: Error = " + returnValue.ToString());

                for (; ; )
                {
                    byte chronoStatus = 0;

                    if (Console.KeyAvailable)
                        throw new Exception("\nKey pressed, stop the sample\n");

                    if ((interruptEnable != 0) && (interruptHandler.i_InterruptCounter > currentInterrupt))
                    {
                        returnValue = PCI3009.i_ConvertChronoValue(interruptHandler.dw_ChronoValue,
                                                                     ref hour,
                                                                     ref minute,
                                                                     ref second,
                                                                     ref milliSecond,
                                                                     ref microSecond,
                                                                     ref nanoSecond);

                        if (returnValue != 0)
                            throw new Exception("\ni_ConvertChronoValue: Error = " + returnValue.ToString());

                        Console.Write("\nChrono value {0}h {1}mn {2}s {3}ms {4}us {5}ns -- #{6}", hour, minute, second, milliSecond, microSecond, nanoSecond, interruptHandler.i_InterruptCounter);

                        /* Don't print this interrupt again */
                        currentInterrupt = interruptHandler.i_InterruptCounter;
                    }
                    else if (interruptEnable != 0)
                    {
                        returnValue = PCI3009.i_GetChronoProgressStatus(ref chronoStatus);
                        if (returnValue != 0)
                            throw new Exception("\ni_GetChronoProgressStatus: Error = " + returnValue.ToString());

                        if (chronoStatus == chronoStatusTemp)
                            continue;

                        if (chronoStatus == 0)
                            Console.Write("\nMeasurement not started.");
                        else if (chronoStatus == 1)
                            Console.Write("\nMeasurement started.");
                        else if (chronoStatus == 2)
                            Console.Write("\nMeasurement stopped.");
                        else if (chronoStatus == 3)
                            Console.Write("\nAn overflow occurred.");
                        else if (chronoStatus != 2)
                            Console.Write("\nb_ChronosStatus undefined");

                        chronoStatusTemp = chronoStatus;
                    }
                    else
                    {
                        long chronoValue = 0;
                        returnValue = PCI3009.i_ReadChronoValue(timeout, ref chronoStatus, ref chronoValue);
                        if (returnValue != 0)
                            throw new Exception("\ni_ReadChronoValue: Error = " + returnValue.ToString());

                        if (chronoStatus == 0)
                            Console.Write("\nMeasurement not started.");
                        else if (chronoStatus == 1)
                            Console.Write("\nMeasurement started.");
                        else if (chronoStatus == 3)
                            Console.Write("\nAn overflow occurred.");
                        else if (chronoStatus == 4)
                            Console.Write("\nA timeout occurred.");
                        else if (chronoStatus != 2)
                            Console.Write("\nb_ChronosStatus undefined");

                        if (chronoStatus != 2)
                            continue;

                        returnValue = PCI3009.i_ConvertChronoValue(chronoValue,
                                                                     ref hour,
                                                                     ref minute,
                                                                     ref second,
                                                                     ref milliSecond,
                                                                     ref microSecond,
                                                                     ref nanoSecond);

                        if (returnValue != 0)
                            throw new Exception("\ni_ConvertChronoValue: Error = " + returnValue.ToString());
                        
                        Console.Write("\nChrono value {0}h {1}mn {2}s {3}ms {4}us {5}ns", hour, minute, second, milliSecond, microSecond, nanoSecond);
                    }
                }
            }
            catch (Exception ex)
            {
                Console.Write(ex.Message);
            }

            returnValue = PCI3009.i_DisableChrono();
            if (returnValue != 0)
                Console.Write("\ni_DisableChrono: Error = {0}", returnValue);
            else
                Console.Write("\ni_DisableChrono OK");

            if (interruptEnable != 0)
            {
                returnValue = PCI3009.i_ResetBoardIntRoutine();
                if (returnValue != 0)
                    Console.Write("\ni_ResetBoardIntRoutine: Error = {0}", returnValue);
                else
                    Console.Write("\ni_ResetBoardIntRoutine OK");
            }
            returnValue = PCI3009.i_CloseBoard();
            if (returnValue != 0)
                Console.Write("\ni_CloseBoard: Error = {0}", returnValue);
            else
                Console.Write("\ni_CloseBoard OK");

            System.Threading.Thread.Sleep(2000);
        }
    }
}
