using System;
using System.Collections.Generic;
using System.Text;
using PCI3009_NET;

/* 
 *  Initialise chronometer withe the selected mode
 *  - High level of signal C start the chronometer
 *  - Low level of signal C stop the chronometer 
 *  Read chronometer with interrupt
 */

namespace Chrono
{
    class Program
    {
        static void initChrono(cl_PCI3009 PCI3009, ref byte chronoMode, ref byte timingUnit, ref long timingInterval, ref long realTimingInterval)
        {
            int returnValue = 0;
            Utils.Utils.getParameter("\nGet the chronometer mode(0 to 7)", ref chronoMode, (byte)0, (byte)7);
            Utils.Utils.getParameter("\nGet the timing unit:\n0: ns\n1: us\n2: ms\n3: s\n4: mn", ref timingUnit, (byte)0, (byte)4);

            String display = "\nGet the timing interval:" +
                             "\n+--------------+--------------------------+--------------------------+" +
                             "\n| b_TimingUnit | dw_TimingInterval (min.) | dw_TimingInterval (max.) |" +
                             "\n+--------------+--------------------------+--------------------------+" +
                             "\n| ns (0)       | 50                       | 4294967295               |" +
                             "\n+--------------+--------------------------+--------------------------+" +
                             "\n| us (1)       |  1                       |  107374182               |" +
                             "\n+--------------+--------------------------+--------------------------+" +
                             "\n| ms (2)       |  1                       |     107374               |" +
                             "\n+--------------+--------------------------+--------------------------+" +
                             "\n|  s (3)       |  1                       |        107               |" +
                             "\n+--------------+--------------------------+--------------------------+" +
                             "\n| mn (4)       |  1                       |          2               |" +
                             "\n+--------------+--------------------------+--------------------------+\n";
            Utils.Utils.getParameter(display, ref timingInterval, (long)0, (long)4294967295);

            returnValue = PCI3009.i_InitChrono(chronoMode, timingUnit, timingInterval, ref realTimingInterval);
            if (returnValue != 0)
                throw new Exception("\ni_InitChrono error = " + returnValue.ToString());
            Console.Write("\ni_InitChrono OK");
        }

        static void Main(string[] args)
        {
            int returnValue = 0;
            cl_PCI3009 PCI3009 = new cl_PCI3009();

            Console.Write("\n+---------------------------------------------------+");
            Console.Write("\n| APCI-3009 ReadChronoChannel example:              |");
            Console.Write("\n|     1/ Select channel or port to read             |");
            Console.Write("\n|     2/ Read the displayed result                  |");
            Console.Write("\n+---------------------------------------------------+\n");
            Console.ReadKey();

            try
            {
                byte chronoMode = 0;
                byte timingUnit = 0;
                long timingInterval = 0;
                long realTimingInterval = 0;

				if (!Utils.Utils.openBoard(PCI3009))
				{
					Console.ReadKey();
					return;
				}

                initChrono(PCI3009, ref chronoMode, ref timingUnit, ref timingInterval, ref realTimingInterval);

                for (; ; )
                {
                    byte channelPort = 0;
                    byte readStatus = 0;
                    Utils.Utils.getParameter("\nRead selection \n\t0 -> channel\n\t1 -> port\n\t2 -> Exit sample\n", ref channelPort, (byte)0, (byte)2);
                    if (channelPort == 2)
                        break;

                    if (channelPort != 0)
                    {
                        returnValue = PCI3009.i_ReadChronoPortValue(ref readStatus);
                        if (returnValue != 0)
                            throw new Exception("\ni_ReadChronoPortValue error : " + returnValue.ToString());
                        Console.WriteLine("\ni_ReadChronoPortValue OK port value : " + readStatus.ToString("X"));
                        
                    }
                    else
                    {
                        Utils.Utils.getParameter("\nEnter the channel to read channel\n\t0 -> channel E\n\t1 -> channel F\n\t2 -> channel G\n\t3 -> Cancel\n", ref channelPort, (byte)0, (byte)3);
                        if (channelPort == 3)
                            continue;

                        string channelDisplay = "E";
                        if (channelPort == 1)
                            channelDisplay = "F";
                        else if (channelPort == 2)
                            channelDisplay = "G";

                        returnValue = PCI3009.i_ReadChronoChlValue(channelPort, ref readStatus);
                        if (returnValue != 0)
                            throw new Exception("\ni_ReadChronoChlValue error : " + returnValue.ToString());
                        Console.WriteLine("\ni_ReadChronoChlValue OK channel " + channelDisplay.ToString() + " value : " + readStatus.ToString("X"));
                    }
                }
            }
            catch (Exception ex)
            {
                Console.Write(ex.Message);
            }

            returnValue = PCI3009.i_CloseBoard();
            if (returnValue != 0)
                Console.Write("\ni_CloseBoard: Error = {0}", returnValue);
            else
                Console.Write("\ni_CloseBoard OK");
        }
    }
}
