using System;
using System.Collections.Generic;
using System.Text;
using PCI3009_NET;

/* 
 *  Initialise chronometer withe the selected mode
 *  - High level of signal C start the chronometer
 *  - Low level of signal C stop the chronometer 
 *  Read chronometer with interrupt
 */

namespace Chrono
{
    class Program
    {
        static void initChrono(cl_PCI3009 PCI3009, ref byte chronoMode, ref byte timingUnit, ref long timingInterval, ref long realTimingInterval)
        {
            int returnValue = 0;
            Utils.Utils.getParameter("\nGet the chronometer mode(0 to 7)", ref chronoMode, (byte)0, (byte)7);
            Utils.Utils.getParameter("\nGet the timing unit:\n0: ns\n1: us\n2: ms\n3: s\n4: mn", ref timingUnit, (byte)0, (byte)4);

            String display = "\nGet the timing interval:" +
                             "\n+--------------+--------------------------+--------------------------+" +
                             "\n| b_TimingUnit | dw_TimingInterval (min.) | dw_TimingInterval (max.) |" +
                             "\n+--------------+--------------------------+--------------------------+" +
                             "\n| ns (0)       | 50                       | 4294967295               |" +
                             "\n+--------------+--------------------------+--------------------------+" +
                             "\n| us (1)       |  1                       |  107374182               |" +
                             "\n+--------------+--------------------------+--------------------------+" +
                             "\n| ms (2)       |  1                       |     107374               |" +
                             "\n+--------------+--------------------------+--------------------------+" +
                             "\n|  s (3)       |  1                       |        107               |" +
                             "\n+--------------+--------------------------+--------------------------+" +
                             "\n| mn (4)       |  1                       |          2               |" +
                             "\n+--------------+--------------------------+--------------------------+\n";
            Utils.Utils.getParameter(display, ref timingInterval, (long)0, (long)4294967295);

            returnValue = PCI3009.i_InitChrono(chronoMode, timingUnit, timingInterval, ref realTimingInterval);
            if (returnValue != 0)
                throw new Exception("\ni_InitChrono error = " + returnValue.ToString());
            Console.Write("\ni_InitChrono OK");
        }

        static void Main(string[] args)
        {
            int returnValue = 0;
            cl_PCI3009 PCI3009 = new cl_PCI3009();

            Console.Write("\n+---------------------------------------------------+");
            Console.Write("\n| APCI-3009 SetChronoChannel example:               |");
            Console.Write("\n| Select channel to set                             |");
            Console.Write("\n| Select channel state to set (On, Off)             |");
            Console.Write("\n+---------------------------------------------------+\n");
            Console.ReadKey();

            try
            {
                byte chronoMode = 0;
                byte timingUnit = 0;
                long timingInterval = 0;
                long realTimingInterval = 0;

				if (!Utils.Utils.openBoard(PCI3009))
				{
					Console.ReadKey();
					return;
				}
                
                initChrono(PCI3009, ref chronoMode, ref timingUnit, ref timingInterval, ref realTimingInterval);

                for (; ; )
                {
                    byte channel = 0;
                    string channelDisplay = "";
                    byte channelStatus = 0;
                    Utils.Utils.getParameter("\nEnter the channel to set:\n\t0 -> channel H\n\t1 -> channel A\n\t2 -> channel B\n\t3 -> Exit sample\n", ref channel, (byte)0, (byte)3);
                    if (channel == 3)
                        break;

                    Utils.Utils.getParameter("\nEnter the level of the channel\n\t0 -> set OFF\n\t1 -> set ON \n\t2 -> Cancel\n", ref channelStatus, (byte)0, (byte)2);
                    if (channelStatus == 2)
                        continue;

                    channelDisplay = "H";
                    if (channel == 1)
                        channelDisplay = "A";
                    else if (channel == 2)
                        channelDisplay = "B";

                    if (channelStatus != 0)
                    {

                        returnValue = PCI3009.i_SetChronoChlOn(channel);
                        if (returnValue != 0)
                            throw new Exception("\ni_SetChronoChlOn error : " + returnValue.ToString());
                        Console.WriteLine("\ni_SetChronoChlOn OK channel " + channelDisplay + " state : " + ((channelStatus != 0) ? "ON" : "OFF"));
                        
                    }
                    else
                    {
                        returnValue = PCI3009.i_SetChronoChlOff(channel);
                        if (returnValue != 0)
                            throw new Exception("\ni_SetDigitalChlOff error : " + returnValue.ToString());
                        Console.Write("\ni_SetDigitalChlOff OK channel " + channelDisplay + " state : " + ((channelStatus != 0) ? "ON" : "OFF"));
                    }
                }
            }
            catch (Exception ex)
            {
                Console.Write(ex.Message);
            }

            returnValue = PCI3009.i_CloseBoard();
            if (returnValue != 0)
                Console.Write("\ni_CloseBoard: Error = {0}", returnValue);
            else
                Console.Write("\ni_CloseBoard OK");
        }
    }
}
