using System;
using System.Collections.Generic;
using System.Text;
using PCI3009_NET;

namespace LatchInterrupt
{
    class Program
    {
        int i_InterruptCounter = 0;
        long dw_CounterAValue = 0;
        long dw_CounterBValue = 0;

        public Program(PCI3009_NET.cl_PCI3009 PCI3009)
        {
            PCI3009.e_EventHandlerDelegate += new PCI3009_NET.cl_PCI3009.v_EventHandlerDelegate(v_Interrupt);
            PCI3009.b_EventHandlerInitialised = 1;
            i_InterruptCounter = 0;
        }

        public void v_Interrupt(object sender, PCI3009_NET.cl_PCI3009.cl_EventHandlerArgs e)
        {
            /* Counter interrupt */
            if (e.dw_InterruptSrc == 1)
            {
                switch (e.dw_InterruptMask)
                {                    
                    /* Hardware latch of the 1. register (High level) */
                    case 0x1: dw_CounterAValue = e.dw_Value[0];
                        break;

                    /* Hardware latch of the 1. register (Low level) */
                    case 0x10001: dw_CounterAValue = e.dw_Value[0];
                        break;

                    /* Hardware latch of the 2. register (High level) */
                    case 0x2: dw_CounterBValue = e.dw_Value[0];
                        break;

                    /* Hardware latch of the 2. register (Low level) */
                    case 0x10002: dw_CounterBValue = e.dw_Value[0];
                        break;

                    /* Index interrupt (High level) */
                    case 0x4: 
                        break;

                    /* Index interrupt (Low level) */
                    case 0x10004: 
                        break;

                    /* Compare interrupt */
                    case 0x8: 
                        break;

                    /* Interrupt at the end of frequency measurement */
                    case 0x10: 
                        break;
                }
            }
          
            i_InterruptCounter = i_InterruptCounter + 1;
        }

        static void Main(string[] args)
        {
            int i_ReturnValue = 0;
            String ReadString;
            byte b_Stop = 0;
            byte b_ExternalStrobeALevel = 0;
            byte b_ExternalStrobeBLevel = 0;
            byte b_OpenOk = 0;
            byte b_SetIRQOk = 0;
            cl_PCI3009 PCI3009 = new cl_PCI3009();
            Program InterruptHandler = new Program(PCI3009);             /* Set the interrupt handler */

            Console.Write("\n+---------------------------------------------------+");
            Console.Write("\n| APCI-3009 incremental counter SAMPLE30:           |");
            Console.Write("\n| Initialise counter                                |");
            Console.Write("\n| 1*32-Bit counter / simple mode / hysteresis ON    |");
            Console.Write("\n| Read counter with interrupt                       |");
            Console.Write("\n+---------------------------------------------------+");
            Console.Write("\n| Press any key for start the sample                |");
            Console.Write("\n+---------------------------------------------------+");
            Console.Write("\n| Press any key to quit                             |");
            Console.Write("\n+---------------------------------------------------+");
            Console.Write("\n");
            Console.ReadKey();

            if (!Utils.Utils.openBoard(PCI3009))
            {
                Console.ReadKey();
                return;
            }

            if (i_ReturnValue == 0)
            {
                i_ReturnValue = PCI3009.i_InitCounter(PCI3009_NET.cl_PCI3009.PCI3009_32BIT_COUNTER,
                                                      PCI3009_NET.cl_PCI3009.PCI3009_SIMPLE_MODE,
                                                      PCI3009_NET.cl_PCI3009.PCI3009_HYSTERESIS_ON,
                                                      PCI3009_NET.cl_PCI3009.PCI3009_SIMPLE_MODE,
                                                      PCI3009_NET.cl_PCI3009.PCI3009_HYSTERESIS_ON);

                switch (i_ReturnValue)
                {
                    case 0: Console.Write("\ni_InitCounter OK");
                        break;

                    case -1: Console.Write("\ni_InitCounter: The handle parameter of the board is wrong");
                        break;

                    case -2: Console.Write("\ni_InitCounter: The module is not a counter module");
                        break;

                    case -3: Console.Write("\ni_InitCounter: The selected counter range is wrong");
                        break;

                    case -4: Console.Write("\ni_InitCounter: The selected first counter operating mode is wrong");
                        break;

                    case -5: Console.Write("\ni_InitCounter: The selected first counter operating option is wrong");
                        break;

                    case -6: Console.Write("\ni_InitCounter: The selected second counter operating mode is wrong");
                        break;

                    case -7: Console.Write("\ni_InitCounter: The selected second counter operating option is wrong");
                        break;

                    default: Console.Write("\ni_InitCounter: Error = {0}", i_ReturnValue);
                        break;
                }
            }

            if (i_ReturnValue == 0)
            {
                Console.Write("\nGet the external strobe A action level");
                Console.Write("\n   0 : Low edge latch");
                Console.Write("\n   1 : High edge latch\n");
                do
                {
                    ReadString = Console.ReadLine();
                    try
                    {
                        b_ExternalStrobeALevel = Convert.ToByte(ReadString);
                    }
                    catch (Exception) {}
                }
                while (!((b_ExternalStrobeALevel >= 0) && (b_ExternalStrobeALevel < 2)));

                i_ReturnValue = PCI3009.i_InitExternalStrobe(0, b_ExternalStrobeALevel);

                switch (i_ReturnValue)
                {
                    case 0: Console.Write("\ni_InitExternalStrobe OK");
                        break;

                    case -1: Console.Write("\ni_InitExternalStrobe: The handle parameter of the board is wrong");
                        break;

                    case -2: Console.Write("\ni_InitExternalStrobe: The module is not a counter module");
                        break;

                    case -3: Console.Write("\ni_InitExternalStrobe: Counter not initialised see function i_InitCounter");
                        break;

                    case -4: Console.Write("\ni_InitExternalStrobe: External strobe selection is wrong");
                        break;

                    case -5: Console.Write("\ni_InitExternalStrobe: External strobe level parameter is wrong");
                        break;

                    default: Console.Write("\ni_InitExternalStrobe: Error = {0}", i_ReturnValue);
                        break;
                }
            }

            if (i_ReturnValue == 0)
            {
                Console.Write("\nGet the external strobe B action level");
                Console.Write("\n   0 : Low edge latch");
                Console.Write("\n   1 : High edge latch\n");
                do
                {
                    ReadString = Console.ReadLine();
                    try
                    {
                        b_ExternalStrobeBLevel = Convert.ToByte(ReadString);
                    }
                    catch (Exception) {}
                }
                while (!((b_ExternalStrobeBLevel >= 0) && (b_ExternalStrobeBLevel < 2)));

                i_ReturnValue = PCI3009.i_InitExternalStrobe(1, b_ExternalStrobeBLevel);

                switch (i_ReturnValue)
                {
                    case 0: Console.Write("\ni_InitExternalStrobe OK");
                        break;

                    case -1: Console.Write("\ni_InitExternalStrobe: The handle parameter of the board is wrong");
                        break;

                    case -2: Console.Write("\ni_InitExternalStrobe: The module is not a counter module");
                        break;

                    case -3: Console.Write("\ni_InitExternalStrobe: Counter not initialised see function i_InitCounter");
                        break;

                    case -4: Console.Write("\ni_InitExternalStrobe: External strobe selection is wrong");
                        break;

                    case -5: Console.Write("\ni_InitExternalStrobe: External strobe level parameter is wrong");
                        break;

                    default: Console.Write("\ni_InitExternalStrobe: Error = {0}", i_ReturnValue);
                        break;
                }
            }

            if (i_ReturnValue == 0)
            {
                i_ReturnValue = PCI3009.i_SetBoardIntRoutine();

                switch (i_ReturnValue)
                {
                    case 0: Console.Write("\ni_SetBoardIntRoutine OK");
                        b_SetIRQOk = 1;
                        break;

                    case -1: Console.Write("\ni_SetBoardIntRoutine: The handle parameter of the board is wrong");
                        break;

                    case -2: Console.Write("\ni_SetBoardIntRoutine: Function call level error");
                        break;

                    case -3: Console.Write("\ni_SetBoardIntRoutine: Wrong calling mode");
                        break;

                    case -4: Console.Write("\ni_SetBoardIntRoutine: Interrupt function already installed for this board");
                        break;

                    case -5: Console.Write("\ni_SetBoardIntRoutine: Interrupt installation error");
                        break;

                    default: Console.Write("\ni_SetBoardIntRoutine: Error = {0}", i_ReturnValue);
                        break;
                }
            }

            if (i_ReturnValue == 0)
            {
                i_ReturnValue = PCI3009.i_EnableLatchInterrupt();

                switch (i_ReturnValue)
                {
                    case 0: Console.Write("\ni_EnableLatchInterrupt OK");
                        break;

                    case -1: Console.Write("\ni_EnableLatchInterrupt: The handle parameter of the board is wrong");
                        break;

                    case -2: Console.Write("\ni_EnableLatchInterrupt: The module is not a counter module");
                        break;

                    case -3: Console.Write("\ni_EnableLatchInterrupt: Counter not initialised see function i_InitCounter");
                        break;

                    case -4: Console.Write("\ni_EnableLatchInterrupt: Interrupt routine not installed see function i_SetBoardIntRoutine");
                        break;

                    default: Console.Write("\ni_EnableLatchInterrupt: Error = {0}", i_ReturnValue);
                        break;
                }
            }

            while ((i_ReturnValue == 0) && (b_Stop == 0))
            {
                if (Console.KeyAvailable)
                    b_Stop = 1;

                if (InterruptHandler.i_InterruptCounter > 0)
                {
                    Console.Write("\nCounter Latch A value = {0} Counter Latch B value = {1} interrupt counter = {2} ", InterruptHandler.dw_CounterAValue, InterruptHandler.dw_CounterBValue, InterruptHandler.i_InterruptCounter);
                }
            }

            if (i_ReturnValue == 0)
            {
                i_ReturnValue = PCI3009.i_DisableLatchInterrupt();

                switch (i_ReturnValue)
                {
                    case 0: Console.Write("\ni_DisableLatchInterrupt OK");
                        break;

                    case -1: Console.Write("\ni_DisableLatchInterrupt: The handle parameter of the board is wrong");
                        break;

                    case -2: Console.Write("\ni_DisableLatchInterrupt: The module is not a counter module");
                        break;

                    case -3: Console.Write("\ni_DisableLatchInterrupt: Counter not initialised see function i_InitCounter");
                        break;

                    default: Console.Write("\ni_DisableLatchInterrupt: Error = {0}", i_ReturnValue);
                        break;
                }
            }

            if (b_SetIRQOk == 1)
            {
                i_ReturnValue = PCI3009.i_ResetBoardIntRoutine();

                switch (i_ReturnValue)
                {
                    case 0: Console.Write("\ni_ResetBoardIntRoutine OK");
                        break;

                    case -1: Console.Write("\ni_ResetBoardIntRoutine: The handle parameter of the board is wrong");
                        break;

                    case -2: Console.Write("\ni_ResetBoardIntRoutine: Function call level error");
                        break;

                    case -3: Console.Write("\ni_ResetBoardIntRoutine: Interrupt function nor installed for this board");
                        break;

                    default: Console.Write("\ni_ResetBoardIntRoutine: Error = {0}", i_ReturnValue);
                        break;
                }
            }

            if (b_OpenOk == 1)
            {
                i_ReturnValue = PCI3009.i_CloseBoard();

                switch (i_ReturnValue)
                {
                    case 0: Console.Write("\ni_CloseBoard OK");
                        break;

                    case -1: Console.Write("\ni_CloseBoard: The handle parameter of the board is wrong");
                        break;

                    default: Console.Write("\ni_CloseBoard: Error = {0}", i_ReturnValue);
                        break;
                }
            }

            Console.WriteLine("\nPress any key");
            Console.ReadKey();
        }
    }
}

