using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using PCI3009_NET;

/* SSI. */

namespace Read1SSI
{
    class Program
    {
        static void Main(string[] args)
        {
            int i_ReturnValue = 0;
            byte b_OpenOk = 0;
            byte b_Stop = 0;
            byte b_SelectedSSI = 0;
            byte b_SSIProfile = 0;
		    byte b_PositionTurnLength = 0;
		    byte b_TurnCptLength = 0;
		    long dw_SSIOutputClock = 0;
            long dw_Position = 0;
            long dw_TurnCpt = 0;
		    byte b_SSICountingMode = 0;
            String ReadString;
            cl_PCI3009 PCI3009 = new cl_PCI3009();

            Console.Write("\nRead 1 SSI counter.");
            Console.Write("\nPress any key to start\n");
            Console.ReadKey();

            if (!Utils.Utils.openBoard(PCI3009))
            {
                Console.ReadKey();
                return;
            }

            if (i_ReturnValue == 0)
            {
                Console.Write("\n\nSet the SSI profile length (1 to 32): ");
                do
                {
                    ReadString = Console.ReadLine();
                    b_SSIProfile = Convert.ToByte(ReadString);
                }
                while (!((b_SSIProfile >= 1) && (b_SSIProfile <= 32)));

                Console.Write("\n\nSet the SSI position data length (1 to 31): ");
                do
                {
                    ReadString = Console.ReadLine();
                    b_PositionTurnLength = Convert.ToByte(ReadString);
                }
                while (!((b_PositionTurnLength >= 1) && (b_PositionTurnLength <= 31)));

                Console.Write("\n\nSet the SSI turn counter data length (0 to 31): ");
                do
                {
                    ReadString = Console.ReadLine();
                    b_TurnCptLength = Convert.ToByte(ReadString);
                }
                while (!((b_TurnCptLength >= 1) && (b_TurnCptLength <= 31)));

                Console.Write("\n\nSet the SSI output clock (252Hz to 5MHz): ");
                do
                {
                    ReadString = Console.ReadLine();
                    dw_SSIOutputClock = Convert.ToInt32(ReadString);
                }
                while (!((dw_SSIOutputClock >= 252) && (dw_SSIOutputClock <= 5000000)));

                Console.Write("\n\nSet the SSI counting mode (0: Binary / 1: Gray): ");
                do
                {
                    ReadString = Console.ReadLine();
                    b_SSICountingMode = Convert.ToByte(ReadString);
                }
                while (!((b_SSICountingMode >= 0) && (b_TurnCptLength <= 1)));

                i_ReturnValue = PCI3009.i_InitSSI(b_SSIProfile, b_PositionTurnLength, b_TurnCptLength, dw_SSIOutputClock, b_SSICountingMode); 	

                switch (i_ReturnValue)
                {
                    case 0: Console.Write("\ni_InitSSI OK");
                        break;

                    case -1: Console.Write("\ni_InitSSI: The handle parameter of the board is wrong");
                        break;

                    case -2: Console.Write("\ni_InitSSI: The module is not a SSI module");
                        break;

                    case -3: Console.Write("\ni_InitSSI: The selected SSI profile length is wrong");
                        break;

                    case -4: Console.Write("\ni_InitSSI: The selected SSI position data length is wrong");
                        break;

                    case -5: Console.Write("\ni_InitSSI: The selected SSI turn counter data length is wrong");
                        break;

                    case -6: Console.Write("\ni_InitSSI: The selected SSI output clock is wrong");
                        break;

                    case -7: Console.Write("\ni_InitSSI: The selected SSI counting mode parameter is wrong");
                        break;

                    default: Console.Write("\ni_InitSSI: Error = {0}", i_ReturnValue);
                        break;
                }
            }

            if (i_ReturnValue == 0)
            {
                Console.Write("\n\nSelect the counter to read (0 - 2): ");
                do
                {
                    ReadString = Console.ReadLine();
                    b_SelectedSSI = Convert.ToByte(ReadString);
                }
                while (!((b_SelectedSSI >= 0) && (b_SelectedSSI <= 2)));
            }

            while ((i_ReturnValue == 0) && (b_Stop == 0))
            {
                if (Console.KeyAvailable)
                    b_Stop = 1;

                if (i_ReturnValue == 0)
                {
                    i_ReturnValue = PCI3009.i_Read1SSIValue(b_SelectedSSI, ref dw_Position, ref dw_TurnCpt);

                    switch (i_ReturnValue)
                    {
                        case 0: Console.Write("\nPosition {0} Turn counter {1}", dw_Position, dw_TurnCpt);
                            break;

                        case -1: Console.Write("\ni_Read1SSIValue: The handle parameter of the board is wrong");
                            break;

                        case -2: Console.Write("\ni_Read1SSIValue: The module is not a SSI module");
                            break;

                        case -3: Console.Write("\ni_Read1SSIValue: SSI not initialised");
                            break;

                        case -4: Console.Write("\ni_Read1SSIValue: The selected SSI is wrong");
                            break;

                        case -5: Console.Write("\ni_Read1SSIValue: This function does not support more than 32 bits profile length");
                            break;

                        default: Console.Write("\ni_Read1SSIValue: Error = {0}", i_ReturnValue);
                            break;
                    }
                }
            }

            if (b_OpenOk == 1)
            {
                i_ReturnValue = PCI3009.i_CloseBoard();

                switch (i_ReturnValue)
                {
                    case 0: Console.Write("\ni_CloseBoard OK");
                        break;

                    case -1: Console.Write("\ni_CloseBoard: The handle parameter of the board is wrong");
                        break;

                    default: Console.Write("\ni_CloseBoard: Error = {0}", i_ReturnValue);
                        break;
                }
            }

            Console.WriteLine("\nPress any key");
            Console.ReadKey();
        }
    }
}
