using System;
using System.Collections.Generic;
using System.Text;
using PCI3009_NET;

/* Autorefresh on 16 channels. */

namespace Utils
{
    public class Utils
    {
        public static void getParameter<T>(String displayString, ref T parameter, T low, T high) where T : IComparable
        {
            do
            {
                Console.Write(displayString);
                String ReadString = Console.ReadLine();
                try
                {
                    parameter = (T)Convert.ChangeType(ReadString, typeof(T));
                }
                catch (Exception) { }
            }
            while (!((parameter.CompareTo(low) >= 0) && (parameter.CompareTo(high) <= 0)));
        }

        public static byte getChannelCount()
        {
            byte channelCount = 0;
            getParameter("\nEnter the number of channels to read:\n", ref channelCount, (byte)1, (byte)16);
            return channelCount;
        }

        public static byte getInterruptMode()
        {
            byte interrupt = 0;
            getParameter("\nDo you want to enable/disable interrupt:\n\t0 -> Disabled\n\t1 -> Enabled\n", ref interrupt, (byte)0, (byte)1);
            return interrupt;
        }

        public static byte getConvertTimeUnit()
        {
            byte convertTimeUnit = 0;
            getParameter("\nEnter the timing unit :\n\t" + PCI3009_NET.cl_PCI3009.PCI3009_MICRO_SECOND.ToString() + " -> us\n\t" + PCI3009_NET.cl_PCI3009.PCI3009_MILLI_SECOND + " -> ms\n", ref convertTimeUnit, (byte)PCI3009_NET.cl_PCI3009.PCI3009_MICRO_SECOND, (byte)PCI3009_NET.cl_PCI3009.PCI3009_MILLI_SECOND);
            return convertTimeUnit;
        }

        public static int getConvertTimeValue()
        {
            int convertTimeValue = 0;
            getParameter("\nEnter the convert time value: ", ref convertTimeValue, (int)1, (int)65535);
            return convertTimeValue;
        }

        public static byte getAnalogInputMode()
        {
            byte mode = 0;
            getParameter("\nEnter the analog input mode\n\t" + PCI3009_NET.cl_PCI3009.PCI3009_SINGLE.ToString() + " -> Single-Ended\n\t" + PCI3009_NET.cl_PCI3009.PCI3009_DIFF.ToString() + " -> Differential\n", ref mode, (byte)PCI3009_NET.cl_PCI3009.PCI3009_SINGLE, (byte)PCI3009_NET.cl_PCI3009.PCI3009_DIFF);
            return mode;
        }

        public static byte getAnalogInputGain(byte channel)
        {
            byte gain = 0;
            getParameter("\nEnter the gain for channel " + channel.ToString() + ":\n\t" +
                PCI3009_NET.cl_PCI3009.PCI3009_1_GAIN + " -> gain 1\n\t" +
                PCI3009_NET.cl_PCI3009.PCI3009_2_GAIN + " -> gain 2\n\t" +
                PCI3009_NET.cl_PCI3009.PCI3009_5_GAIN + " -> gain 5\n\t" +
                PCI3009_NET.cl_PCI3009.PCI3009_10_GAIN + " -> gain 10\n", ref gain, (byte)0, (byte)3);
            return gain;
        }

        public static byte getAnalogInputPolarity(byte channel)
        {
            byte polarity = 0;
            getParameter("\nEnter the analog input mode for channel " + channel.ToString() + "\n\t0 -> Bipolar\n\t1 -> Unipolar\n", ref polarity, (byte)0, (byte)1);
            return polarity;
        }

        static string getModule(UInt32 moduleConfiguration)
        {
            moduleConfiguration = ((moduleConfiguration >> 16) & 0xFFFF);
            if (moduleConfiguration == 0x5343)
                return "Incremental counter";
            else if (moduleConfiguration == 0x5349)
                return "SSI";
            else if (moduleConfiguration == 0x4449)
                return "Digital IO";
            else if (moduleConfiguration == 0x4348)
                return "Chronometer";
            else
                return "Unknown module";
        }

        static string getRevision(UInt32 revision)
        {
            string rev = "";
            for (int i = (System.Runtime.InteropServices.Marshal.SizeOf(revision) - 1); i >= 0; i--)
                rev += Convert.ToChar((revision >> (i * 8)) & 0xFF);
            return rev;
        }

        public static bool openBoard(cl_PCI3009 PCI3009)
        {
            try
            {
                byte numberOfBoards = 0;
				uint version = 0;

                int returnValue = PCI3009.i_GetDLLVersion(out version);
                if (returnValue != 0)
		            throw new Exception("i_PCI3009_GetDLLVersion error " + returnValue.ToString());

                Console.WriteLine("\nDLL version: \t\t" + ((version >> 16) & 0xFFFF).ToString("X4") + "-" + (version & 0xFFFF).ToString("X4"));

                returnValue = PCI3009.i_GetNumberOfBoards(out numberOfBoards);
                if (returnValue != 0)
                    throw new Exception("i_GetNumberOfBoards error " + returnValue.ToString());

                if (numberOfBoards == 0)
                    throw new Exception("No board found!");

                Console.WriteLine("\nNumber of boards = " + numberOfBoards.ToString());

                Byte b_BoardIndex = 0;
                UInt32 dw_IdentifierStringSize = 50;
                Byte[] pc_Identifier = new Byte[dw_IdentifierStringSize + 1];
                UInt32 pdw_UINumber = 0;
                UInt32 pdw_DeviceNumber = 0;
                UInt32 pdw_BusNumber = 0;
                UInt32 pdw_BaseAddress0 = 0;
                UInt32 pdw_BaseAddress1 = 0;
                UInt32 pdw_BaseAddress2 = 0;
                UInt32 pdw_BaseAddress3 = 0;
                Byte pb_Interrupt = 0;
                UInt32 pdw_ModuleConfiguration = 0;
                UInt32 pdw_Revision = 0;

                for (int i = 0; i < numberOfBoards; i++)
                {
                    returnValue = PCI3009.i_GetBoardInformation(b_BoardIndex,
                        dw_IdentifierStringSize,
                        pc_Identifier,
                        out pdw_UINumber,
                        out pdw_DeviceNumber,
                        out pdw_BusNumber,
                        out pdw_BaseAddress0,
                        out pdw_BaseAddress1,
                        out pdw_BaseAddress2,
                        out pdw_BaseAddress3,
                        out pb_Interrupt,
                        out pdw_ModuleConfiguration,
                        out pdw_Revision);
                    if (returnValue != 0)
                        throw new Exception("i_GetNumberOfBoards error " + returnValue.ToString());

                    /* Display informations about the board */
                    Console.WriteLine("\nBoard index:\t\t" + b_BoardIndex.ToString());
                    Console.WriteLine("Identifier:\t\t" + System.Text.Encoding.ASCII.GetString(pc_Identifier));
                    Console.WriteLine("Base address 0:\t\t0x" + pdw_BaseAddress0.ToString("X"));
                    Console.WriteLine("Base address 1:\t\t0x" + pdw_BaseAddress1.ToString("X"));
                    Console.WriteLine("Base address 2:\t\t0x" + pdw_BaseAddress2.ToString("X"));
                    Console.WriteLine("Base address 3:\t\t0x" + pdw_BaseAddress3.ToString("X"));
                    Console.WriteLine("UI Number:\t\t" + pdw_UINumber.ToString());
                    Console.WriteLine("Device Number:\t\t" + pdw_DeviceNumber);
                    Console.WriteLine("Bus Number:\t\t" + pdw_BusNumber.ToString());
                    Console.WriteLine("Interrupt line:\t\t" + pb_Interrupt);
                    Console.WriteLine("Module configuration:\t" + getModule(pdw_ModuleConfiguration) + " version : " + Convert.ToChar((pdw_ModuleConfiguration >> 8) & 0xFF) + Convert.ToChar(pdw_ModuleConfiguration & 0xFF));
                    Console.WriteLine("Revision :\t\t" + getRevision(pdw_Revision));

                    byte openMode = 0;
                    getParameter("Do you want to open board via Index (0) or via Identifier (1)?\n", ref openMode, (byte)0, (byte)1);
                    if (openMode == 0)
                    {
                        Byte index = 0;
                        getParameter("Enter the index of the board (0" + ((numberOfBoards > 1) ? " to " + (numberOfBoards - 1).ToString() : "") + ")\n", ref index, (byte)0, (byte)(numberOfBoards - 1));
                        returnValue = PCI3009.i_OpenBoardViaIndex(index);
                        if (returnValue != 0)
                            throw new Exception("i_OpenBoardViaIndex error " + returnValue.ToString());
                    }
                    else
                    {
                        Console.WriteLine("Enter the identifier of the board:\n");
                        String identifier = Console.ReadLine();
                        returnValue = PCI3009.i_OpenBoardViaIdentifer(identifier);
                        if (returnValue != 0)
                            throw new Exception("i_OpenBoardViaIdentifer error " + returnValue.ToString());
                    }

                    returnValue = PCI3009.i_GetDeviceDriverVersion(out version);
                    if (returnValue != 0)
                        throw new Exception("i_PCI3009_GetDeviceDriverVersion error " + returnValue.ToString());

                    Console.WriteLine("\nDevice driver version: \t" + ((version >> 16) & 0xFFFF).ToString("X4") + "-" + (version & 0xFFFF).ToString("X4"));

                }
            }
            catch (Exception ex)
            {
                Console.Write(ex.Message);
                return false;
            }
            return true;
        }
    }
}
